package de.mannheim.uni.expansion.tablefusion.adapters.columnmatches;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
import org.apache.commons.io.IOUtils;

import au.com.bytecode.opencsv.CSVReader;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.CellIdx;

public class CellMatchesReader {

	public ColumnMatchesRepository read(String columnMappingsFile) {
		Map<AnnotatedTableId, Map<CellIdx, InstanceId>> matches = new HashMap<>();

		try {

			TarArchiveInputStream tarStream = new TarArchiveInputStream(new GzipCompressorInputStream(new FileInputStream(columnMappingsFile)));

			TarArchiveEntry entry = tarStream.getNextTarEntry();

			while ((entry = tarStream.getNextTarEntry()) != null) {

				byte[] content = new byte[(int) entry.getSize()];

				tarStream.read(content, 0, content.length);

				InputStreamReader is1 = new InputStreamReader(new ByteArrayInputStream(content));

				CSVReader reader = new CSVReader(is1, ';', '"');

				Map<CellIdx, InstanceId> columnMatches = new HashMap<CellIdx, InstanceId>();

				for (String[] line = reader.readNext(); line != null; line = reader.readNext()) {

					CellIdx valueId = new CellIdx(Integer.parseInt(line[0]), Integer.parseInt(line[1]));
					InstanceId instanceId = new InstanceId(line[2]);
					columnMatches.put(valueId, instanceId);

				}

				reader.close();

				String name = entry.getName();
				name = name.substring(0, name.length() - 4);
				AnnotatedTableId id = new AnnotatedTableId(name);

				matches.put(id, columnMatches);

			}

			tarStream.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {

		}

		return new ColumnMatchesRepositoryImpl(matches);
	}

	public Map<CellIdx, InstanceId> readSingleTable(String columnMappingsFile) {

		try {

			byte[] content = IOUtils.toByteArray(new FileReader(columnMappingsFile));

			InputStreamReader is1 = new InputStreamReader(new ByteArrayInputStream(content));

			CSVReader reader = new CSVReader(is1, ';', '"');

			Map<CellIdx, InstanceId> columnMatches = new HashMap<CellIdx, InstanceId>();

			for (String[] line = reader.readNext(); line != null; line = reader.readNext()) {

				CellIdx valueId = new CellIdx(Integer.parseInt(line[0]), Integer.parseInt(line[1]));
				InstanceId instanceId = new InstanceId(line[2]);
				columnMatches.put(valueId, instanceId);

			}

			reader.close();

			return columnMatches;

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} finally {
		}
	}
}
