package de.mannheim.uni.expansion.tablefusion.core.matching.impl.reference;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.kbtables.RepositoryCandidateMap;
import de.mannheim.uni.expansion.data.dbpedia.kbtables.RepositoryColSet;
import de.mannheim.uni.expansion.data.dbpedia.kbtables.RepositoryRowSet;
import de.mannheim.uni.expansion.data.dbpedia.kbtables.RepositoryTableList;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.matching.dbpedia.model.MatchingData;
import de.mannheim.uni.matching.dbpedia.model.TableRow;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;

public class DBPediaMatchingDataBuilder {

	DBpediaRepositoryNewInterface dBpediaRepository;

	@Inject
	public DBPediaMatchingDataBuilder(DBpediaRepositoryNewInterface dBpediaRepository) {
		this.dBpediaRepository = dBpediaRepository;
	}

	public MatchingData getMatchingData() {

		Map<String, List<TableRow>> candidateMap = new RepositoryCandidateMap(dBpediaRepository);
		Collection<TableRow> dbpediaRowSet = new RepositoryRowSet(dBpediaRepository);
		Collection<TableColumn> dbpediaColSet = new RepositoryColSet(dBpediaRepository);
		Collection<Table> dbpediaTables = new RepositoryTableList(dBpediaRepository);

		MatchingData d = new MatchingData();
		d.setCandidateMap(candidateMap);
		d.setDbpediaColSet(dbpediaColSet);
		d.setDbpediaPropUris(new HashSet<String>());
		d.setDbpediaRowSet(dbpediaRowSet);
		d.setDbpediaTables(dbpediaTables);
		return d;

	}

	public MatchingData getMatchingData(Set<ClassId> allowedClasses) {

		Set<ClassId> checkedAllowedClasses = new HashSet<ClassId>();

		for (Iterator<ClassId> it = allowedClasses.iterator(); it.hasNext();) {
			ClassId next = it.next();
			if (dBpediaRepository.hasClass(next)) {
				checkedAllowedClasses.add(next);
			}
		}

		Map<String, List<TableRow>> candidateMap = new RepositoryCandidateMap(dBpediaRepository, checkedAllowedClasses);
		Collection<TableRow> dbpediaRowSet = new RepositoryRowSet(dBpediaRepository, checkedAllowedClasses);
		Collection<TableColumn> dbpediaColSet = new RepositoryColSet(dBpediaRepository, checkedAllowedClasses);
		Collection<Table> dbpediaTables = new RepositoryTableList(dBpediaRepository, checkedAllowedClasses);

		MatchingData d = new MatchingData();
		d.setCandidateMap(candidateMap);
		d.setDbpediaColSet(dbpediaColSet);
		d.setDbpediaPropUris(new HashSet<String>());
		d.setDbpediaRowSet(dbpediaRowSet);
		d.setDbpediaTables(dbpediaTables);
		return d;
	}
}
