package de.mannheim.uni.expansion.tablefusion.core.matching.impl.reference;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassOntology;
import de.mannheim.uni.expansion.data.ontology.model.properties.ObjectPropertyOntology;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.matching.dbpedia.model.MatchingData;
import de.mannheim.uni.matching.dbpedia.model.TableRow;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableToRowHierarchyAdapter;
import de.mannheim.uni.tableprocessor.model.Table;

public class MatchingDataBuilder {

	private final DBPediaMatchingDataBuilder dbPediaData;
	private final OntologyRepository ontologyRepository;

	@Inject
	public MatchingDataBuilder(DBPediaMatchingDataBuilder dbPediaData, OntologyRepository ontologyRepository) {
		this.dbPediaData = dbPediaData;
		this.ontologyRepository = ontologyRepository;

	}

	// public MatchingData get(Table webTable) {
	//
	// MatchingData data = dbPediaData.getMatchingData();
	//
	// TableToRowHierarchyAdapter rowAdapter = new TableToRowHierarchyAdapter();
	// Collection<TableRow> webtableRowSet = rowAdapter.getParts(webTable);
	// data.setWebtableRowSet(webtableRowSet);
	// Collection<Table> webTables = new ArrayList<Table>();
	// data.setWebTables(webTables);
	// webTables.add(webTable);
	// data.setWebtable(webTable);
	//
	// return data;
	// }

	public MatchingData get(Table webTable, PropertyId propertyId, int newKey) {

		ObjectPropertyOntology opo = (ObjectPropertyOntology) ontologyRepository.get(propertyId);

		ClassId range = opo.getRange();
		MatchingData data;

		if (range != null) {
			// System.out.println(propertyId.toString() + "::" +
			// range.toString());
			Set<ClassId> allowedClasses = getAllSubClases(range);

			allowedClasses.addAll(getAllParentClases(range));

			data = dbPediaData.getMatchingData(allowedClasses);

		} else {
			data = dbPediaData.getMatchingData();
		}
		TableToRowHierarchyAdapter rowAdapter = new TableToRowHierarchyAdapter();
		Collection<TableRow> webtableRowSet = rowAdapter.getParts(webTable);
		data.setWebtableRowSet(webtableRowSet);
		Collection<Table> webTables = new ArrayList<Table>();
		data.setWebTables(webTables);
		webTables.add(webTable);
		data.setWebtable(webTable);

		return data;
	}

	Set<ClassId> getAllSubClases(ClassId classId) {

		Set<ClassId> classes = new HashSet<ClassId>();
		classes.add(classId);

		Set<ClassId> subClasses = ontologyRepository.getSubClasses(classId);
		if (subClasses != null) {
			for (ClassId subClassId : subClasses) {
				classes.addAll(getAllSubClases(subClassId));
			}
		}

		return classes;
	}

	Set<ClassId> getAllParentClases(ClassId classId) {

		Set<ClassId> classes = new HashSet<ClassId>();
		classes.add(classId);

		ClassOntology ontology = ontologyRepository.get(classId);
		if (ontology != null) {
			Set<ClassId> parentClasses = ontology.getSubClassOf();
			if (parentClasses != null) {
				for (ClassId subClassId : parentClasses) {
					classes.addAll(getAllParentClases(subClassId));
				}
			}
		}
		return classes;
	}

}
