package de.mannheim.uni.expansion.data.dbpedia.adapter.raw;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections4.BidiMap;
import org.apache.commons.collections4.bidimap.DualHashBidiMap;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.io.Input;
import com.esotericsoftware.kryo.io.Output;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNew;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.matching.dbpedia.Preprocessing;
import de.mannheim.uni.tableprocessor.IO.TableReader;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;
import de.mannheim.uni.utils.concurrent.Consumer;
import de.mannheim.uni.utils.concurrent.Parallel;

public class CreateDBPediaRepository {
	private final String dbPediaPath;
	private final boolean useUnits;
	private final FileEnding fileEnding;

	public CreateDBPediaRepository(String dbPediaPath, FileEnding fileEnding, boolean useUnits) {
		this.dbPediaPath = dbPediaPath;
		this.useUnits = useUnits;
		this.fileEnding = fileEnding;
	}

	// public static void main(String[] args) {
	//
	// String dataHome = "C:/Users/yaser/dev/data";
	//
	// if (args.length > 0 && args[0] != null && !args[0].equals("")) {
	// dataHome = args[0];
	// }
	//
	// String dbpediaFile = dataHome + "/serialised_kryo_subset.bin";
	// if (args.length > 1 && args[1] != null && args[1].equals("full")) {
	// dbpediaFile = dataHome + "/serialised_kryo.bin";
	// System.out.println("Reading Full Dataset");
	// }
	//
	// // String uriIndexPath = "C:/Users/yaser/dev/data/index";
	//
	// boolean useUnits = true;
	//
	// MatchingParameters mp = new MatchingParameters();
	// mp.setUseUnitDetection(useUnits);
	// mp.setFastJoinDelta(1); // = edit distance threshold
	// mp.setFastJoinTau(1); // = jaccard threshold
	//
	// MatchingData md = new MatchingData();
	// md.loadDBpedia(dbpediaFile, mp);
	//
	// bucket.upsert(SerializableDocument.create("123", new Date()));
	// bucket.upsert(SerializableDocument.create("124", "fsadfsadf"));
	//
	// CreateDBPediaRepository createDBPediaRepository = new
	// CreateDBPediaRepository();
	// createDBPediaRepository.run(bucket, md);
	//
	// bucket.close();
	// }

	public DBpediaRepositoryNew run() {

		System.out.println("Loading DBpedia ... ");
		System.err.println("Loading DBpedia ... ");
		DBpediaRepositoryNew repository;

		// load dbpedia
		File directory = new File(dbPediaPath);

		if (directory.isDirectory()) {
			File f2 = new File(dbPediaPath + ".bin");
			if (f2.exists() && !f2.isDirectory()) {
				directory = f2;
			}
		}

		if (directory.isDirectory()) {

			DBPediaLoadCache cache = new DBPediaLoadCache();
			final Map<ClassId, Integer> tableIndex = new HashMap<ClassId, Integer>();

			final Map<ClassId, BidiMap<PropertyId, Integer>> propertyIndices = new HashMap<ClassId, org.apache.commons.collections4.BidiMap<PropertyId, Integer>>();

			final Map<ClassId, BidiMap<InstanceId, Integer>> instanceIndices = new HashMap<ClassId, BidiMap<InstanceId, Integer>>();

			final Set<TripleId> valueExists = Collections.newSetFromMap(new ConcurrentHashMap<TripleId, Boolean>());

			final Map<ClassPropertyId, ColumnDataType> columnDataTypes = new HashMap<ClassPropertyId, ColumnDataType>();
			final Map<ClassId, PropertyId> classKeys = new HashMap<ClassId, PropertyId>();

			TableReader r = new TableReader();
			r.setUseUnitDetection(useUnits);

			int tCounter = 0;
			for (File f : directory.listFiles()) {
				System.out.println("loading " + f.getAbsolutePath());
				Table t;
				try {
					t = r.readLODTable(f.getPath());
					Preprocessing p = new Preprocessing();
					p.handleIncorrectDates(t);

					ClassId dbPediaClass = cache.getFromCache(ClassId.fromString(t.getHeader(), fileEnding));

					tableIndex.put(dbPediaClass, tCounter);

					BidiMap<PropertyId, Integer> columns = new DualHashBidiMap<PropertyId, Integer>();
					int c = 0;
					for (TableColumn column : t.getColumns()) {

						PropertyId dbPediaProperty = cache.getFromCache(new PropertyId(column.getURI()));

						columns.put(dbPediaProperty, c);

						c++;
					}

					propertyIndices.put(dbPediaClass, columns);

					if (t.getKey() != null) {
						BidiMap<InstanceId, Integer> instances = new DualHashBidiMap<InstanceId, Integer>();

						int instanceCounter = 0;
						for (Entry<Integer, Object> instance : t.getKey().getValues().entrySet()) {
							InstanceId dbPediaInstance = cache.getFromCache(InstanceId.fromString(instance.getValue().toString()));
							instances.put(dbPediaInstance, instance.getKey());

							final int tCounterFinal = tCounter;
							final int instanceCounterFinal = instanceCounter;
							Parallel.forLoop(0, t.getColumns().size(), new Consumer<Integer>() {

								@Override
								public void execute(Integer cCounter) {
									TableColumn column = t.getColumns().get(cCounter);

									if (column.getHeader().endsWith("_label")) {
										return;
									}

									PropertyId propertyId = cache.getFromCache(new PropertyId(column.getURI()));
									String fullKey = "DP_" + tCounterFinal + "_" + instanceCounterFinal + "_" + cCounter;

									if (column.getDataType() == ColumnDataType.list) {
										column.setDataType(ColumnDataType.string);
									}

									columnDataTypes.put(new ClassPropertyId(dbPediaClass, propertyId), column.getDataType());

									if (column.isKey()) {
										classKeys.put(dbPediaClass, propertyId);
									}

								}
							}

							);

							instanceCounter++;

						}

						instanceIndices.put(dbPediaClass, instances);

					} else {
						System.out.println("WARNING: Table without key");
					}

					tCounter++;

				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			System.out.println("Loading DBpedia ... done");
			System.err.println("Loading DBpedia ... done");
			repository = new DBpediaRepositoryNew(tableIndex, propertyIndices, instanceIndices, columnDataTypes, classKeys, null, null, null, null);

			Kryo kryo = new Kryo();

			Output output;
			try {
				output = new Output(new FileOutputStream(dbPediaPath + ".bin"));

				kryo.writeObject(output, repository);
				output.close();

				System.out.println("done.");
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		} else {

			try {
				Kryo kryo = new Kryo();

				Input input = new Input(new FileInputStream(dbPediaPath));
				repository = kryo.readObject(input, DBpediaRepositoryNew.class);
				input.close();

				System.out.println("Loading DBpedia ... done");
				System.err.println("Loading DBpedia ... done");
				return repository;
			} catch (Exception e) {
				throw new RuntimeException(e);
			}

		}

		return repository;

	}

	private static class DBPediaLoadCache {
		Map<ClassId, ClassId> classCache = new HashMap<ClassId, ClassId>();
		Map<InstanceId, InstanceId> instanceCache = new HashMap<InstanceId, InstanceId>();
		Map<PropertyId, PropertyId> propertyCache = new HashMap<PropertyId, PropertyId>();

		public ClassId getFromCache(ClassId dBpediaClass) {
			ClassId cached = classCache.get(dBpediaClass);
			if (cached == null) {
				classCache.put(dBpediaClass, dBpediaClass);
				return dBpediaClass;
			} else {
				return cached;
			}
		}

		public PropertyId getFromCache(PropertyId dBpediaClass) {
			PropertyId cached = propertyCache.get(dBpediaClass);
			if (cached == null) {
				propertyCache.put(dBpediaClass, dBpediaClass);
				return dBpediaClass;
			} else {
				return cached;
			}
		}

		public InstanceId getFromCache(InstanceId dBpediaClass) {
			InstanceId cached = instanceCache.get(dBpediaClass);
			if (cached == null) {
				instanceCache.put(dBpediaClass, dBpediaClass);
				return dBpediaClass;
			} else {
				return cached;
			}
		}

	}

}
