package de.mannheim.uni.expansion.data.dbpedia.kbtables;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.tableprocessor.model.TableColumn;

public class RepositoryColSet implements Collection<TableColumn> {

	private final DBpediaRepositoryNewInterface dbPediaRepository;
	private final int size;

	private final Set<ClassId> allowedClasses;

	public RepositoryColSet(DBpediaRepositoryNewInterface dbPediaRepository) {
		this.dbPediaRepository = dbPediaRepository;
		size = dbPediaRepository.getTotalNumberOfColumns();
		this.allowedClasses = null;
	}

	public RepositoryColSet(DBpediaRepositoryNewInterface dbPediaRepository, Set<ClassId> allowedClasses) {
		this.dbPediaRepository = dbPediaRepository;

		int size = 0;
		for (ClassId classId : allowedClasses) {
			size += dbPediaRepository.getNumberOfColumns(classId);
		}

		this.size = size;
		this.allowedClasses = new HashSet<ClassId>(allowedClasses);
	}

	@Override
	public boolean add(TableColumn arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean addAll(Collection<? extends TableColumn> arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void clear() {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean contains(Object arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean containsAll(Collection<?> arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean isEmpty() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Iterator<TableColumn> iterator() {
		final Iterator<ClassId> classes;

		if (allowedClasses != null)
			classes = allowedClasses.iterator();
		else
			classes = dbPediaRepository.getClasses().iterator();

		return new Iterator<TableColumn>() {

			private ClassId currentTable;
			private Iterator<PropertyId> instancesPerClass;

			@Override
			public TableColumn next() {
				if ((currentTable == null || !instancesPerClass.hasNext()) && classes.hasNext()) {

					currentTable = classes.next();
					instancesPerClass = dbPediaRepository.getProperties(currentTable).keySet().iterator();

					if (instancesPerClass.hasNext()) {
						return new RepositoryTableColumn(dbPediaRepository, currentTable, instancesPerClass.next());
					}

				} else if (currentTable != null && instancesPerClass.hasNext()) {
					return new RepositoryTableColumn(dbPediaRepository, currentTable, instancesPerClass.next());
				}

				return null;
			}

			@Override
			public boolean hasNext() {
				if ((currentTable == null || !instancesPerClass.hasNext()) && classes.hasNext()) {

					currentTable = classes.next();

					instancesPerClass = dbPediaRepository.getProperties(currentTable).keySet().iterator();

					if (instancesPerClass.hasNext()) {
						return true;
					}

				} else if (currentTable != null && instancesPerClass.hasNext()) {
					return true;
				}

				return false;
			}
		};
	}

	@Override
	public boolean remove(Object arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean removeAll(Collection<?> arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean retainAll(Collection<?> arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	public int size() {
		return size;
	}

	@Override
	public Object[] toArray() {
		throw new UnsupportedOperationException();
	}

	@Override
	public <T> T[] toArray(T[] arg0) {
		throw new UnsupportedOperationException();
	}

}
