package de.mannheim.uni.expansion.data.dbpedia.kbtables;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;

public class RepositoryTableColumnValues implements Map<Integer, Object> {

	private final DBpediaRepositoryNewInterface repository;
	private final ClassId classId;
	private final PropertyId propertyId;
	private final boolean isKey;

	public RepositoryTableColumnValues(DBpediaRepositoryNewInterface repository, ClassId classId, PropertyId propertyId) {
		super();
		this.repository = repository;
		this.classId = classId;
		this.propertyId = propertyId;
		if (repository.isKey(classId, propertyId)) {
			isKey = true;
		} else {
			isKey = false;
		}
	}

	@Override
	public void clear() {
		throw new UnsupportedOperationException();

	}

	@Override
	public boolean containsKey(Object key) {
		if (key instanceof Integer && key != null) {
			int intKey = (int) key;
			return repository.hasValue(new ValueId(repository.getInstance(classId, intKey), propertyId));
		} else {
			return false;
		}

	}

	@Override
	public boolean containsValue(Object value) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Set<java.util.Map.Entry<Integer, Object>> entrySet() {
		return new Set<Map.Entry<Integer, Object>>() {

			@Override
			public <T> T[] toArray(T[] a) {
				throw new UnsupportedOperationException();
			}

			@Override
			public Object[] toArray() {
				throw new UnsupportedOperationException();
			}

			@Override
			public int size() {
				return size();
			}

			@Override
			public boolean retainAll(Collection<?> c) {

				throw new UnsupportedOperationException();
			}

			@Override
			public boolean removeAll(Collection<?> c) {

				throw new UnsupportedOperationException();
			}

			@Override
			public boolean remove(Object o) {

				throw new UnsupportedOperationException();
			}

			@Override
			public Iterator<Entry<Integer, Object>> iterator() {

				final Iterator<InstanceId> rowIterator = repository.getInstances(classId, propertyId).iterator();
				return new Iterator<Map.Entry<Integer, Object>>() {

					@Override
					public Entry<Integer, Object> next() {
						InstanceId instanceId = rowIterator.next();
						Integer key = repository.getInstanceIndex(instanceId, classId);
						Object value = get(instanceId);

						return new Entry<Integer, Object>() {

							@Override
							public Object setValue(Object value) {

								throw new UnsupportedOperationException();
							}

							@Override
							public Object getValue() {
								return value;
							}

							@Override
							public Integer getKey() {
								return key;
							}
						};

					}

					@Override
					public boolean hasNext() {
						return rowIterator.hasNext();
					}
				};
			}

			@Override
			public boolean isEmpty() {
				return isEmpty();
			}

			@Override
			public boolean containsAll(Collection<?> c) {

				throw new UnsupportedOperationException();
			}

			@Override
			public boolean contains(Object o) {

				throw new UnsupportedOperationException();
			}

			@Override
			public void clear() {

				throw new UnsupportedOperationException();

			}

			@Override
			public boolean addAll(Collection<? extends Entry<Integer, Object>> c) {

				throw new UnsupportedOperationException();
			}

			@Override
			public boolean add(Entry<Integer, Object> e) {

				throw new UnsupportedOperationException();
			}
		};
	}

	@Override
	public Object get(Object key) {

		InstanceId instance = repository.getInstance(classId, (Integer) key);
		Object value = repository.getValue(classId, instance, propertyId);
		if (isKey) {
			if (value instanceof String) {
				return new InstanceId((String) value).getReadableString();
			} else {
				throw new IllegalStateException("KEY MUST BE STRING");
			}
		}
		return value;
	}

	@Override
	public boolean isEmpty() {
		if (this.size() == 0)
			return true;
		else
			return false;
	}

	@Override
	public Set<Integer> keySet() {
		return new Set<Integer>() {

			@Override
			public <T> T[] toArray(T[] a) {

				throw new UnsupportedOperationException();
			}

			@Override
			public Object[] toArray() {

				throw new UnsupportedOperationException();
			}

			@Override
			public int size() {

				return repository.getNumberOfRows(classId);
			}

			@Override
			public boolean retainAll(Collection<?> c) {
				throw new UnsupportedOperationException();
			}

			@Override
			public boolean removeAll(Collection<?> c) {
				throw new UnsupportedOperationException();
			}

			@Override
			public boolean remove(Object o) {
				throw new UnsupportedOperationException();
			}

			@Override
			public Iterator<Integer> iterator() {

				final Iterator<InstanceId> rowIterator = repository.getInstances(classId, propertyId).iterator();
				return new Iterator<Integer>() {

					@Override
					public Integer next() {
						return repository.getInstanceIndex(rowIterator.next(), classId);
					}

					@Override
					public boolean hasNext() {
						return rowIterator.hasNext();
					}
				};
			}

			@Override
			public boolean isEmpty() {
				return isEmpty();
			}

			@Override
			public boolean containsAll(Collection<?> c) {

				for (Object oc : c) {
					if (!contains(oc)) {
						return false;
					}
				}

				return false;
			}

			@Override
			public boolean contains(Object o) {
				return containsKey(o);
			}

			@Override
			public void clear() {
				throw new UnsupportedOperationException();

			}

			@Override
			public boolean addAll(Collection<? extends Integer> c) {
				throw new UnsupportedOperationException();
			}

			@Override
			public boolean add(Integer e) {
				throw new UnsupportedOperationException();
			}
		};
	}

	@Override
	public Object put(Integer key, Object value) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void putAll(Map<? extends Integer, ? extends Object> m) {
		throw new UnsupportedOperationException();

	}

	@Override
	public Object remove(Object key) {
		throw new UnsupportedOperationException();
	}

	@Override
	public int size() {

		return repository.getNumberOfRows(classId);
	}

	@Override
	public Collection<Object> values() {

		throw new UnsupportedOperationException();
	}
}
