package de.mannheim.uni.expansion.data.dbpedia.repository;

import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections4.BidiMap;

import de.mannheim.uni.expansion.data.RemoteInterfaceWithCache;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNew.ClassPropertyNamePair;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.ListType;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class DBpediaRepositoryNewInterfaceImpl extends RemoteInterfaceWithCache implements DBpediaRepositoryNewInterface {

	private DBpediaRepositoryNewInterfaceRemote remote;

	public DBpediaRepositoryNewInterfaceImpl(DBpediaRepositoryNewInterfaceRemote remote) {
		this.remote = remote;
	}

	Set<ClassId> classes;

	@Override
	public Set<ClassId> getClasses() {
		try {
			if (classes == null)
				classes = remote.getClasses();
			return classes;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public BidiMap<InstanceId, Integer> getInstances(ClassId currentTable) {
		try {
			return remote.getInstances(currentTable);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int getTotalNumberOfColumns() {
		try {
			return remote.getTotalNumberOfColumns();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public BidiMap<PropertyId, Integer> getProperties(ClassId currentTable) {
		try {
			return remote.getProperties(currentTable);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int getTotalNumberOfClasses() {
		try {
			return remote.getTotalNumberOfClasses();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getNumberOfRows(ClassId dbPediaClass) {
		try {
			return remote.getNumberOfRows(dbPediaClass);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getNumberOfColumns(ClassId classId) {
		try {
			return remote.getNumberOfColumns(classId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public InstanceId getInstance(ClassId dbPediaClass, int index) {
		try {
			return remote.getInstance(dbPediaClass, index);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int getNumberOfHeaderRows(ClassId currentTable) {
		try {
			return remote.getNumberOfHeaderRows(currentTable);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getInstanceIndex(InstanceId instanceId, ClassId classId) {
		try {
			return remote.getInstanceIndex(instanceId, classId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getNumberOfInstances() {
		try {
			return remote.getNumberOfInstances();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getNumberOfInstances(ClassId classId) {
		try {
			return remote.getNumberOfInstances(classId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public Set<ClassId> getClasses(InstanceId instanceId) {
		try {
			return remote.getClasses(instanceId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public PropertyId getProperty(ClassId classId, int index) {
		try {
			return remote.getProperty(classId, index);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	ConcurrentHashMap<ClassPropertyId, Object> dataTypes = new ConcurrentHashMap<ClassPropertyId, Object>();

	private ValueGetter<ClassPropertyId> dataTypeGetter = (key) -> {
		return remote.getDataType(key.getClassId(), key.getPropertyId());
	};

	@Override
	public ColumnDataType getDataType(ClassId dbPediaClass, PropertyId dBpediaProperty) {
		return (ColumnDataType) cacheGetter(dataTypes, new ClassPropertyId(dbPediaClass, dBpediaProperty), dataTypeGetter);
	}

	@Override
	public boolean isKey(ClassId dbPediaClass, PropertyId dBpediaProperty) {
		try {
			return remote.isKey(dbPediaClass, dBpediaProperty);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	ConcurrentHashMap<ClassId, Object> classKeys = new ConcurrentHashMap<ClassId, Object>();

	private ValueGetter<ClassId> classKeyGetter = (key) -> {
		return (PropertyId) remote.getClassKey(key);
	};

	@Override
	public PropertyId getClassKey(ClassId classId) {

		return (PropertyId) cacheGetter(classKeys, classId, classKeyGetter);

	}

	@Override
	public boolean hasValue(TripleId tripleId) {
		try {
			return remote.hasValue(tripleId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public boolean hasValue(ValueId valueId) {
		try {
			return remote.hasValue(valueId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public boolean hasValue(ClassId classId, PropertyId propertyId, InstanceId instanceId) {
		try {
			return remote.hasValue(classId, propertyId, instanceId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public boolean hasValue(InstanceId instanceId, PropertyId propertyId) {
		try {
			// TODO Auto-generated method stub
			return remote.hasValue(instanceId, propertyId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public Set<InstanceId> getInstances(ClassId classId, PropertyId propertyId) {
		try {
			return remote.getInstances(classId, propertyId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int getInstancesSize(ClassId classId, PropertyId propertyId) {
		try {
			return remote.getInstancesSize(classId, propertyId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public int getTotalNumberOfRows() {
		try {
			return remote.getTotalNumberOfRows();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	ConcurrentHashMap<ValueId, Object> values = new ConcurrentHashMap<ValueId, Object>();

	private ValueGetter<ValueId> valueGetter = (key) -> {
		return remote.getValue(key);
	};

	@Override
	public Object getValue(ValueId key) {
		return cacheGetter(values, key, valueGetter);
	}

	@Override
	public Object getValue(ClassId dbPediaClass, InstanceId dbPediaInstance, PropertyId propertyKey) {
		return getValue(new ValueId(dbPediaInstance, propertyKey));
	}

	ConcurrentHashMap<ClassPropertyNamePair, Object> propertyShortNames = new ConcurrentHashMap<ClassPropertyNamePair, Object>();

	private ValueGetter<ClassPropertyNamePair> propertyShortNameGetter = (key) -> {
		return remote.getPropertyFromShortName(key.getClassId(), key.getPropertyName());
	};

	@Override
	public PropertyId getPropertyFromShortName(ClassId classId, String propertyName) {
		return (PropertyId) cacheGetter(propertyShortNames, new ClassPropertyNamePair(classId, propertyName), propertyShortNameGetter);
	}

	@Override
	public Object getValue(TripleId key) {
		return getValue(new ValueId(key.getInstanceId(), key.getPropertyId()));
	}

	@Override
	public boolean hasClass(ClassId classId) {
		try {
			return remote.hasClass(classId);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	ConcurrentHashMap<ClassPropertyId, Object> propertyExists = new ConcurrentHashMap<ClassPropertyId, Object>();

	private ValueGetter<ClassPropertyId> propertyExistsGetter = (key) -> {
		return (Boolean) remote.propertyExists(key.getClassId(), key.getPropertyId());
	};

	@Override
	public boolean propertyExists(ClassId classId, PropertyId propertyId) {
		return (boolean) cacheGetter(propertyExists, new ClassPropertyId(classId, propertyId), propertyExistsGetter);
	}

	ConcurrentHashMap<ClassPropertyId, Object> listTypes = new ConcurrentHashMap<ClassPropertyId, Object>();

	private ValueGetter<ClassPropertyId> isListTypeGetter = (key) -> {
		return (Boolean) remote.isListType(key);
	};

	@Override
	public boolean isListType(ClassPropertyId classPropertyId) {
		return (boolean) cacheGetter(listTypes, classPropertyId, isListTypeGetter);
	}

	ConcurrentHashMap<ListTypeClassPropertyId, Object> listTypesMap = new ConcurrentHashMap<ListTypeClassPropertyId, Object>();

	private ValueGetter<ListTypeClassPropertyId> isListTypeMapGetter = (key) -> {
		return (Boolean) remote.isListType(key.getListType(), key.getClassPropertyId());
	};

	@Override
	public boolean isListType(ListType listType, ClassPropertyId classPropertyId) {
		return (Boolean) cacheGetter(listTypesMap, new ListTypeClassPropertyId(listType, classPropertyId), isListTypeMapGetter);
	}

	public static class ListTypeClassPropertyId {
		private final ListType listType;
		private final ClassPropertyId classPropertyId;

		public ListTypeClassPropertyId(ListType listType, ClassPropertyId classPropertyId) {
			super();
			this.listType = listType;
			this.classPropertyId = classPropertyId;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((classPropertyId == null) ? 0 : classPropertyId.hashCode());
			result = prime * result + ((listType == null) ? 0 : listType.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			ListTypeClassPropertyId other = (ListTypeClassPropertyId) obj;
			if (classPropertyId == null) {
				if (other.classPropertyId != null)
					return false;
			} else if (!classPropertyId.equals(other.classPropertyId))
				return false;
			if (listType != other.listType)
				return false;
			return true;
		}

		public ListType getListType() {
			return listType;
		}

		public ClassPropertyId getClassPropertyId() {
			return classPropertyId;
		}

	}

	@Override
	public Map<ClassId, BidiMap<InstanceId, Integer>> getInstanceIndices() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Map<ClassId, BidiMap<PropertyId, Integer>> getPropertyIndices() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Map<ClassId, Integer> getTableIndeces() {
		throw new UnsupportedOperationException();
	}

	Set<ClassPropertyId> listTypesCache;

	@Override
	public Set<ClassPropertyId> getListTypes() {
		try {
			if (listTypesCache == null) {
				listTypesCache = remote.getListTypes();
			}
			return listTypesCache;
		} catch (RemoteException e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public Map<ValueId, Object> getValues() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Map<ClassId, PropertyId> getClassKeys() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Map<ClassPropertyId, ColumnDataType> getColumnDataTypes() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Map<ClassPropertyId, Set<InstanceId>> getInstancesPerClassProperty() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Set<ClassPropertyId> getListTypes(ListType listType) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Integer getPropertyIndex(ClassId classId, PropertyId propertyId) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean hasInstance(ClassId classId, InstanceId instanceId) {
		throw new UnsupportedOperationException();
	}

}
