package de.mannheim.uni.expansion.data.ontology.model.repository;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassOntology;
import de.mannheim.uni.expansion.data.ontology.model.properties.DataTypePropertyOntology;
import de.mannheim.uni.expansion.data.ontology.model.properties.DataTypePropertyOntology.Type;
import de.mannheim.uni.expansion.data.ontology.model.properties.ObjectPropertyOntology;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyOntology;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class OntologyRepositoryImplMemory implements OntologyRepository {

	final private Map<ClassId, ClassOntology> classes;
	final private Map<PropertyId, PropertyOntology> properties;

	final DBpediaRepositoryNewInterface dBpediaRepositoryNew;

	final private Map<ClassId, Set<ClassId>> subClasses;

	public OntologyRepositoryImplMemory(DBpediaRepositoryNewInterface dBpediaRepositoryNew, Map<ClassId, ClassOntology> classes,
			Map<PropertyId, PropertyOntology> properties) {

		this.classes = classes;
		this.properties = properties;
		this.dBpediaRepositoryNew = dBpediaRepositoryNew;

		this.subClasses = new HashMap<>();

		for (Entry<ClassId, ClassOntology> subClass : classes.entrySet()) {

			for (ClassId classId : subClass.getValue().getSubClassOf()) {
				Set<ClassId> subClassesSet = CHMHelper.getFromMap(this.subClasses, classId, () -> new HashSet<ClassId>());
				subClassesSet.add(subClass.getKey());
			}

		}
	}

	@Override
	public PropertyOntology get(PropertyId propertyId) {
		return properties.get(propertyId);
	}

	@Override
	public ClassOntology get(ClassId classId) {
		return classes.get(classId);
	}

	@Override
	public ColumnDataType getDataType(ClassPropertyId classPropertyId) {
		if (this.get(classPropertyId.getPropertyId()) instanceof ObjectPropertyOntology) {
			return ColumnDataType.reference;
		}

		ColumnDataType cdt = dBpediaRepositoryNew.getDataType(classPropertyId.getClassId(), classPropertyId.getPropertyId());

		if (cdt == null) {

			PropertyOntology property = this.properties.get(classPropertyId.getPropertyId());
			if (property instanceof DataTypePropertyOntology) {
				Type range = ((DataTypePropertyOntology) property).getRange();
				if (range == null) {
					int fsdfsdf = 112123;
				} else {
					cdt = range.getColumnDataType();
				}
			} else if (property != null) {
				System.out.println("WHY IS SOMEONE ASKING FOR DATATYPE OF OBJECT PROPERTY");
			}
		}
		if (cdt == null) {
			int fddf = 123;
			// FIXME:
			// System.out.println("OWL PROPERTY MISSING: " +
			// classPropertyId.getPropertyId());
		}
		return cdt;
	}

	@Override
	public ColumnDataType getDataType(ClassId classId, PropertyId propertyId) {
		return getDataType(new ClassPropertyId(classId, propertyId));
	}

	@Override
	public Set<ClassId> getSubClasses(ClassId classId) {
		return this.subClasses.get(classId);
	}

}
