package de.mannheim.uni.expansion.data.tables.reader;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.utils.BatchConsumerProducer;
import de.mannheim.uni.expansion.utils.containers.Pair;

public class AnnotatedTablesGzReader implements AnnotatedTablesReader {

	private final String folder;
	private final File[] files;
	private final AnnotatedTableParser parser;
	private final FileEnding fileEnding;
	private final String badAnnotatedTablesFile;
	private final TableType tableType;

	public AnnotatedTablesGzReader(String folder, FileEnding fileEnding, TableType tableType, AnnotatedTableParser parser, String badAnnotatedTablesFile) {
		this.fileEnding = fileEnding;
		this.tableType = tableType;
		this.folder = folder;

		files = new File(folder).listFiles();

		this.parser = parser;
		this.badAnnotatedTablesFile = badAnnotatedTablesFile;

	}

	public int length() {
		return files.length;
	}

	public String getFolder() {
		return folder;
	}

	public void runForEachTableMetaData(Consumer<AnnotatedTablesReaderData> consumer) {
		run(consumer, null);
	}

	public void runForEachTable(Consumer<AnnotatedTablesReaderData> fullConsumer) {
		run(null, fullConsumer);
	}

	private void run(Consumer<AnnotatedTablesReaderData> metaConsumer, Consumer<AnnotatedTablesReaderData> fullConsumer) {

		AtomicInteger ai = new AtomicInteger(0);
		Consumer<Pair<byte[], String>> ppBody = (entry) -> {
			try {
				processEntry(entry.getFirst(), entry.getSecond(), metaConsumer, fullConsumer, ai);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		};

		BatchConsumerProducer<Pair<byte[], String>> consumer = new BatchConsumerProducer<>("GZREADER", ppBody, 1000);

		try {
			final BadAnnotatedTablesWriter badAnnotatedTablesWriter;
			if (badAnnotatedTablesFile != null) {
				badAnnotatedTablesWriter = new BadAnnotatedTablesWriter(badAnnotatedTablesFile);
			} else {
				badAnnotatedTablesWriter = null;
			}

			try {

				CountDownLatch finalLatch = new CountDownLatch(1);

				Runnable r = () -> {
					for (int i = 0; i < files.length; i++) {

						List<Pair<byte[], String>> entryList = new ArrayList<>();

						TarArchiveInputStream tarStream = null;
						try {
							tarStream = new TarArchiveInputStream(new GzipCompressorInputStream(new FileInputStream(files[i])));

							TarArchiveEntry entry;

							while ((entry = tarStream.getNextTarEntry()) != null) {

								if (!entry.isDirectory()) {

									byte[] content = new byte[(int) entry.getSize()];

									tarStream.read(content, 0, content.length);

									String a = new String(content);
									//
									// int numLines = countLines(content) - 9;

									String[] nameParts = entry.getName().split("/");

									String name = nameParts[nameParts.length - 1];
									entryList.add(new Pair<byte[], String>(content, name));

									// System.out.println(name);

								}
							}

							consumer.addBatch(entryList);

						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} finally {
							if (tarStream != null)
								try {
									tarStream.close();
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
						}

					}

					consumer.finish();

					finalLatch.countDown();
				};

				Thread t = new Thread(r);
				t.start();

				finalLatch.await();

				// PersistentParallel.forLoop(0, files.length, new
				// Consumer<Integer>() {
				//
				// @Override
				// public void accept(Integer i) {
				//
				// try {
				//
				// } catch (IOException e) {
				// // TODO Auto-generated catch block
				// e.printStackTrace();
				// } finally {
				//
				// }
				// }
				//
				// }, "AnnotatedTableReader");
			} catch (Exception e) {
				e.printStackTrace();
			}

			if (badAnnotatedTablesFile != null) {
				badAnnotatedTablesWriter.finishAndClose();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("BAD TABLES REMOVED DUE TO BUG (NO TABLE TYPE?): " + ai.get());
	}

	private void processEntry(byte[] content, String name, Consumer<AnnotatedTablesReaderData> metaConsumer, Consumer<AnnotatedTablesReaderData> fullConsumer,
			AtomicInteger ai) throws IOException {
		try {
			if (fullConsumer != null) {

				AnnotatedTablesReaderData data = parser.get(content, name, fileEnding, tableType);

				if (data == null) {
					ai.incrementAndGet();
				} else {
					fullConsumer.accept(data);

					if (metaConsumer != null) {
						metaConsumer.accept(data);
					}
				}

				// int size1 = numLines;
				// int size2 =
				// data.getMetaData().getNumberOfRows()
				// +
				// data.getMetaData().getNumHeaderRows();
				// if (size1 != size2) {
				// ai.incrementAndGet();
				// }
				// if (size1 != size2 &&
				// badAnnotatedTablesFile != null) {
				// BufferedReader br = new
				// BufferedReader(new
				// InputStreamReader(new
				// ByteArrayInputStream(content)));
				// for (int sk = 0; sk < 9; sk++) {
				// br.readLine();
				// }
				//
				// ByteArrayOutputStream writer =
				// new
				// ByteArrayOutputStream();
				// IOUtils.copy(br, writer,
				// "UTF-8");
				//
				// badAnnotatedTablesWriter.writeEntry(name,
				// writer.toByteArray());
				//
				// } else {

				// }

			} else if (metaConsumer != null) {
				AnnotatedTableId id = new AnnotatedTableId(name.substring(0, name.length() - 4));
				AnnotatedTableMetaData metaData = parser.getMetaData(content, name, fileEnding);
				AnnotatedTablesReaderData data = new AnnotatedTablesReaderData(id, metaData, null);
				metaConsumer.accept(data);
			}
		} catch (InvalidTableException e) {
			System.out.println("WARNING: Invalid Header in File: " + e.getTable());
		} finally {

		}
		// TODO Auto-generated method stub

	}

	public static int countLines(byte[] filename) throws IOException {
		InputStream is = new BufferedInputStream(new ByteArrayInputStream(filename));
		try {
			byte[] c = new byte[1024];
			int count = 0;
			int readChars = 0;
			boolean empty = true;
			while ((readChars = is.read(c)) != -1) {
				empty = false;
				for (int i = 0; i < readChars; ++i) {
					if (c[i] == '\n') {
						++count;
					}
				}
			}
			return (count == 0 && !empty) ? 1 : count;
		} finally {
			is.close();
		}
	}
}
