package de.mannheim.uni.expansion.data.tables.repository;

import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import de.mannheim.uni.expansion.data.RemoteInterfaceWithCache;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableData;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.HostId;
import de.mannheim.uni.expansion.data.tables.repository.remote.AnnotatedTablesRepositoryRemote;
import de.mannheim.uni.tableprocessor.model.Table;

public class AnnotatedTablesRepositoryImpl extends RemoteInterfaceWithCache implements AnnotatedTablesRepository {

	AnnotatedTablesRepositoryRemote remote;

	public AnnotatedTablesRepositoryImpl(AnnotatedTablesRepositoryRemote remote) {
		this.remote = remote;

	}

	@Override
	public Set<AnnotatedTableId> ofClass(ClassId dbpediaClass) {
		try {
			return remote.ofClass(dbpediaClass);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Set<AnnotatedTableId> hasProperty(PropertyId property) {
		try {
			return remote.hasProperty(property);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Set<AnnotatedTableId> hasProperties(Set<PropertyId> properties) {
		try {
			return remote.hasProperties(properties);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Set<AnnotatedTableId> hasInstance(InstanceId instances) {
		try {
			return remote.hasInstance(instances);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Set<AnnotatedTableId> hasInstances(Set<InstanceId> instances) {
		try {
			return remote.hasInstances(instances);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	ConcurrentHashMap<AnnotatedTableId, Object> metaData = new ConcurrentHashMap<AnnotatedTableId, Object>();

	private ValueGetter<AnnotatedTableId> metaDataGetter = (key) -> {
		return remote.getMetaData(key);
	};

	@Override
	public AnnotatedTableMetaData getMetaData(AnnotatedTableId id) {
		return (AnnotatedTableMetaData) cacheGetter(metaData, id, metaDataGetter);
	}

	@Override
	public Table getData(AnnotatedTableId id) {
		try {
			return remote.getData(id);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public AnnotatedTableData getData(AnnotatedTableId id, int row) {
		try {
			return remote.getData(id, row);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Map<AnnotatedTableId, String> getValues(InstanceId instance, PropertyId property) {
		try {
			return remote.getValues(instance, property);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Set<InstanceId> getInstances() {
		try {
			return remote.getInstances();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	Set<AnnotatedTableId> allTableIds;

	@Override
	public Set<AnnotatedTableId> getAllTableIds() {
		try {
			if (allTableIds == null)
				allTableIds = remote.getAllTableIds();
			return allTableIds;
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Map<Integer, Object> getColumn(AnnotatedTableId id, int columnIndex) {
		try {
			return remote.getColumn(id, columnIndex);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Object getValue(AnnotatedTableId id, int column, int row) {
		try {
			return remote.getValue(id, column, row);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public Object getKeyValue(AnnotatedTableId id, int row) {
		try {
			return remote.getKeyValue(id, row);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	ConcurrentHashMap<AnnotatedTableId, Object> hostIds = new ConcurrentHashMap<AnnotatedTableId, Object>();

	private ValueGetter<AnnotatedTableId> hostIdGetter = (key) -> {
		return remote.getHostId(key);
	};

	@Override
	public HostId getHostId(AnnotatedTableId id) {
		return (HostId) cacheGetter(hostIds, id, hostIdGetter);
	}

	@Override
	public void printStatistics() {
		// TODO Auto-generated method stub

	}

	// @Override
	// public HostId getHostId(AnnotatedTableId id) {
	// try {
	// return remote.getHostId(id);
	// } catch (RemoteException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// return null;
	// }

}
