package de.mannheim.uni.expansion.data.tables.repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.EquivalenceClusterer;
import de.mannheim.uni.expansion.tablefusion.core.fusing.EquivalenceClusterer.Cluster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseNumericFuser;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MagnitudeFixer {

	private final DBpediaRepositoryNewInterface dbpedia;
	private final EquivalenceClusterer equivalenceClusterer;
	private final SimilarityMeasure measure;
	private final BaseFuser baseFuser;

	@Inject
	public MagnitudeFixer(DBpediaRepositoryNewInterface dbpedia, EquivalenceClusterer equivalenceClusterer, SimilarityCalculator sc,
			BaseNumericFuser baseFuser) {
		super();
		this.dbpedia = dbpedia;
		this.equivalenceClusterer = equivalenceClusterer;
		this.measure = sc.getMeasure(ColumnDataType.numeric);
		this.baseFuser = baseFuser;
	}

	public Table transform(AnnotatedTableMetaData md, Table td) {
		for (Entry<Integer, PropertyId> prop : md.getProperties().entrySet()) {
			PropertyId propertyId = prop.getValue();
			if (prop.getValue() != null && td.getColumns().get(prop.getKey()).getDataType().equals(ColumnDataType.numeric)) {
				TableColumn tc = td.getColumns().get(prop.getKey());

				List<Double> magnitudes = new ArrayList<>();

				for (Entry<Integer, Object> v : tc.getValues().entrySet()) {
					InstanceId instanceId = md.getInstance(v.getKey());
					if (instanceId != null) {
						ValueId vid = new ValueId(instanceId, propertyId);
						if (dbpedia.hasValue(instanceId, propertyId)) {
							Object o = dbpedia.getValue(vid);
							if (o instanceof Double) {
								Double dbpediaVaue = (Double) dbpedia.getValue(vid);
								Double tableValue = (Double) v.getValue();
								if (tableValue != 0.0) {
									magnitudes.add(dbpediaVaue / tableValue);
								}
							}
						}
					}

				}

				if (magnitudes.size() > 0.4 * tc.getValues().size()) {
					List<Cluster> clusters = new ArrayList<>();
					for (Double value : magnitudes) {
						clusters = equivalenceClusterer.addValue(clusters, measure, baseFuser, value);
					}

					Cluster c = equivalenceClusterer.getHighestRatedCluster(clusters);
					Double magnitude = (Double) c.getValue();
					if (c.getCount() > magnitudes.size() * 0.7 && (magnitude < 0.90 || magnitude > 1.1)) {
						for (Entry<Integer, Object> v : tc.getValues().entrySet()) {
							v.setValue((Double) v.getValue() * magnitude);
						}
					}

				}
			}

		}

		return td;
	}
}
