package de.mannheim.uni.expansion.data.tables.repository.memory;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableData;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.HostId;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.data.tables.repository.remote.AnnotatedTablesRepositoryRemote;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;

public class AnnotatedTablesRepositoryImplMemory implements AnnotatedTablesRepositoryRemote, AnnotatedTablesRepository {

	private final ConcurrentHashMap<ClassId, Set<AnnotatedTableId>> classIndex;
	private final ConcurrentHashMap<PropertyId, Set<AnnotatedTableId>> propertyIndex;
	private final ConcurrentHashMap<InstanceId, Set<AnnotatedTableId>> instanceIndex;

	private final ConcurrentHashMap<AnnotatedTableId, Table> tableData;
	private final ConcurrentHashMap<AnnotatedTableId, AnnotatedTableMetaData> tableMetaData;

	AnnotatedTablesRepositoryImplMemory(ConcurrentHashMap<ClassId, Set<AnnotatedTableId>> classIndex,
			ConcurrentHashMap<PropertyId, Set<AnnotatedTableId>> propertyIndex, ConcurrentHashMap<InstanceId, Set<AnnotatedTableId>> instanceIndex,
			ConcurrentHashMap<AnnotatedTableId, Table> tableData, ConcurrentHashMap<AnnotatedTableId, AnnotatedTableMetaData> tableMetaData) {
		this.classIndex = classIndex;
		this.propertyIndex = propertyIndex;
		this.instanceIndex = instanceIndex;
		this.tableData = tableData;
		this.tableMetaData = tableMetaData;

		int i = 0;
		for (AnnotatedTableMetaData md : tableMetaData.values()) {
			for (List<PropertyId> list : md.getAllProperties().values()) {
				i += list.size();
			}
		}
		System.out.println("Number of mapped properties: " + i);
	}

	public Set<AnnotatedTableId> ofClass(ClassId dbpediaClass) {
		Set<AnnotatedTableId> set = classIndex.get(dbpediaClass);
		if (set == null)
			return null;
		return Collections.unmodifiableSet(set);
	}

	public Set<AnnotatedTableId> hasProperty(PropertyId property) {
		Set<AnnotatedTableId> set = propertyIndex.get(property);
		if (set == null)
			return null;
		return Collections.unmodifiableSet(set);
	}

	public Set<AnnotatedTableId> hasProperties(Set<PropertyId> properties) {
		// TODO Auto-generated method stub
		throw new IllegalStateException("NOT YET IMPLEMENTED");
	}

	public Set<AnnotatedTableId> hasInstance(InstanceId instances) {

		Set<AnnotatedTableId> set = instanceIndex.get(instances);
		if (set == null)
			return null;
		return Collections.unmodifiableSet(set);
	}

	public Set<AnnotatedTableId> hasInstances(Set<InstanceId> instances) {
		// TODO Auto-generated method stub
		throw new IllegalStateException("NOT YET IMPLEMENTED");
	}

	public AnnotatedTableMetaData getMetaData(AnnotatedTableId id) {
		return this.tableMetaData.get(id);
	}

	public Table getData(AnnotatedTableId id) {
		return this.tableData.get(id);
	}

	public AnnotatedTableData getData(AnnotatedTableId id, int row) {
		// TODO Auto-generated method stub
		throw new IllegalStateException("NOT YET IMPLEMENTED");
	}

	public Map<AnnotatedTableId, String> getValues(InstanceId instance, PropertyId property) {
		Map<AnnotatedTableId, String> values = new HashMap<AnnotatedTableId, String>();

		Set<AnnotatedTableId> tables = new HashSet<AnnotatedTableId>();

		Set<AnnotatedTableId> instanceTables = this.instanceIndex.get(instance);
		Set<AnnotatedTableId> propertyTables = this.propertyIndex.get(property);

		if (instanceTables == null | propertyTables == null)
			return values;

		for (AnnotatedTableId id : instanceTables) {
			if (propertyTables.contains(id)) {
				tables.add(id);
			}
		}

		for (AnnotatedTableId id : tables) {

			AnnotatedTableMetaData metaData = this.tableMetaData.get(id);
			int row = metaData.getRow(instance);
			int col = metaData.getCol(property);

			Table t = this.tableData.get(id);
			Object o = t.getColumns().get(col).getValues().get(row);
			if (o != null) {

				values.put(id, o.toString());
			}
		}

		return values;
	}

	@Override
	public Set<InstanceId> getInstances() {
		return Collections.unmodifiableSet(instanceIndex.keySet());
	}

	@Override
	public Set<AnnotatedTableId> getAllTableIds() {
		return Collections.unmodifiableSet(tableMetaData.keySet());
	}

	@Override
	public Map<Integer, Object> getColumn(AnnotatedTableId id, int columnIndex) {
		return tableData.get(id).getColumns().get(columnIndex).getValues();
	}

	@Override
	public Object getValue(AnnotatedTableId id, int column, int row) {

		try {
			return getData(id).getColumns().get(column).getValues().get(row);
		} catch (NullPointerException e) {
			return null;
		}
	}

	@Override
	public Object getKeyValue(AnnotatedTableId id, int row) {

		try {
			return getData(id).getKey().getValues().get(row);
		} catch (NullPointerException e) {
			return null;
		}
	}

	@Override
	public HostId getHostId(AnnotatedTableId id) {
		return this.getMetaData(id).getHostId();
	}

	@Override
	public void printStatistics() {
		AtomicInteger totalValues = new AtomicInteger(0);
		AtomicInteger mappedValues = new AtomicInteger(0);
		AtomicInteger allTables = new AtomicInteger(0);
		AtomicInteger mappedColumns = new AtomicInteger(0);

		// ConcurrentHashMap<AnnotatedTableId, Boolean> tableIds = new
		// ConcurrentHashMap<AnnotatedTableId, Boolean>();

		try {
			new PersistentParallel<AnnotatedTableId>("Print Table Statistics").foreach(getAllTableIds(), (id) -> {
				allTables.incrementAndGet();

				int totalValuesTemp = 0;
				int mappedValuesTemp = 0;
				int mappedColumnsTemp = 0;

				AnnotatedTableMetaData md = this.getMetaData(id);
				Table td = this.getData(id);

				Map<Integer, List<PropertyId>> columnMapings = md.getAllProperties();
				Map<Integer, InstanceId> rowMappings = md.getInstances();

				int columnIdx = 0;
				for (TableColumn tc : td.getColumns()) {
					boolean mapped = false;
					if (columnMapings.get(columnIdx).size() > 0) {
						mapped = true;
						mappedColumnsTemp++;
					}
					for (Entry<Integer, Object> v : tc.getValues().entrySet()) {

						if (v.getValue() != null) {
							totalValuesTemp++;
							if (mapped && rowMappings.containsKey(v.getKey())) {
								mappedValuesTemp++;
							}
						}
					}

					columnIdx++;
				}

				totalValues.addAndGet(totalValuesTemp);
				mappedValues.addAndGet(mappedValuesTemp);
				mappedColumns.addAndGet(mappedColumnsTemp);
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("Number of all Tables: " + allTables.get());
		System.out.println("Number of mapped Properties: " + mappedColumns.get());
		System.out.println("Number of total Values: " + totalValues.get());
		System.out.println("Number of mapped Values: " + mappedValues.get());

	}
}
