package de.mannheim.uni.expansion.tablefusion.core.similarity;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.BooleanSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.DateSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.EmptySimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.LinkSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.NumericSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.ReferenceSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.StringSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.StringSimilarityOld;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching.MatchingDateSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching.MatchingNumericSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching.MatchingStringSimilarity;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class SimilarityCalculator {

	public final BooleanSimilarity booleanSimilarity;
	public final NumericSimilarity numericSimilarity;
	public final StringSimilarity stringSimilarity;
	public final StringSimilarityOld stringSimilarityOld;
	public final EmptySimilarity emptySimilarity;
	public final DateSimilarity dateSimilarity;
	public final LinkSimilarity linkSimilarity;
	public final ReferenceSimilarity referenceSimilarity;

	public final MatchingNumericSimilarity matchingNumericSimilarity;
	public final MatchingStringSimilarity matchingStringSimilarity;
	public final MatchingDateSimilarity matchingDateSimilarity;

	@Inject
	public SimilarityCalculator(BooleanSimilarity booleanSimilarity, NumericSimilarity numericSimilarity, StringSimilarity stringSimilarity,
			StringSimilarityOld stringSimilarityOld,
			EmptySimilarity emptySimilarity, DateSimilarity dateSimilarity, LinkSimilarity linkSimilarity, MatchingNumericSimilarity matchingNumericSimilarity,
			MatchingStringSimilarity matchingStringSimilarity, MatchingDateSimilarity matchingDateSimilarity, ReferenceSimilarity referenceSimilarity) {
		this.booleanSimilarity = booleanSimilarity;
		this.numericSimilarity = numericSimilarity;
		this.stringSimilarity = stringSimilarity;
		this.stringSimilarityOld = stringSimilarityOld;
		this.dateSimilarity = dateSimilarity;
		this.emptySimilarity = emptySimilarity;
		this.linkSimilarity = linkSimilarity;
		this.referenceSimilarity = referenceSimilarity;

		this.matchingNumericSimilarity = matchingNumericSimilarity;
		this.matchingStringSimilarity = matchingStringSimilarity;
		this.matchingDateSimilarity = matchingDateSimilarity;
	}

	public double getSimilarity(ColumnDataType cdt, Object a, Object b) {
		return getMeasure(cdt).get(a, b);
	}

	public boolean isEqual(ColumnDataType cdt, Object a, Object b) {
		return getMeasure(cdt).isEqual(a, b);
	}

	public double getEquivalenceThreshold(ColumnDataType cdt) {
		return getMeasure(cdt).getEquivalenceThreshold();
	}

	public SimilarityMeasure getMeasure(ColumnDataType cdt) {
		if (cdt == null) {
			// TODO: FIX THIS SERIOUSLY
			return emptySimilarity;
		}
		switch (cdt) {
		case bool:
			return booleanSimilarity;

		case coordinate:
			break;
		case date:
			return dateSimilarity;
		case link:
			return linkSimilarity;
		case list:
			break;
		case numeric:
			return numericSimilarity;
		case string:
			return stringSimilarity;
		case reference:
			return referenceSimilarity;
		case unit:
			break;
		case unknown:
			break;
		default:
			break;

		}

		// System.out.println(cdt.toString());

		return emptySimilarity;
	}
}
