package de.mannheim.uni.expansion.tablefusion.core.similarity.list;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;

public class OverlapSimilarity {

	
	@Inject
	public OverlapSimilarity() {
	
	}
	
	public int getOverlap(List<?> aList, List<?> bList, SimilarityMeasure measure) {
		int intersection = 0;
		double et = measure.getEquivalenceThreshold();

		Double[][] matrix = new Double[aList.size()][bList.size()];

		for (int a = 0; a < aList.size(); a++) {
			for (int b = 0; b < bList.size(); b++) {
				matrix[a][b] = measure.getQuickSimilarity(aList.get(a), bList.get(b));
			}
		}

		Set<Integer> aIndeces = new HashSet<>();
		for (int a = 0; a < aList.size(); a++) {
			aIndeces.add(a);
		}

		Set<Integer> bIndeces = new HashSet<>();
		for (int b = 0; b < bList.size(); b++) {
			bIndeces.add(b);
		}

		double max = 0.0;
		int aMax = -1;
		int bMax = -1;

		outerloop: while (true) {

			max = 0.0;
			aMax = -1;
			bMax = -1;

			for (Iterator<Integer> aIt = aIndeces.iterator(); aIt.hasNext();) {
				int a = aIt.next();

				for (Iterator<Integer> bIt = bIndeces.iterator(); bIt.hasNext();) {
					int b = bIt.next();

					Double sim = matrix[a][b];
					if (sim > 0.999999) {
						intersection++;
						aIt.remove();
						bIt.remove();
						continue outerloop;
					} else if (sim > measure.getEquivalenceThreshold() && sim > max) {
						max = sim;
						aMax = a;
						bMax = b;
					}

				}

			}

			if (max < measure.getEquivalenceThreshold()) {
				break;
			} else {
				intersection++;
				aIndeces.remove(aMax);
				bIndeces.remove(bMax);
			}

		}
		return intersection;
	}

	public double getSimilarity(List<?> aList, List<?> bList, SimilarityMeasure measure) {
		int smallerSize = 0;

		if (aList.size() < bList.size()) {
			smallerSize = aList.size();
		} else {
			smallerSize = bList.size();
		}

		int intersection = getOverlap(aList, bList, measure);

		double sim = ((double) intersection) / ((double) smallerSize);
		return sim;
	}

	public boolean hasCompleteOverlap(List<?> aList, List<?> bList, SimilarityMeasure measure) {
		int smallerSize = 0;

		if (aList.size() < bList.size()) {
			smallerSize = aList.size();
		} else {
			smallerSize = bList.size();
		}

		int intersection = getOverlap(aList, bList, measure);
		if (intersection == smallerSize) {
			return true;
		} else {
			return false;
		}

	}

	public boolean isIdentical(List<?> aList, List<?> bList, SimilarityMeasure measure) {
		int size = 0;

		if (aList.size() == bList.size()) {
			size = aList.size();

			int intersection = getOverlap(aList, bList, measure);
			if (intersection == size) {
				return true;
			} else {
				return false;
			}

		} else {
			return false;
		}

	}

	public boolean hasOverlap(List<?> aList, List<?> bList, SimilarityMeasure measure, int size) {
		int intersection = getOverlap(aList, bList, measure);
		if (intersection == size)
			return true;
		else
			return false;
	}


	public double getMAP(List<Object> correctList, List<Set<Object>> rankedList, SimilarityMeasure measure) {
		
		List<Double> averagePrecisions = new ArrayList<Double>();
		double precisionSum = 0.0;

		double negativeCounter = 0.0;

		double positiveCounter = 0.0;
		
		
		// https://www.kaggle.com/wiki/MeanAveragePrecision
		for (Set<Object> tr : rankedList) {
			double averagePrecision;
			
			for(Object o : tr) {
				if (isCorrect(correctList, o, measure)) {
					positiveCounter++;
				} else {
					negativeCounter++;

				}
			}

			double precisionAtItem = positiveCounter / (negativeCounter + positiveCounter);
			precisionSum += precisionAtItem;

			averagePrecision = precisionSum / (averagePrecisions.size() + 1);

			averagePrecisions.add(averagePrecision);
		}

		// TODO Auto-generated method stub
		Double precision = 0.0;
		for (double p : averagePrecisions) {
			precision += p;
		}
		precision = precision / averagePrecisions.size();

		if (precision.equals(Double.NaN)) {
			return 0.0;
		} else {
			return precision;
		}
	}

	private boolean isCorrect(List<Object> correctList, Object object, SimilarityMeasure measure) {

		for (Object correct : correctList) {
			if (measure.isEqual(correct, object)) {
				return true;
			}

		}
		return false;

	}
}
