package de.mannheim.uni.expansion.tablefusion.core.similarity.measures;

import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityMeasures;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThreshold;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThresholdsRepository;
import de.mannheim.uni.expansion.utils.ThreadSafeLocalCalendar;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class DateSimilarity extends AbstractSimilarity {

	private final ThreadSafeLocalCalendar threadSafeCalendar;

	private final NumericSimilarity numericSimilarity;

	private final SimilarityThreshold similarityThreshold;

	@Inject
	public DateSimilarity(SimilarityThresholdsRepository similarityThresholdsRepository, ThreadSafeLocalCalendar threadSafeCalendar,
			NumericSimilarity numericSimilarity) {
		this.similarityThreshold = similarityThresholdsRepository.get(SimilarityMeasures.DATE);
		this.numericSimilarity = numericSimilarity;
		this.threadSafeCalendar = threadSafeCalendar;
	}

	private boolean isYearDate(Date date) {
		Calendar calendar = threadSafeCalendar.getCalendar();
		calendar.setTime(date);

		int month = calendar.get(Calendar.MONTH);
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		int hourOfDay = calendar.get(Calendar.HOUR_OF_DAY);
		int minute = calendar.get(Calendar.MINUTE);
		int second = calendar.get(Calendar.SECOND);
		if (month == 0 && day == 1 && hourOfDay == 0 && minute == 0 && second == 0) {
			return true;
		}

		return false;

	}

	private boolean isMonthDate(Date date) {
		Calendar calendar = threadSafeCalendar.getCalendar();
		calendar.setTime(date);

		int day = calendar.get(Calendar.DAY_OF_MONTH);
		int hourOfDay = calendar.get(Calendar.HOUR_OF_DAY);
		int minute = calendar.get(Calendar.MINUTE);
		int second = calendar.get(Calendar.SECOND);
		if (day == 1 && hourOfDay == 0 && minute == 0 && second == 0) {
			return true;
		}

		return false;

	}

	private boolean sameMonth(Date dateA, Date dateB) {
		Calendar calendar = threadSafeCalendar.getCalendar();
		calendar.setTime(dateA);

		int yearA = calendar.get(Calendar.YEAR);
		int monthA = calendar.get(Calendar.MONTH);

		calendar.setTime(dateB);

		int yearB = calendar.get(Calendar.YEAR);
		int monthB = calendar.get(Calendar.MONTH);

		if (yearA == yearB && monthA == monthB) {
			return true;
		}

		return false;
	}

	private boolean sameYear(Date dateA, Date dateB) {
		Calendar calendar = threadSafeCalendar.getCalendar();
		calendar.setTime(dateA);

		int yearA = calendar.get(Calendar.YEAR);

		calendar.setTime(dateB);

		int yearB = calendar.get(Calendar.YEAR);

		if (yearA == yearB) {
			return true;
		}

		return false;
	}

	@Override
	public double normalSimilarity(Object a, Object b) {

		if (a == null || b == null) {
			return 0.0;
		} else if (!(a instanceof Date) || !(b instanceof Date)) {
			return 0.0;
		} else {
			Date dateA = (Date) a;
			Date dateB = (Date) b;

			if (isYearDate(dateA) || isYearDate(dateB)) {

				if (sameYear(dateA, dateB)) {
					return 1.0;
				} else {
					return 0.0;
				}

			} else if (isMonthDate(dateA) || isMonthDate(dateB)) {
				if (sameMonth(dateA, dateB)) {
					return 1.0;
				} else {
					return 0.0;
				}
			} else {
				long days1 = TimeUnit.MILLISECONDS.toDays(((Date) a).getTime());
				long days2 = TimeUnit.MILLISECONDS.toDays(((Date) b).getTime());

				double v1 = (double) days1;
				double v2 = (double) days2;

				return numericSimilarity.get(v1, v2);
			}

		}
	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.date;
	}

	@Override
	public double getEquivalenceThreshold() {
		return similarityThreshold.get();
	}

}
