package de.mannheim.uni.expansion.tablefusion.core.similarity.measures;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class LinkSimilarity extends AbstractSimilarity {

	@Inject
	public LinkSimilarity() {

	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected double normalSimilarity(Object a, Object b) {
//		System.out.println("Link not matched: " + a + " : " + b);
		if (a == null || b == null) {
			return 0.0;
		} else if (a.equals(b)) {
			return 1.0;
		} else if (!(a instanceof String) || !(b instanceof String)) {
			return 0.0;
		} else {
			String a2 = ((String) a).toLowerCase().trim();
			String b2 = ((String) b).toLowerCase().trim();

			if (a2.startsWith("http://")) {
				a2 = a2.substring(7);
			} else if (a2.startsWith("https://")) {
				a2 = a2.substring(8);
			}

			if (b2.startsWith("http://")) {
				b2 = b2.substring(7);
			} else if (b2.startsWith("https://")) {
				b2 = b2.substring(8);
			}

			if (a2.equals(b2)) {
				return 1.0;
			} else {
//				System.out.println("Link not matched: " + a2 + " : " + b2);
				return 0.0;
			}
		}
	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.link;
	}

	@Override
	public double getEquivalenceThreshold() {
		return 0.95;
	}
}
