package de.mannheim.uni.expansion.tablefusion.core.similarity.measures;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityMeasures;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThreshold;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThresholdsRepository;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class NumericSimilarity extends AbstractSimilarity {

	private final SimilarityThreshold similarityThreshold;
	@Inject
	public NumericSimilarity(SimilarityThresholdsRepository similarityThresholdsRepository) {
		this.similarityThreshold = similarityThresholdsRepository.get(SimilarityMeasures.NUMERIC);
	}

	@Override
	public double normalSimilarity(Object a, Object b) {
		if (a == null || b == null) {
			return 0.0;
		} else {
			try {
				double v1 = cast(a);
				double v2 = cast(b);

				return get(v1, v2);
			} catch (ClassCastException e) {
				return 0.0;
			}
		}
	}

	private double cast(Object o) {
		if (o instanceof Double) {
			return (Double) o;
		} else if (o instanceof Integer) {
			return ((Integer) o).doubleValue();
		} else if (o instanceof String) {
			return Double.parseDouble((String) o);
		}
		throw new ClassCastException();
	}

	double get(double v1, double v2) {
		double diff = 0.0;

		double sim = 0.0;
		if (v1 < v2) {
			diff = v2 - v1;
		} else {
			diff = v1 - v2;
		}

		double abs1 = Math.abs(v1);
		double abs2 = Math.abs(v2);

		if (abs1 > abs2) {
			if (abs1 < 0.0001 && abs1 > -0.0001) {
				sim = 1.0;
			} else {
				sim = 1.0 - (diff / abs1);
			}
		} else {
			if (abs2 < 0.0001 && abs2 > -0.0001) {
				sim = 1.0;
			} else {
				sim = 1.0 - (diff / abs2);
			}
		}

		if (sim < 0.0) {
			sim = 0.0;
		}

		return sim;
	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.numeric;
	}

	@Override
	public double getEquivalenceThreshold() {
		return similarityThreshold.get();
	}
}
