package de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching;

import java.util.concurrent.ConcurrentHashMap;

import com.wcohen.ss.Levenstein;

import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.StringSimilarity.StringPair;
import de.mannheim.uni.similarity.functions.SimilarityFunction;

public class LevelSteinSimilarityWithCache extends SimilarityFunction<String> {

	private final ThreadLocal<Levenstein> levenSteins = new ThreadLocal<>();

	private static final ConcurrentHashMap<StringPair, Double> levensteinCache = new ConcurrentHashMap<>();

	public Double calculate(String first, String second) {

		if (first == null || second == null) {
			return null;
		} else {

			if (first.toLowerCase().contains("#")) {
				first = first.split("#")[1];
			}
			if (second.toLowerCase().contains("#")) {
				second = second.split("#")[1];
			}

			if (first.toLowerCase().contains("label") && !first.equals("label")) {
				first = first.toLowerCase().replace("label", "");
			}
			if (second.toLowerCase().contains("label") && !second.equals("label")) {
				second = second.toLowerCase().replace("label", "");
			}

			if (first.toLowerCase().contains("name") && !first.toLowerCase().equals("name")) {
				first = first.toLowerCase().replace("name", "");
			}
			if (second.toLowerCase().contains("name") && !second.toLowerCase().equals("name")) {
				second = second.toLowerCase().replace("name", "");
			}

			if (first.toLowerCase().contains("has") && !first.toLowerCase().equals("has")) {
				first = first.replace("has", "");
			}
			if (second.toLowerCase().contains("has") && !second.toLowerCase().equals("has")) {
				second = second.toLowerCase().replace("has", "");
			}

			return getLevenstein(first, second);
		}
	}

	private double getLevenstein(String first, String second) {
		StringPair sp = new StringPair(first, second);
		Double sim = levensteinCache.get(sp);
		if (sim == null) {

			Levenstein l = levenSteins.get();
			if (l == null) {
				l = new Levenstein();
				levenSteins.set(l);
			}

			double score = l.score(first, second);
			score = score / Math.max(first.length(), second.length());
			if (score < 0) {
				score = score * -1;
			}
			score = 1 - score;

			sim = score;
			levensteinCache.put(sp, sim);
		}
		return sim;
	}
}
