package de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityMeasures;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThreshold;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThresholdsRepository;
import de.mannheim.uni.matching.ValueRange;
import de.mannheim.uni.similarity.functions.date.WeightedDatePartSimilarity;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MatchingDateSimilarity extends AbstractSimilarity {

	private final WeightedDatePartSimilarity sim;

	private final SimilarityThreshold similarityThreshold;

	@Inject
	public MatchingDateSimilarity(SimilarityThresholdsRepository similarityThresholdsRepository) {
		sim = new WeightedDatePartSimilarity(1, 3, 5);
		DateFormat formatter = new SimpleDateFormat("yyyy");
		try {
			Date date1 = formatter.parse("0000");

			Date date2 = formatter.parse("2030");

			sim.setValueRange(new ValueRange(date1, date2));
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		this.similarityThreshold = similarityThresholdsRepository.get(SimilarityMeasures.MATCHING_DATE);
	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected double normalSimilarity(Object a, Object b) {

		if (a == null || b == null) {
			return 0.0;
		} else if (a.equals(b)) {
			return 1.0;
		} else if (!(a instanceof Date) || !(b instanceof Date)) {
			return 0.0;
		} else {
			double simValue = sim.calculate((Date) a, (Date) b);
			return simValue;

		}

	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.date;
	}

	@Override
	public double getEquivalenceThreshold() {
		return similarityThreshold.get();
	}

}
