package de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityMeasures;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThreshold;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThresholdsRepository;
import de.mannheim.uni.similarity.functions.QuadraticSimilarityDecorator;
import de.mannheim.uni.similarity.functions.numeric.DeviationSimilarity;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MatchingNumericSimilarity extends AbstractSimilarity {

	private final QuadraticSimilarityDecorator<Double> sim;

	private final SimilarityThreshold similarityThreshold;

	@Inject
	public MatchingNumericSimilarity(SimilarityThresholdsRepository similarityThresholdsRepository) {
		sim = new QuadraticSimilarityDecorator<Double>(new DeviationSimilarity());
		this.similarityThreshold = similarityThresholdsRepository.get(SimilarityMeasures.MATCHING_NUMERIC);
	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected double normalSimilarity(Object a, Object b) {

		if (a == null || b == null) {
			return 0.0;
		} else if (a.equals(b)) {
			return 1.0;
		} else if (!(a instanceof Double) || !(b instanceof Double)) {
			return 0.0;
		} else {
			return sim.calculate((Double) a, (Double) b);

		}

	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.numeric;
	}

	@Override
	public double getEquivalenceThreshold() {
		return similarityThreshold.get();
	}

}
