package de.mannheim.uni.expansion.tablefusion.core.similarity.measures.matching;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.AbstractSimilarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityMeasures;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThreshold;
import de.mannheim.uni.expansion.tablefusion.core.similarity.thresholds.SimilarityThresholdsRepository;
import de.mannheim.uni.similarity.functions.string.GeneralisedStringJaccard;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MatchingStringSimilarity extends AbstractSimilarity {

	private final GeneralisedStringJaccard sim;

	private final SimilarityThreshold similarityThreshold;

	@Inject
	public MatchingStringSimilarity(SimilarityThresholdsRepository similarityThresholdsRepository) {
		sim = new GeneralisedStringJaccard(new LevelSteinSimilarityWithCache(), 0.5, 0.5);
		this.similarityThreshold = similarityThresholdsRepository.get(SimilarityMeasures.MATCHING_STRING);
	}

	@Override
	public double getQuickSimilarity(Object a, Object b) {
		return normalSimilarity(a, b);
	}

	@Override
	protected double normalSimilarity(Object a, Object b) {

		if (a == null || b == null) {
			return 0.0;
		} else if (a.equals(b)) {
			return 1.0;
		} else if (!(a instanceof String) || !(b instanceof String)) {
			return 0.0;
		} else {
			return sim.calculate((String) a, (String) b);

		}

	}

	@Override
	protected ColumnDataType getColumnDataType() {
		return ColumnDataType.string;
	}

	@Override
	public double getEquivalenceThreshold() {
		return similarityThreshold.get();
	}

}
