package de.mannheim.uni.expansion.tablefusion;

import java.io.IOException;
import java.io.PrintStream;
import java.rmi.registry.Registry;
import java.util.Map;
import java.util.Scanner;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;
import javax.inject.Named;

import dagger.ObjectGraph;
import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.dbpedia.adapter.csv.DBPediaRepositoryCSVWriter;
import de.mannheim.uni.expansion.data.dbpedia.adapter.csv.DBpediaRepositoryCSVAdapter;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNew;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.HostId;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParser;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesGzReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesWriter;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.data.tables.repository.memory.AnnotatedTablesRepositoryImplMemoryFactory;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.ColumnMatchesWriter;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankCSVAdapter;
import de.mannheim.uni.expansion.tablefusion.analysis.AnnotatedInputProfiler;
import de.mannheim.uni.expansion.tablefusion.analysis.DistributionAnalyzer;
import de.mannheim.uni.expansion.tablefusion.analysis.GroupAnalyzer;
import de.mannheim.uni.expansion.tablefusion.analysis.MaxCorrectFinder;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.InputProfileStoreCreator.InputProfileStore;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.NumericSimilarity;
import de.mannheim.uni.expansion.tablefusion.evaluation.CountTypes;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationExporter;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationExporterRandom;
import de.mannheim.uni.expansion.tablefusion.evaluation.ProblemAnalyzer;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.AccuracyAnalyzer;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.FusionAnalyzer;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.ScoreOutputter;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.optimization.ThresholdExporter;
import de.mannheim.uni.expansion.tablefusion.optimization.ThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.profiling.timestamps.TimeStampProfiler;
import de.mannheim.uni.expansion.tablefusion.profiling.timestamps.TimeStampQualityProfiler;
import de.mannheim.uni.expansion.tablefusion.testers.FusionTester;
import de.mannheim.uni.expansion.utils.containers.Pair;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;
import de.mannheim.uni.units.UnitParserDomi;
import de.uni_mannheim.informatik.dws.tablefusion.matching.SingleMatchingService;
import de.uni_mannheim.informatik.dws.tablefusion.matching.SingleMatchingService2;

/**
 * Hello world!
 *
 */
public class TableFusionApp {

	AccuracyAnalyzer accuracyAnalyzer;

	FusionAnalyzer fusionAnalyzer;

	ProblemAnalyzer problemAnalzyer;

	ColumnMatchesWriter columnMatcherWriter;

	RankCSVAdapter rankCSVAdapter;

	@Named(NamedProperties.HOST_RANKINGS_FILE)
	@Inject
	String hostRankingsFile;
	@Named(NamedProperties.HOST_RANKINGS_FILE_FILTERED)
	@Inject
	String hostRankingsFileFiltered;

	@Inject
	@Named(DataNamedProperties.BAD_ANNOTATED_TABLES_FILE)
	String badAnnotatedTablesFile;

	@Inject
	@Named(DataNamedProperties.DBPEDIA_REPOSITORY_PATH)
	String dbPediaRepositoryPath;

	@Inject
	@Named(DataNamedProperties.TABLES_OUTPUT_FOLDERS)
	String tablesOutputFolders;

	static String badTablesFolder;

	AnnotatedTableParser annotatedTableParser;

	AnnotatedTablesReader annotatedTablesReader;

	PrintStream out;

	DBpediaRepositoryCSVAdapter csvAdapter;

	DBpediaRepositoryNew dBpediaRepositoryNew;

	AnnotatedTablesRepository annotatedTablesRepository;

	DBPediaRepositoryCSVWriter csvWriter;

	AnnotatedTablesWriter annotatedTablesWriter;

	AnnotatedInputProfiler annotatedInputProfiler;

	CountTypes countTypes;

	ThresholdExporter thresholdFinder;

	DistributionAnalyzer distributionAnalyzer;
	
	MaxCorrectFinder maxCorrectFinder;
	
	ScoreFusionThresholdFinder scoreFusionThresholdFinder;
	
	EvaluationExporterRandom evaluationExporterRandom;

	NumericSimilarity numericSimilarity;
	
	ScoreOutputter scoreOutputter;
	
	@Inject
	@Named(DataNamedProperties.TABLES_MAPPING_FOLDER)
	String mappingsFolder;

	@Inject
	@Named(DataNamedProperties.TABLES_MAPPING_ENDING)
	FileEnding tablesMappingsEnding;
	
	@Inject
	@Named(DataNamedProperties.TABLES_TYPE)
	TableType tableType;

	@Inject
	@Named(NamedProperties.DATASETS_DIRECTORY)
	String datasetsDirectory;

	private boolean tablesLoaded;

	private Registry registry;

	private EvaluationExporter evaluationExporter;

	private GroupAnalyzer groupAnalyzer;

	@Inject
	public TableFusionApp() {
		out = System.out;
		System.setOut(System.err);
	}

	public void run(int option, TableFusionModule tableFusionModule, DBpediaModule dBpediaModule, TablesModule tablesModule, OntologyModule ontologyModule,
			HeidelTimeModule heidelTimeModule, ObjectGraph objectGraph) throws Exception {

		switch (option) {
		case 1:
			out.println("Running Column Matching and storing the data");
			if (columnMatcherWriter == null)
				columnMatcherWriter = objectGraph.get(ColumnMatchesWriter.class);
			columnMatcherWriter.run();
			break;
		case 2:
			out.println("Running Fusion Accuracy calculator");
			if (accuracyAnalyzer == null)
				accuracyAnalyzer = objectGraph.get(AccuracyAnalyzer.class);
			accuracyAnalyzer.getPerformance("");
			// accuracyAnalyzer.getPerformanceWithLists();
			break;
		case 3:
			out.println("Running Problem Analyzer");
			if (problemAnalzyer == null)
				problemAnalzyer = objectGraph.get(ProblemAnalyzer.class);
			problemAnalzyer.run();
			break;
		case 4:
			out.println("Creating Host Ranking Subset");
			if (rankCSVAdapter == null)
				rankCSVAdapter = objectGraph.get(RankCSVAdapter.class);
			Map<HostId, Double> rr = rankCSVAdapter.read(hostRankingsFile, true);

			rankCSVAdapter.write(hostRankingsFileFiltered, rr);
			break;

		case 5:
			out.println("Running Fusion Accuracy calculator");
			if (accuracyAnalyzer == null)
				accuracyAnalyzer = objectGraph.get(AccuracyAnalyzer.class);
			accuracyAnalyzer.getPerformanceWithLists();

			break;

		case 6:
			out.println("Export bad Tables");

			if (annotatedTableParser == null)
				annotatedTableParser = objectGraph.get(AnnotatedTableParser.class);

			AnnotatedTablesReader reader = new AnnotatedTablesGzReader(badTablesFolder, FileEnding.NO_URI_GZ, tableType, annotatedTableParser, badAnnotatedTablesFile);

			AnnotatedTablesRepositoryImplMemoryFactory factory = new AnnotatedTablesRepositoryImplMemoryFactory(reader, false, null, null);
			factory.getRepository();

			break;

		case 7:
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			objectGraph.get(DBpediaRepositoryNewInterface.class);

			break;
		case 8:

			if (dBpediaRepositoryNew == null) {
				out.println("You need to load DBPedia locally before being able to convert it!");
				return;
			}
			if (csvWriter == null)
				csvWriter = objectGraph.get(DBPediaRepositoryCSVWriter.class);

			try {
				csvWriter.writeValuesNew(dBpediaRepositoryNew, dbPediaRepositoryPath);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			break;
		case 9: {
			AnnotatedTablesRepository annotatedTablesRepository = objectGraph.get(AnnotatedTablesRepository.class);

			if (annotatedTablesWriter == null)
				annotatedTablesWriter = objectGraph.get(AnnotatedTablesWriter.class);

			annotatedTablesWriter.writeAll(annotatedTablesRepository);
			break;

		}
		case 10:

			if (dBpediaRepositoryNew == null) {
				out.println("You need to load DBPedia locally before being able to convert it!");
				return;
			}
			if (csvWriter == null)
				csvWriter = objectGraph.get(DBPediaRepositoryCSVWriter.class);

			try {
				csvWriter.writeInstanceIndicesNew(dBpediaRepositoryNew, dbPediaRepositoryPath);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			break;
		case 11:

			out.println("Counting and Printing out Data Types in Web Tables");
			if (countTypes == null)
				countTypes = objectGraph.get(CountTypes.class);
			countTypes.printOutNumbers();
			break;
		case 12:
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setLoadFromCache(true);
			tablesModule.setFilter(true);
			break;
		case 13:
			out.println("Running for multiple Probabilities");
			// TODO: 
//			if (accuracyAnalyzer == null)
//				accuracyAnalyzer = objectGraph.get(AccuracyAnalyzer.class);
//			accuracyAnalyzer.runAllProbabilities();
			break;

		case 14:
			out.println("Calculating Table Statistics (Duplicate and number of Mappings)");

			if (annotatedTableParser == null)
				annotatedTableParser = objectGraph.get(AnnotatedTableParser.class);

			AtomicInteger duplicateTables = new AtomicInteger(0);
			AtomicInteger allTables = new AtomicInteger(0);

			ConcurrentHashMap<AnnotatedTableId, Boolean> tableIds = new ConcurrentHashMap<AnnotatedTableId, Boolean>();
			AnnotatedTablesGzReader gzReader = new AnnotatedTablesGzReader(mappingsFolder, tablesMappingsEnding, tableType, annotatedTableParser, null);
			gzReader.runForEachTableMetaData((e) -> {
				allTables.incrementAndGet();
				if (tableIds.putIfAbsent(e.getId(), true) != null) {
					duplicateTables.incrementAndGet();
				} else {

				}
			});

			System.out.println("Number of all Tables: " + allTables.get());
			System.out.println("Number of duplicate tables: " + duplicateTables.get());
			System.out.println("Number of mapped Properties: " + annotatedTableParser.getColumnCount());
			break;

		case 15:
			out.println("Calculating Table Statistics (except duplicates)");

			if (annotatedTablesRepository == null) {
				out.println("You need to load the annotated tables locally before being able to calculate statistics!");
				return;
			}
			annotatedTablesRepository.printStatistics();

			break;
		case 16:

			out.println("Writing DBpedia List Types");

			if (dBpediaRepositoryNew == null) {
				out.println("You need to load DBPedia locally before being able to list types!");
				return;
			}
			if (csvWriter == null)
				csvWriter = objectGraph.get(DBPediaRepositoryCSVWriter.class);

			try {
				csvWriter.writeListTypes(dBpediaRepositoryNew, dbPediaRepositoryPath);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			break; 
		case 17:

			if (annotatedInputProfiler == null)
				annotatedInputProfiler = objectGraph.get(AnnotatedInputProfiler.class);

			InputProfileStore profiledData = annotatedInputProfiler.getProfiled();
			try {
				annotatedInputProfiler.write(profiledData);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			break;

		case 18:
			out.println("Outputting Scores");
			if (scoreOutputter == null)
				scoreOutputter = objectGraph.get(ScoreOutputter.class);
			// scoreOutputter.outputScores();

			break;
		case 19:
			out.println("Outputting unique Mapped values");

			if (groupAnalyzer == null)
				groupAnalyzer = objectGraph.get(GroupAnalyzer.class);
			groupAnalyzer.printUniqueMatchedValues();

			break;
		case 20: {
	 
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setLoadFromCache(true);
			
			objectGraph.get(AnnotatedTablesRepository.class);

				tablesLoaded = true;

				// } else {
				// out.println("TABLES ALRADY LOADED");
				// } 
			break;
		}
		case 21: {
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setLoadFromCache(false);
			
			objectGraph.get(AnnotatedTablesRepository.class);

				tablesLoaded = true;

			break;
		}
		case 22: {
			out.println("Running Problem Analyzer for Columns");
			if (problemAnalzyer == null)
				problemAnalzyer = objectGraph.get(ProblemAnalyzer.class);
			try {
				System.out.println(problemAnalzyer.getColumnScores());
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			break;
		}
		case 23: {
			out.println("Store problematic Columns");
			if (problemAnalzyer == null)
				problemAnalzyer = objectGraph.get(ProblemAnalyzer.class);
			try {
				problemAnalzyer.storeProblematicColumns();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			break;
		}
		case 24:

			out.println("Writing DBpedia Instance Indices Per Class Property");

			if (dBpediaRepositoryNew == null) {
				out.println("You need to load DBPedia locally before being able to store instance indices!");
				return;
			}
			if (csvWriter == null)
				csvWriter = objectGraph.get(DBPediaRepositoryCSVWriter.class);

			try {
				csvWriter.writeInstancesPerClassProperty(dBpediaRepositoryNew, dbPediaRepositoryPath);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			break;

		case 25:
			out.println("Export Threshold Tables");
			if (thresholdFinder == null)
				thresholdFinder = objectGraph.get(ThresholdExporter.class);
			try {
				thresholdFinder.exportThresholdData();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			break;
		case 26:
			out.println("Test Numeric Similarty");
			if (numericSimilarity == null)
				numericSimilarity = objectGraph.get(NumericSimilarity.class);

			System.out.println(numericSimilarity.get(189.0, 0.0));

			break;

		case 27:
			out.println("Find Threshold");
			ThresholdFinder thresholdFinder = objectGraph.get(ThresholdFinder.class);
			try {
				thresholdFinder.findThreshold();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			break;


		case 29:
			out.println("Analyze and Print distribution");
			if (distributionAnalyzer == null)
				distributionAnalyzer = objectGraph.get(DistributionAnalyzer.class);

			distributionAnalyzer.printTripleSourceDistribution();

			break;
		case 30:
			out.println("Print Max Correct Values in Corpus");
			if (maxCorrectFinder == null)
				maxCorrectFinder = objectGraph.get(MaxCorrectFinder.class);

			maxCorrectFinder.printMaxCorrectValuesInCorpus(false);

			break;
		case 31:
			// out.println("Find Threshold for Column Scores");
			// if (scoreFusionThresholdFinder == null)
			// scoreFusionThresholdFinder =
			// objectGraph.get(ScoreFusionThresholdFinder.class);
			//
			// scoreFusionThresholdFinder.findOptimumThresholdForColumnScores();

			break;
		case 32:
			out.println("Export Randomized Evaluation");
			if (evaluationExporterRandom == null)
				evaluationExporterRandom = objectGraph.get(EvaluationExporterRandom.class);
			// try {
			// evaluationExporterRandom.exportThresholdDataRandomColumnThresholded();
			// } catch (IOException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }

			break;

		case 33:
			out.println("Find Threshold for PageRank");
			if (scoreFusionThresholdFinder == null)
				scoreFusionThresholdFinder = objectGraph.get(ScoreFusionThresholdFinder.class);

			scoreFusionThresholdFinder.findOptimumThresholdForPageRank();

			break;

		case 34:
			out.println("Output Full performance (Precision, Recall, F-Measure)");
			if (distributionAnalyzer == null)
				distributionAnalyzer = objectGraph.get(DistributionAnalyzer.class);

			// distributionAnalyzer.performanceUsingCrossValidation();

			break;

		case 35:
			out.println("Group Analyzer");
			if (groupAnalyzer == null)
				groupAnalyzer = objectGraph.get(GroupAnalyzer.class);

			// try {
			// // groupAnalyzer.run();
			// } catch (IOException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }

			break;
		case 36: {
			// dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			// tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			// tablesModule.setLoadFromCache(true);
			// tablesModule.setFilter(true);
			//
			// objectGraph.get(AnnotatedTablesRepository.class);
			//
			// tablesLoaded = true;
			//
			//
			break;
		}
		case 37: {
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setLoadFromCache(true);
			tablesModule.setFilter(true);

			objectGraph.get(AnnotatedTablesRepository.class);

				tablesLoaded = true;
		 
		break;

	}
		case 38: {
			 

			TimeStampProfiler profiler = objectGraph.get(TimeStampProfiler.class);
			profiler.profileTimeStamps();
		 
		break;
		}
		case 39: {
			 

			TimeStampQualityProfiler profiler = objectGraph.get(TimeStampQualityProfiler.class);
			profiler.analyze();
		 
		break;
		}
		case 40: {
			dBpediaModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);

			DBpediaRepositoryNewInterface dbpedia = objectGraph.get(DBpediaRepositoryNewInterface.class);

			tablesModule.setRepositoryLoading(RepositoryLoading.REMOTE_LOAD_LOCALLY);
			tablesModule.setLoadFromCache(true);
			tablesModule.setClassFilter(dbpedia);

			objectGraph.get(AnnotatedTablesRepository.class);

			break;
		}

		case 41: {
			FusionTester fusionTester = objectGraph.get(FusionTester.class);

			// fusionTester.test();

			break;
		}

		case 42: {
			SingleMatchingService2 sms = objectGraph.get(SingleMatchingService2.class);
			// sms.matchTable(datasetsDirectory, "nfl2010.csv");
			// sms.matchTable(datasetsDirectory, "nfl2012.csv");
			// sms.matchTable(datasetsDirectory, "nfl2014.csv");
			// sms.matchTable(datasetsDirectory, "nflMixed.csv");

			// sms.matchTable(datasetsDirectory, "cities2008.csv");
			// sms.matchTable(datasetsDirectory, "cities2014.csv");
			// sms.matchTable(datasetsDirectory, "fortune2014.csv");
			sms.matchTable(datasetsDirectory, "citiesMixed.csv");

			break;
		}
		case 43: {
			SingleMatchingService sms = objectGraph.get(SingleMatchingService.class);
			// sms.matchColumns(datasetsDirectory, "countries2014.csv");
			sms.matchColumns(datasetsDirectory, "cities2008.csv");
			sms.matchColumns(datasetsDirectory, "cities2014.csv");
			sms.matchColumns(datasetsDirectory, "citiesMixed.csv");
			// sms.matchColumns(datasetsDirectory, "nflMixed.csv");
			break;
		}
		default:
			out.println("Option not recognized! Please try again");
			break;
		}
	}


	private TableFusionModule tableFusionModule;
	private DBpediaModule dBpediaModule;
	private TablesModule tablesModule;
	private OntologyModule ontologyModule;
	private HeidelTimeModule heidelTimeModule;
	private MatchingModule matchingModule;
	private RankingModule rankingModule;
	private RegressionModule regressionModule;
	private ObjectGraph objectGraph;

	public void run(TableFusionModule tableFusionModule, DBpediaModule dBpediaModule, TablesModule tablesModule, OntologyModule ontologyModule,
			HeidelTimeModule heidelTimeModule, MatchingModule matchingModule, RankingModule rankingModule, RegressionModule regressionModule,
			ObjectGraph objectGraph) {
		this.tableFusionModule = tableFusionModule;
		this.dBpediaModule = dBpediaModule;
		this.tablesModule = tablesModule;
		this.ontologyModule = ontologyModule;
		this.heidelTimeModule = heidelTimeModule;
		this.matchingModule = matchingModule;
		this.regressionModule = regressionModule;
		this.objectGraph = objectGraph;

		Scanner in = new Scanner(System.in);
		System.err.flush();

		try {
			while (true) {

				out.println("=================================================");
				out.println("FUUUU SIOOON - HAAAAAA");
				out.println("=================================================");
				out.println(" |=|  Please Select");
				out.println(" |=|  --");
				out.println(" |=|  [1] Run Column Matching");
				out.println(" |=|  [2] Caculate Fusion Accuracy");
				out.println(" |=|  [3] Analyze and store Problems");
				out.println(" |=|  [4] Create Host-Rankings Subset");
				out.println(" |=|  [5] Caculate Fusion Accuracy With Lists");
				out.println(" |=|  [6] Export bad Tables");
				out.println(" |=|  [7] Load Data into memory and share on RMI");
				out.println(" |=|  [8] Convert and Store DBPedia value type");
				out.println(" |=|  [9] Convert and Store Table value type");
				out.println(" |=|  [10] Convert and Store DBPedia instance indices");
				out.println(" |=|  [11] Count and Print out Data Types for Web Tables");
				out.println(" |=|  [12] Set Load DBPedia and Tables locally");
				out.println(" |=|  [13] Run for multiple Probabilities");
				out.println(" |=|  [14] Count Column Properties (GZ - Only)");
				out.println(" |=|  [15] Calculate Table Statistics");
				out.println(" |=|  [16] Writing DBpedia List Types");
				out.println(" |=|  [17] Profile input data and store results");
				out.println(" |=|  [18] Output and Write Scores");
				out.println(" |=|  [19] Print unique matched values");
				out.println(" |=|  [20] Load Tables From Kryo");
				out.println(" |=|  [21] Load and Parse Tables From GZ");
				out.println(" |=|  [22] Print out column scores");
				out.println(" |=|  [23] Store problematic columns");
				out.println(" |=|  [24] Writing DBpedia Instance Indices Per Class Property");
				out.println(" |=|  [25] Export Thresholds");
				out.println(" |=|  [26] Test Numeric Similarity");
				out.println(" |=|  [27] Find Threshold");
				out.println(" |=|  [29] Print Source Table Distribution");
				out.println(" |=|  [30] Print Max Correct Values in Corpus");
				out.println(" |=|  [31] EMPTY (Find Threshold for Column Scores)");
				out.println(" |=|  [32] Export Random Evaluation");
				out.println(" |=|  [33] Find Threshold for Page Rank");
				out.println(" |=|  [34] Output Performance Using Cross Validation");
				out.println(" |=|  [35] Exports Group Analysis");
				out.println(" |=|  [36] Load Tables From Kryo without References");
				out.println(" |=|  [37] Load Tables From Kryo with filtering");
				out.println(" |=|  [38] Profile Timestamps");
				out.println(" |=|  [39] Check Quality of Timestamps");
				out.println(" |=|  [40] Read Kryo with tables filtered to subset");
				out.println(" |=|  [41] FusionTester");
				out.println(" |=|  --");
				out.println(" |=|  [0] Exit");
				out.println("=================================================");

				out.println();

				out.print("Your Input: ");
				out.flush();

				if (in.hasNextInt()) {
					int a = in.nextInt();
 
					if (a == 0)
						break;
					else {
						try {
							run(a, tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, objectGraph);
						} catch (Exception e) {
							e.printStackTrace();
						}

					}

				} else {

					out.println("Sorry, couldn't understand you!");
					in.next();
				}

			}
		} finally {
			in.close();
		}
		// fusionAnalyzer.printDensityDistribution(dbpediaData, FileEnding.NONE,
		// annotatedTableData);

		//

	}

	public static Pair<String, TableType> getDataSet(PrintStream out) {

		Scanner in = new Scanner(System.in);
		try {
			while (true) {

				out.println("=================================================");
				out.println("CHOOSE YOUR DESTINY (DATASET)");
				out.println("=================================================");
				out.println(" |=|  Please Select");
				out.println(" |=|  --");
				out.println(" |=|  [1] Subset");
				out.println(" |=|  [2] Rerun");
				out.println(" |=|  [3] Matched GoldStandard");
				out.println(" |=|  [4] GoldStandard 2");
				out.println(" |=|  [5] GoldStandard with equivalence");
				out.println(" |=|  [6] LOD2");
				out.println(" |=|  [7] BTC");
				out.println(" |=|  [8] referenceTest");
				out.println(" |=|  [9] v2subset");
				out.println(" |=|  [10] v2");
				out.println(" |=|  --");
				out.println(" |=|  [0] Exit");
				out.println("=================================================");

				out.println();

				out.print("Your Input: ");
				out.flush();

				if (in.hasNextInt()) {
					int a = in.nextInt();

					if (a == 0)
						break;
					else {
						switch (a) {
						case 1: {
							return new Pair<>("subset", TableType.WEB_TABLE);
						}
						case 2: {

							return new Pair<>("rerun", TableType.WEB_TABLE);
						}
						case 3: {

							return new Pair<>("gs", TableType.WEB_TABLE);
						}
						case 4: {

							return new Pair<>("gs2", TableType.WEB_TABLE);
						}
						case 5: {

							return new Pair<>("gseq", TableType.WEB_TABLE);
						}
						case 6: {

							return new Pair<>("LOD2", TableType.LOD_TABLE);
						}
						case 7: {
							return new Pair<>("BTC", TableType.LOD_TABLE);
						}
						case 8: {
							return new Pair<>("referenceTest", TableType.WEB_TABLE);
						}
						case 9: {
							return new Pair<>("v2sample", TableType.JSON_TABLE);
						}
						case 10: {
							return new Pair<>("v2", TableType.JSON_TABLE);
						}
						}

					}

				} else {

					out.println("Sorry, couldn't understand you!");
					in.next();
				}

			}
		} finally {
			in.reset();
		}

		return null;
		// fusionAnalyzer.printDensityDistribution(dbpediaData, FileEnding.NONE,
		// annotatedTableData);

		//

	}

	public static void main(String[] args) {
		System.setProperty("java.net.preferIPv4Stack", "true");
		// System.setProperty("java.rmi.server.hostname",
		// "wifo5-09.informatik.uni-mannheim.de");

		// try {
		// PrintStream out;
		// out = new PrintStream(new FileOutputStream("output.txt"));
		// System.setOut(out);
		// } catch (FileNotFoundException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// } 

//		 Parallel.SetDefaultNumProcessors(1);
//		 PersistentParallel.overideNumberOfProcessors(1);

		String dataHome = "C:/Users/yaser/dev/data";

		if (args.length > 0 && args[0] != null && !args[0].equals("")) {
			dataHome = args[0];

		}

		if (args.length > 2 && args[2] != null && !args[2].equals("")) {
			System.setProperty("java.rmi.server.hostname", args[2]);
		}

		String dbpediaRepository = dataHome + "/repository_subset/";
		String dbpediaFile = dataHome + "/serialised_kryo_subset.bin";

		FileEnding dbpediaFileEnding = FileEnding.CSV;
		String keyIndexPath = dataHome + "/index_subset";

		if (args.length > 1 && args[1] != null && args[1].equals("full")) {
			dbpediaRepository = dataHome + "/repository/";
			dbpediaFile = dataHome + "/serialised_kryo.bin";
			dbpediaFileEnding = FileEnding.CSV_GZ;
			keyIndexPath = dataHome + "/index_full";

			System.out.println("Reading Full Dataset");
		}

		Pair<String, TableType> dataSetData = getDataSet(System.out);
		
		if (dataSetData == null)
			return;

		String dataSet = dataSetData.getFirst();
		TableType tableType = dataSetData.getSecond();

		String mappingFolder = dataHome + "/tables/" + dataSet + "/mappings";
		String mappingType = "gz";
		FileEnding tablesFileEnding = FileEnding.NO_URL;

		String learningSetsOutputFolder = dataHome + "/tables/" + dataSet + "/learningSets";
		String columnMappingsFolder = dataHome + "/tables/" + dataSet + "/column_mappings";

		String hostRankingsFileFiltered = dataHome + "/tables/" + dataSet + "/rankings/filtered.tsv.gz";

		String tablesOutputFolders = dataHome + "/tables/" + dataSet + "/repository";

		String scoresFolder = dataHome + "/tables/" + dataSet + "/scores";

		String weightsFolder = dataHome + "/tables/" + dataSet + "/weights";

		String problemsFolder = dataHome + "/tables/" + dataSet + "/problems";

		String candidateSelectionConfig = dataHome + "/candindateSelection.config";
		String matchingConfig = dataHome + "/matching.config";

		String emptyFile = dataHome + "/empty.txt";
		String surfaceForms = dataHome + "/surfaceForms";
		String redirect = dataHome + "/redirects";
		String equivalentProperties = dataHome + "/equivalentProperties.tsv";

		String optimizingFolder = dataHome + "/optimizing";

		// // tablesOutputFolders = dataHome +
		// // "/tables_repository_subset_with_lists";
		// if (args.length > 1 && args[1] != null && args[1].equals("full")) {
		//
		//
		// mappingFolder = dataHome + "/tables/" + dataSet + "/mappings";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;
		//
		// learningSetsOutputFolder = dataHome + "/learningSets_full/" +
		// dataSet;
		//
		// columnMappingsFolder = dataHome + "/column_mappings/" + dataSet;
		//
		// hostRankingsFileFiltered = dataHome +
		// "/rankings/filtered_full.tsv.gz";
		//
		// tablesOutputFolders = dataHome + "/tables/" + dataSet +
		// "/repository";
		//
		// scoresFolder = dataHome + "/scores_full/" + dataSet;
		//
		// weightsFolder = dataHome + "/weights_full/" + dataSet;
		//
		// System.out.println("Reading Full Dataset");
		// }

		TableFusionApp.badTablesFolder = dataHome + "/completeWithSources";

		String propertyTypesFile = dataHome + "/proeprty_types_kryo.bin";
		String ontologyOwlFile = dataHome + "/dbpedia_2014.owl";
		
		String resultsFolder = dataHome + "/tables/" + dataSet + "/results";


		String badAnnotatedTablesFile = dataHome + "/bad_tables.tar.gz";

		String hostRankingsFile = dataHome + "/rankings/hostgraph-pr.tsv.gz";

		String inputProfilingFolder = dataHome + "/results/inputProfiling";

		mappingType = "kryo";
		String uriIndexPath = "C:/Users/yaser/dev/data/index";

		// mappingFolder = dataHome + "/completeWithSourcesNew";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;

		UnitParserDomi.unitPath = dataHome + "/Units/Convertible";
		TableColumnBuilder.computeStatistiscs = false;
		

		String heidelTimeConfigPath = dataHome + "/heidelTime/config.props";

		String datasetsDirectory = dataHome + "/datasets/";

		// mappingFolder = dataHome + "/tableMappingsRerun";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;
		//
		// mappingFolder = dataHome + "/completeWithSources";
		// mappingType = "gz";
		// tablesFileEnding = FileEnding.NO_URI_GZ;
		// // tablesFileEnding = FileEnding.NO_URL;

		boolean useUnits = true;

		TableFusionModule tableFusionModule = new TableFusionModule(resultsFolder, 
				problemsFolder,
 learningSetsOutputFolder, inputProfilingFolder,
				optimizingFolder, datasetsDirectory);
		
		DBpediaModule dBpediaModule = new DBpediaModule(dbpediaRepository, dbpediaFileEnding, RepositoryLoading.REMOTE_LOAD_REMOTELY);
		
		TablesModule tablesModule = new TablesModule(mappingFolder, tablesOutputFolders, RepositoryLoading.REMOTE_LOAD_REMOTELY, true, tableType,
				tablesFileEnding, badAnnotatedTablesFile, false);

		OntologyModule ontologyModule = new OntologyModule(ontologyOwlFile);

		HeidelTimeModule heidelTimeModule = new HeidelTimeModule(heidelTimeConfigPath);
		
		MatchingModule matchingModule = new MatchingModule(useUnits, keyIndexPath, columnMappingsFolder, candidateSelectionConfig, matchingConfig, emptyFile,
				surfaceForms,
				redirect, equivalentProperties);

		RankingModule rankingModule = new RankingModule(hostRankingsFile, hostRankingsFileFiltered);

		RegressionModule regressionModule = new RegressionModule(scoresFolder, weightsFolder);

		SimilarityModule similarityModule = new SimilarityModule();

		ObjectGraph objectGraph = ObjectGraph.create(tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, matchingModule,
				rankingModule, regressionModule, similarityModule);


		TableFusionApp tableFusionApp = objectGraph.get(TableFusionApp.class);
		tableFusionApp.run(tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, matchingModule, rankingModule, regressionModule,
				objectGraph);
	}
}
