package de.mannheim.uni.expansion.tablefusion;

import javax.inject.Named;
import javax.inject.Singleton;

import com.couchbase.client.java.Bucket;

import dagger.Module;
import dagger.Provides;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParser;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesGzReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesKryoReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesWriter;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.ColumnMatchesWriter;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankCSVAdapter;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankRepositoryImpl;
import de.mannheim.uni.expansion.tablefusion.analysis.AnnotatedInputProfiler;
import de.mannheim.uni.expansion.tablefusion.analysis.DistributionAnalyzer;
import de.mannheim.uni.expansion.tablefusion.analysis.GroupAnalyzer;
import de.mannheim.uni.expansion.tablefusion.analysis.MaxCorrectFinder;
import de.mannheim.uni.expansion.tablefusion.core.matching.impl.BasicGroupMatcher;
import de.mannheim.uni.expansion.tablefusion.core.matching.impl.ColumnMatchesGrouper;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.NumericSimilarity;
import de.mannheim.uni.expansion.tablefusion.evaluation.CountTypes;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationExporter;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationExporterRandom;
import de.mannheim.uni.expansion.tablefusion.evaluation.ProblemAnalyzer;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.AccuracyAnalyzer;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinderSimple;
import de.mannheim.uni.expansion.tablefusion.optimization.ThresholdExporter;
import de.mannheim.uni.expansion.tablefusion.optimization.ThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.profiling.timestamps.TimeStampProfiler;
import de.mannheim.uni.expansion.tablefusion.profiling.timestamps.TimeStampQualityProfiler;
import de.mannheim.uni.expansion.tablefusion.services.MagnitudeFixer;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategyRepository;
import de.mannheim.uni.expansion.tablefusion.services.fusion.strategies.ScorerRepository;
import de.mannheim.uni.expansion.tablefusion.services.grouping.BasicMatcherService;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.tablefusion.testers.FusionTester;
import de.mannheim.uni.expansion.utils.timestamps.HeidelTimeWrapper;
import de.uni_mannheim.informatik.dws.tablefusion.matching.SingleMatchingService;
import de.uni_mannheim.informatik.dws.tablefusion.matching.SingleMatchingService2;

@Module(injects = { DBpediaRepositoryNewInterface.class, AnnotatedTablesRepository.class, OntologyRepository.class, HeidelTimeWrapper.class,
		RankCSVAdapter.class, AccuracyAnalyzer.class, TableFusionApp.class, ColumnMatchesWriter.class, AnnotatedTableParser.class,
		AnnotatedTablesReader.class, AnnotatedTablesWriter.class, CountTypes.class,
		AnnotatedInputProfiler.class, ProblemAnalyzer.class, AnnotatedTablesGzReader.class, AnnotatedTablesKryoReader.class, MagnitudeFixer.class,
 ThresholdExporter.class, NumericSimilarity.class,
		EvaluationExporter.class, DistributionAnalyzer.class, GroupAnalyzer.class, MaxCorrectFinder.class, EvaluationExporterRandom.class,
 TimeStampProfiler.class, FusionTester.class,
		TimeStampQualityProfiler.class, ScoreFusionThresholdFinder.class, SingleMatchingService.class, SingleMatchingService2.class,
		TimeStampsDataService.class, TimeDependentFusion.class, ThresholdFinder.class, ScoreFusionThresholdFinderSimple.class },

includes = { DBpediaModule.class, TablesModule.class, OntologyModule.class, HeidelTimeModule.class, MatchingModule.class, RankingModule.class,
		RegressionModule.class, SimilarityModule.class },

library = true
// , includes = PumpModule.class
)
public class TableFusionModule {  


	private final String resultsFolder;
	private final String problemsFolder;
	private final String learningSetsOutputFolder;
	private final String inputProfilingFolder;
	private final String optimizingFolder;

	private final String datasetsDirectory;

	public TableFusionModule(String resultsFolder, String problemsFolder, String learningSetsOutputFolder,
 String inputProfilingFolder,
 String optimizingFolder,
			String datasetsDirectory) {

		this.resultsFolder = resultsFolder;
		this.problemsFolder = problemsFolder;
		this.learningSetsOutputFolder = learningSetsOutputFolder;
		this.inputProfilingFolder = inputProfilingFolder;
		this.optimizingFolder = optimizingFolder;
		this.datasetsDirectory = datasetsDirectory;
	}









	@Provides
	@Singleton
	RankRepository loadRankRankings(RankCSVAdapter rankCSVAdapter, @Named(NamedProperties.HOST_RANKINGS_FILE_FILTERED) String filteredRankingFile,
			AnnotatedTablesRepository annotatedTablesRepository) {

		return new RankRepositoryImpl(rankCSVAdapter, filteredRankingFile, annotatedTablesRepository);

	}

	// @Provides
	// AnnotatedTablesReader getReader(AnnotatedTableParser parser) {
	// return new AnnotatedTablesSimpleReader(mappingFolder, parser);
	//
	// }





	@Provides
	@Singleton
	Bucket getBucket() {
		// Cluster cluster = CouchbaseCluster.create();
		// Bucket bucket = cluster.openBucket("default2");
		//
		// Runtime.getRuntime().addShutdownHook(new Thread() {
		// public void run() {
		// bucket.close();
		// cluster.disconnect();
		// }
		// });
		//
		// return bucket;
		return null;
	}

	// @Provides
	// @Singleton
	// KeyIndex getLuceIndex(MatchingData data) {
	// KeyIndex keyIndex = new KeyIndex();
	//
	// IIndex index = new InMemoryIndex();
	// DBpediaIndexer indexer = new DBpediaIndexer();
	//
	// for (Table t : data.getDbpediaTables()) {
	// System.out.println("indexing " + t.getHeader());
	// indexer.indexInstances(index, t);
	// }
	//
	// keyIndex.setLuceneIndex(index);
	// return keyIndex;
	// }



	// @Provides
	// @Singleton
	// IIndex getUriIndex(MatchingData data) {
	// return new DefaultIndex(uriIndexPath);
	// }






	@Provides
	@Named(NamedProperties.RESULTS_FOLDER)
	String getResultsFolder() {
		return this.resultsFolder;
	}

	@Provides
	@Named(NamedProperties.PROBLEMS_FOLDER)
	String getProblemsFolder() {
		return this.problemsFolder;
	}










	@Provides
	@Named(NamedProperties.LEARNING_SETS_OUTPUT_FOLDER)
	String getLearningSetsOutputFolder() {
		return this.learningSetsOutputFolder;
	}

	@Provides
	@Named(NamedProperties.INPUT_PROFILING_FOLDER)
	String getInputProfilingFolder() {
		return this.inputProfilingFolder;
	}




	@Provides
	@Singleton
	public GroupedValuesRepository getGroupedValuesRepository(BasicMatcherService matcherService, ColumnMatchesGrouper columnMatchesGrouper,
			BasicGroupMatcher groupMatcher) {
		return new GroupedValuesRepository(matcherService, columnMatchesGrouper, groupMatcher);
	}

	@Provides
	@Named(NamedProperties.OPTIMIZING_FOLDER)
	String getOptimizingFolder() {
		return this.optimizingFolder;
	}
	
	
	@Provides
	ScoreFusionThresholdFinder finder(ScoreFusionThresholdFinderSimple finder) {
		return finder;
	}

	@Provides
	@Named(NamedProperties.DATASETS_DIRECTORY)
	String getDataSetsDirectory() {
		return this.datasetsDirectory;
	}

	
	@Provides
	@Singleton
	ScorerRepository getScorerRepository(FusionStrategyRepository fusionStrategyRepository) {

		// Field[] declaredFields =
		// FusionStrategyRepository.class.getDeclaredFields();
		// List<Field> staticFields = new ArrayList<Field>();
		// for (Field field : declaredFields) {
		// if (java.lang.reflect.Modifier.isStatic(field.getModifiers()) &&
		// FusionStrategy.class.isAssignableFrom(field.getType())) {
		// FusionStrategy fs = (FusionStrategy) field.get(null);
		// fs.getSc
		//
		// staticFields.add(field);
		// }
		// }
		ScorerRepository repository = new ScorerRepository();
		fusionStrategyRepository.addScorers(repository);
		return repository;
	}

}
