package de.mannheim.uni.expansion.tablefusion;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Map;
import java.util.Scanner;

import javax.inject.Inject;

import dagger.ObjectGraph;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.HostId;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesWriter;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.ColumnMatchesWriter;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankCSVAdapter;
import de.mannheim.uni.expansion.tablefusion.analysis.GroupAnalyzer;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationExporterRandom;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.AccuracyAnalyzer;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;
import de.mannheim.uni.units.UnitParserDomi;

public class TableFusionSample {

	
	public static void main(String[] args) {



		String dataHome = null;

		if (args.length > 0 && args[0] != null && !args[0].equals("")) {
			dataHome = args[0];

		}
		else {
			throw new IllegalStateException("You need to define the dataDir Variable as the first argument of this program, i.e."
					+ "jre/bin/java -Xmx120000M -XX:+UseG1GC de.mannheim.uni.expansion.tablefusion.TableFusionSample \"/home/user/t2k/dataDir/\"");
		}

		if (args.length > 2 && args[2] != null && !args[2].equals("")) {
			System.setProperty("java.rmi.server.hostname", args[2]);
		}

		String dbpediaRepository = dataHome + "/dbpedia/";

		String keyIndexPath = dataHome + "/lucene";

		TableType tableType = TableType.WEB_TABLE;

		String mappingFolder = dataHome + "/tables/mappings";
		FileEnding tablesFileEnding = FileEnding.NO_URL;

		String learningSetsOutputFolder = null;
		String columnMappingsFolder = dataHome + "/tables/columnMappings";

		String hostRankingsFileFiltered = dataHome + "/tables/rankings/filtered.tsv.gz";

		String tablesOutputFolders = dataHome + "/tables/cache";


		String candidateSelectionConfig = dataHome + "/split.config";

		String emptyFile = dataHome + "/empty.txt";
		String surfaceForms = dataHome + "/surfaceForms";
		String redirect = dataHome + "/redirects";
		String equivalentProperties = dataHome + "/equivalentProperties.tsv";

		String resultsFolder = dataHome + "/tables/results";

		String optimizingFolder = resultsFolder;

		// // tablesOutputFolders = dataHome +
		// // "/tables_repository_subset_with_lists";
		// if (args.length > 1 && args[1] != null && args[1].equals("full")) {
		//
		//
		// mappingFolder = dataHome + "/tables/" + dataSet + "/mappings";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;
		//
		// learningSetsOutputFolder = dataHome + "/learningSets_full/" +
		// dataSet;
		//
		// columnMappingsFolder = dataHome + "/column_mappings/" + dataSet;
		//
		// hostRankingsFileFiltered = dataHome +
		// "/rankings/filtered_full.tsv.gz";
		//
		// tablesOutputFolders = dataHome + "/tables/" + dataSet +
		// "/repository";
		//
		// scoresFolder = dataHome + "/scores_full/" + dataSet;
		//
		// weightsFolder = dataHome + "/weights_full/" + dataSet;
		//
		// System.out.println("Reading Full Dataset");
		// }

		TableFusionApp.badTablesFolder = null;

		String ontologyOwlFile = dataHome + "/dbpedia_2014.owl";
		


 

		String hostRankingsFile = dataHome + "/rankings/hostgraph-pr.tsv.gz";


		// mappingFolder = dataHome + "/completeWithSourcesNew";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;

		UnitParserDomi.unitPath = dataHome + "/Units/Convertible";
		TableColumnBuilder.computeStatistiscs = false;

		String heidelTimeConfigPath = null;

		String scoresFolder = null;
		String weightsFolder = null;

		// mappingFolder = dataHome + "/tableMappingsRerun";
		// mappingType = "gz";
		// // tablesFileEnding = FileEnding.NO_URI_GZ;
		// tablesFileEnding = FileEnding.NO_URL;
		//
		// mappingFolder = dataHome + "/completeWithSources";
		// mappingType = "gz";
		// tablesFileEnding = FileEnding.NO_URI_GZ;
		// // tablesFileEnding = FileEnding.NO_URL;

		boolean useUnits = true;

		TableFusionModule tableFusionModule = new TableFusionModule(resultsFolder, null, learningSetsOutputFolder, resultsFolder,
				optimizingFolder, null);

		DBpediaModule dBpediaModule = new DBpediaModule(dbpediaRepository, null, RepositoryLoading.LOCAL_ONLY);

		TablesModule tablesModule = new TablesModule(mappingFolder, tablesOutputFolders, RepositoryLoading.LOCAL_ONLY, false, tableType,
 tablesFileEnding, null,
				false);

		OntologyModule ontologyModule = new OntologyModule(ontologyOwlFile);

		HeidelTimeModule heidelTimeModule = new HeidelTimeModule(heidelTimeConfigPath);

		MatchingModule matchingModule = new MatchingModule(useUnits, keyIndexPath, columnMappingsFolder, candidateSelectionConfig, null, emptyFile,
				surfaceForms,
				redirect, equivalentProperties);

		RankingModule rankingModule = new RankingModule(hostRankingsFile, hostRankingsFileFiltered);

		RegressionModule regressionModule = new RegressionModule(scoresFolder, weightsFolder);

		SimilarityModule similarityModule = new SimilarityModule();

		ObjectGraph objectGraph = ObjectGraph.create(tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, matchingModule,
				rankingModule, regressionModule, similarityModule);

		TableFusionSample tableFusionApp = new TableFusionSample();
		tableFusionApp.run(tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, matchingModule, rankingModule, regressionModule,
				objectGraph);
	}

	private PrintStream out;
	
	
	@Inject
	public TableFusionSample() {
		out = System.out;
		System.setOut(System.err);
	}

	private TableFusionModule tableFusionModule;
	private DBpediaModule dBpediaModule;
	private TablesModule tablesModule;
	private OntologyModule ontologyModule;
	private HeidelTimeModule heidelTimeModule;
	private MatchingModule matchingModule;
	private RankingModule rankingModule;
	private RegressionModule regressionModule;
	private ObjectGraph objectGraph;

	private void run(TableFusionModule tableFusionModule, DBpediaModule dBpediaModule, TablesModule tablesModule, OntologyModule ontologyModule,
			HeidelTimeModule heidelTimeModule, MatchingModule matchingModule, RankingModule rankingModule, RegressionModule regressionModule,
			ObjectGraph objectGraph) {
		this.tableFusionModule = tableFusionModule;
		this.dBpediaModule = dBpediaModule;
		this.tablesModule = tablesModule;
		this.ontologyModule = ontologyModule;
		this.heidelTimeModule = heidelTimeModule;
		this.matchingModule = matchingModule;
		this.regressionModule = regressionModule;
		this.objectGraph = objectGraph;
		this.rankingModule = rankingModule;
		
		
		Scanner in = new Scanner(System.in);
		System.err.flush();

		try {
			while (true) {

				out.println("=================================================");
				out.println("T2K Sample Fusion App");
				out.println("=================================================");
				out.println(" |=|  Please Select");
				out.println(" |=|  --");
				out.println(" |=|  DATA");
				out.println(" |=|  --");
				out.println(" |=|  [1] Load DBpedia from CSV");
				out.println(" |=|  [2] Load Tables From CSV");
				out.println(" |=|      - Requires: loaded DBpedia from CSV");
				out.println(" |=|  [3] Store Tables into Cache");
				out.println(" |=|  [4] Load Tables from Cache");
				out.println(" |=|      - Requires: loaded DBpedia from CSV");
				out.println(" |=|  [5] Create PageRank SubGraph");
				out.println(" |=|      - Requires: loaded DBpedia and Tables from CSV");
				out.println(" |=|  --");
				out.println(" |=|  Column Matching");
				out.println(" |=|  --");
				out.println(" |=|  [6] Run Column Matching");
				out.println(" |=|  --");
				out.println(" |=|  Pre-Fusion Analysis");
				out.println(" |=|  --");
				out.println(" |=|  [7] Run Group Analysis (stores Results in Results Folder)");
				out.println(" |=|  --");
				out.println(" |=|  Fusion using automatic evaluation");
				out.println(" |=|  --");
				out.println(" |=|  [8] Run and evaluate Fusion (stores results in results folder in data home)");
				out.println(" |=|  --");
				out.println(" |=|  Manual evaluation");
				out.println(" |=|  --");
				out.println(" |=|  [9] Extract sample to test Local Closed World Assumption (stores Sample in results Folder)");
				out.println(" |=|  --");
				out.println(" |=|  [0] Exit");
				out.println("=================================================");


				out.println();

				out.print("Your Input: ");
				out.flush();

				if (in.hasNextInt()) {
					int a = in.nextInt();
 
					if (a == 0)
						break;
					else {
						try {
							run(a);
						} catch (Exception e) {
							e.printStackTrace();
						}

					}

				} else {

					out.println("Sorry, couldn't understand you!");
					in.next();
				}

			}
		} finally {
			in.close();
		}
	}

	private void run(int a) {
		switch (a) {
		case 1: {
			objectGraph.get(DBpediaRepositoryNewInterface.class);
			break;
		}
		case 2: {
			tablesModule.setLoadFromCache(false);
			objectGraph.get(AnnotatedTablesRepository.class);
			break;
		}
		case 3: {
			AnnotatedTablesRepository annotatedTablesRepository = objectGraph.get(AnnotatedTablesRepository.class);

			AnnotatedTablesWriter annotatedTablesWriter = objectGraph.get(AnnotatedTablesWriter.class);

			annotatedTablesWriter.writeAll(annotatedTablesRepository);
			break;
		}
		case 4: {
			tablesModule.setLoadFromCache(true);

			objectGraph.get(AnnotatedTablesRepository.class);
			break;
		}
		case 5: {
			out.println("Creating Host Ranking Subset");

			String hostRankingsFile = rankingModule.getHostRankingsFile();
			String hostRankingsFileFiltered = rankingModule.getHostRankingsFileFiltered();

			RankCSVAdapter rankCSVAdapter = objectGraph.get(RankCSVAdapter.class);
			Map<HostId, Double> rr = rankCSVAdapter.read(hostRankingsFile, true);

			rankCSVAdapter.write(hostRankingsFileFiltered, rr);
			break;
		}
		case 6: {
			try {
			out.println("Running Column Matching and storing the data");
			ColumnMatchesWriter columnMatcherWriter = objectGraph.get(ColumnMatchesWriter.class);

				columnMatcherWriter.run();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			break;
		}
		case 7: {
			out.println("Group Analyzer");
			GroupAnalyzer groupAnalyzer = objectGraph.get(GroupAnalyzer.class);

			// try {
			// groupAnalyzer.run();
			// } catch (IOException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }

			break;
		}

		case 8: {
			out.println("Running Fusion Accuracy calculator");
			AccuracyAnalyzer accuracyAnalyzer = objectGraph.get(AccuracyAnalyzer.class);
			try {
				accuracyAnalyzer.getPerformanceWWW();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 

			break;
		}
		case 9: {
			out.println("Export Randomized Evaluation");

			EvaluationExporterRandom evaluationExporterRandom = objectGraph.get(EvaluationExporterRandom.class);
			// try {
			// evaluationExporterRandom.exportThresholdDataRandomColumnThresholded();
			// } catch (IOException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }
		}

	}
	}
}
