package de.mannheim.uni.expansion.tablefusion;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import org.apache.commons.io.IOUtils;

import dagger.ObjectGraph;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.CellIdx;
import de.mannheim.uni.expansion.data.tables.model.TableFactory;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParserCSV;
import de.mannheim.uni.expansion.data.tables.reader.InvalidTableException;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.CellMatchesReader;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.AccuracyAnalyzer;
import de.mannheim.uni.tableprocessor.ColumnTypeGuesser;
import de.mannheim.uni.tableprocessor.IO.ConvertFileToTable;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;
import de.mannheim.uni.units.UnitParserDomi;

/**
 * Hello world!
 *
 */
public class TimeDependentFusion {

	@Inject
	AccuracyAnalyzer accuracyAnalyzer;


	public void run(String tableFileName) {
		try {
			accuracyAnalyzer.getPerformance(tableFileName);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	

	public static void main(String[] args) {

		String dataHome = null;

		if (args.length > 0 && args[0] != null && !args[0].equals("")) {
			dataHome = args[0];

		} else {
			throw new IllegalStateException("You need to define the dataDir Variable as the first argument of this program, i.e."
					+ "jre/bin/java -Xmx120000M -XX:+UseG1GC de.mannheim.uni.expansion.tablefusion.TimeDependentFusion \"/home/user/t2k/dataDir/\"");
		}

		PrintStream out = System.out;
		System.setOut(System.err);

		System.setProperty("java.net.preferIPv4Stack", "true");
		// System.setProperty("java.rmi.server.hostname",
		// "wifo5-09.informatik.uni-mannheim.de");

		// try {
		// PrintStream out;
		// out = new PrintStream(new FileOutputStream("output.txt"));
		// System.setOut(out);
		// } catch (FileNotFoundException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }

		// Parallel.SetDefaultNumProcessors(1);
		// PersistentParallel.overideNumberOfProcessors(1);

		while (true) {

			TableType tableType = TableType.JSON_TABLE;



			if (args.length > 0 && args[0] != null && !args[0].equals("")) {
				dataHome = args[0];

			}

			if (args.length > 2 && args[2] != null && !args[2].equals("")) {
				System.setProperty("java.rmi.server.hostname", args[2]);
			}

			String dbpediaRepository = dataHome + "/repository/";

			FileEnding dbpediaFileEnding = FileEnding.CSV_GZ;
			String keyIndexPath = dataHome + "/index_full";





			String mappingFolder = dataHome + "/tables/" + "mappingsRaw";
			String mappingType = "gz";
			FileEnding tablesFileEnding = FileEnding.NO_URL;

			String learningSetsOutputFolder = dataHome + "/tables/" + "learningSets";
			String columnMappingsFolder = dataHome + "/tables/" + "columnMappings";

			String hostRankingsFileFiltered = dataHome + "/tables/" + "rankings/filtered.tsv.gz";

			String tablesOutputFolders = dataHome + "/tables/" + "mappings";

			String scoresFolder = dataHome + "/tables/" + "scores";

			String weightsFolder = dataHome + "/tables/" + "weights";

			String problemsFolder = dataHome + "/tables/" + "problems";

			String candidateSelectionConfig = dataHome + "/candindateSelection.config";
			String matchingConfig = dataHome + "/matching.config";

			String emptyFile = dataHome + "/empty.txt";
			String surfaceForms = dataHome + "/surfaceForms";
			String redirect = dataHome + "/redirects";
			String equivalentProperties = dataHome + "/equivalentProperties.tsv";

			String optimizingFolder = dataHome + "/optimizing";

			// // tablesOutputFolders = dataHome +
			// // "/tables_repository_subset_with_lists";
			// if (args.length > 1 && args[1] != null && args[1].equals("full"))
			// {
			//
			//
			// mappingFolder = dataHome + "/tables/" + "mappings";
			// mappingType = "gz";
			// // tablesFileEnding = FileEnding.NO_URI_GZ;
			// tablesFileEnding = FileEnding.NO_URL;
			//
			// learningSetsOutputFolder = dataHome + "/learningSets_full/" +
			// dataSet;
			//
			// columnMappingsFolder = dataHome + "/column_mappings/" + dataSet;
			//
			// hostRankingsFileFiltered = dataHome +
			// "/rankings/filtered_full.tsv.gz";
			//
			// tablesOutputFolders = dataHome + "/tables/" + dataSet +
			// "/repository";
			//
			// scoresFolder = dataHome + "/scores_full/" + dataSet;
			//
			// weightsFolder = dataHome + "/weights_full/" + dataSet;
			//
			// System.out.println("Reading Full Dataset");
			// }

			TableFusionApp.badTablesFolder = dataHome + "/completeWithSources";

			String ontologyOwlFile = dataHome + "/dbpedia_2014.owl";

			String resultsFolder = dataHome + "/results";

			String badAnnotatedTablesFile = dataHome + "/bad_tables.tar.gz";

			String hostRankingsFile = dataHome + "/rankings/hostgraph-pr.tsv.gz";

			String inputProfilingFolder = dataHome + "/results/inputProfiling";

			mappingType = "kryo";

			// mappingFolder = dataHome + "/completeWithSourcesNew";
			// mappingType = "gz";
			// // tablesFileEnding = FileEnding.NO_URI_GZ;
			// tablesFileEnding = FileEnding.NO_URL;

			UnitParserDomi.unitPath = dataHome + "/Units/Convertible";
			TableColumnBuilder.computeStatistiscs = false;

			String heidelTimeConfigPath = dataHome + "/heidelTime/config.props";

			// mappingFolder = dataHome + "/tableMappingsRerun";
			// mappingType = "gz";
			// // tablesFileEnding = FileEnding.NO_URI_GZ;
			// tablesFileEnding = FileEnding.NO_URL;
			//
			// mappingFolder = dataHome + "/completeWithSources";
			// mappingType = "gz";
			// tablesFileEnding = FileEnding.NO_URI_GZ;
			// // tablesFileEnding = FileEnding.NO_URL;

			boolean useUnits = true;

			TableFusionModule tableFusionModule = new TableFusionModule(resultsFolder, problemsFolder, learningSetsOutputFolder, inputProfilingFolder,
					optimizingFolder, null);


			TablesModule tablesModule = new TablesModule(mappingFolder, tablesOutputFolders, RepositoryLoading.LOCAL_ONLY, true, tableType,
					tablesFileEnding, badAnnotatedTablesFile, false);
			tablesModule.setLoadFromCache(true);
			tablesModule.setFilter(true);

			OntologyModule ontologyModule = new OntologyModule(ontologyOwlFile);

			HeidelTimeModule heidelTimeModule = new HeidelTimeModule(heidelTimeConfigPath);

			MatchingModule matchingModule = new MatchingModule(useUnits, keyIndexPath, columnMappingsFolder, candidateSelectionConfig, matchingConfig,
					emptyFile, surfaceForms, redirect, equivalentProperties);

			RankingModule rankingModule = new RankingModule(hostRankingsFile, hostRankingsFileFiltered);

			RegressionModule regressionModule = new RegressionModule(scoresFolder, weightsFolder);

			SimilarityModule similarityModule = new SimilarityModule();


			try {

				String tableDirectory = dataHome + "/gs/";

				List<String> tableNames = new ArrayList<>();

				// tableNames.add("fortune2008.csv");
				// tableNames.add("fortune2010.csv");
				// tableNames.add("fortune2014.csv");
				// tableNames.add("fortuneMixed.csv");

				tableNames.add("nflMixed.csv");
				// tableNames.add("nfl2010.csv");
				// tableNames.add("nfl2012.csv");
				// tableNames.add("nfl2014.csv");

				// tableNames.add("countries2008.csv");
				// tableNames.add("countries2014.csv");
				tableNames.add("countriesMixed.csv");

				// tableNames.add("cities2008.csv");
				// tableNames.add("cities2014.csv");
				// tableNames.add("citiesMixed.csv");

				// for (String tableFileName : tableNames) {
				runModules(tableDirectory, tableNames, dbpediaRepository, dbpediaFileEnding, tableFusionModule, tablesModule, ontologyModule,
							heidelTimeModule, matchingModule, rankingModule, regressionModule, similarityModule);
				// }

			} catch (IOException | InvalidTableException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}


	}

	private static void runModules(String tableDirectory, List<String> tableFileNames, String dbpediaRepository, FileEnding dbpediaFileEnding,
			TableFusionModule tableFusionModule, TablesModule tablesModule, OntologyModule ontologyModule, HeidelTimeModule heidelTimeModule,
			MatchingModule matchingModule, RankingModule rankingModule, RegressionModule regressionModule, SimilarityModule similarityModule)
					throws FileNotFoundException, IOException, InvalidTableException {

		AnnotatedTableParserCSV csvParser = new AnnotatedTableParserCSV(new TableFactory(new ColumnTypeGuesser(), new ConvertFileToTable()), null);
		List<AnnotatedTablesReaderData> tables = new ArrayList<>();
		List<Map<CellIdx, InstanceId>> columnMatchesList = new ArrayList<>();
		CellMatchesReader reader = new CellMatchesReader();

		for (String tableFileName : tableFileNames) {
			byte[] content = IOUtils.toByteArray(new FileReader(tableDirectory + tableFileName));

			AnnotatedTablesReaderData table = csvParser.get(content, "KB.csv", FileEnding.NO_URL, TableType.WEB_TABLE);

			Map<CellIdx, InstanceId> columnMatches = reader.readSingleTable(tableDirectory + "matches/" + tableFileName);
			tables.add(table);
			columnMatchesList.add(columnMatches);
		}

		DBpediaModule dBpediaModule = new DBpediaModule(tables, columnMatchesList, dbpediaRepository, dbpediaFileEnding, RepositoryLoading.LOCAL_ONLY);

		ObjectGraph objectGraph = ObjectGraph.create(tableFusionModule, dBpediaModule, tablesModule, ontologyModule, heidelTimeModule, matchingModule,
				rankingModule, regressionModule, similarityModule);

		tablesModule.setClassFilter(objectGraph.get(DBpediaRepositoryNewInterface.class));

		TimeDependentFusion tableFusionApp = objectGraph.get(TimeDependentFusion.class);
		tableFusionApp.run("");

	}


}
