package de.mannheim.uni.expansion.tablefusion.core.fusing;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;

public class EquivalenceClusterer {

	@Inject
	public EquivalenceClusterer() {

	}

	public List<Cluster> addValue(List<Cluster> clusters, SimilarityMeasure measure, BaseFuser baseFuser, Object value) {
		clusters = new ArrayList<>(clusters);
		boolean added = false;
		for (int i = 0; i < clusters.size(); i++) {
			if (measure.isEqual(clusters.get(i).getValue(), value)) {
				List<Object> valueList = new ArrayList<>(clusters.get(i).getValues());
				valueList.add(value);

				clusters.set(i, new Cluster(baseFuser.simpleFuse(valueList).getSingle(), valueList));

				added = true;
				break;
			}
		}
		if (!added) {
			List<Object> valueList = new ArrayList<>();
			valueList.add(value);

			clusters.add(new Cluster(value, valueList));
		}

		return clusters;
	}

	public Cluster getHighestRatedCluster(List<Cluster> clusters) {
		clusters = new ArrayList<>(clusters);
		Collections.sort(clusters);
		return clusters.get(clusters.size() - 1);
	}

	public class Cluster implements Comparable<Cluster> {
		private final Object fusedValue;
		private final List<Object> values;
		private final int count;

		public Cluster(Object fusedValue, List<Object> values) {
			super();
			this.fusedValue = fusedValue;
			this.values = Collections.unmodifiableList(values);
			this.count = values.size();
		}

		public List<Object> getValues() {
			return values;
		}

		@Override
		public int compareTo(Cluster o) {
			return Integer.compare(count, o.getCount());
		}

		public Object getValue() {
			return fusedValue;
		}

		public int getCount() {
			return count;
		}

	}
}
