package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base;

import java.util.List;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class BaseAverageFuser implements BaseFuser {

	@Inject
	public BaseAverageFuser() {

	} 

	@Override
	public FusedValue simpleFuse(List<Object> values) {
		double sum = 0.0;
		int count = 0;

		for (Object o : values) {

			Double d = getDouble(o);
			if (d != null) {
				sum += d;
				count++;
			}

			else {
				// System.err.println("WARNING: Value not numeric: " + o);
			}

		}

		Double sim = (Double) sum / count;
		if (sim.equals(Double.NaN))
			return null;
		else
			return new FusedValue(sim, (double) values.size());
	}

	@Override
	public FusedValue fuse(List<? extends ValueProbabilityPair> values, Double threshold) {

		double sum = 0.0;
		double count = 0;

		for (ValueProbabilityPair pair : values) {

			Double d = getDouble(pair.getValue());
			Double probability = pair.getScore();
			if (d != null) {
				sum += d * probability;
				count += probability;
			}

		}

		Double sim = (Double) sum / count;
		if (sim.equals(Double.NaN))
			return null;
		else
			return new FusedValue(sim, sum);
	}

	protected Double getDouble(Object o) {
		if (o instanceof Double) {
			Double d = (Double) o;
			return d;
		} else if (o instanceof String) {
			// TODO: Wrong Place for fix
			String s = (String) o;
			s = s.replace("&nbsp;", "").trim();
			try {
				Double d = Double.valueOf(s);
				return d;
			} catch (NumberFormatException fe) {
				// if (s.length() > 0)
				// System.err.println("WARNING: Value not numeric: " +
				// o);
			}
		}

		return null;
	}

	@Override
	public boolean checkType(Object value) {
		if (getDouble(value) != null)
			return true;
		else
			return false;

	}

}
