package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base;

import java.util.Calendar;
import java.util.Collections;
import java.util.Date;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMajorityMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.Averager;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.Comparer;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.ListSorter;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.TypeCaster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.TypeChecker;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.utils.ThreadSafeLocalCalendar;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class BaseDateFuser extends GenericMajorityMedianFuser<Date> {

	private final static TypeChecker<Date> typeChecker = (o) -> {
		return (o instanceof Date);
	};

	private final static TypeCaster<Date> typeCaster = (o) -> {
		return (Date) o;
	};

	private final static ListSorter<Date> listSorter = (list) -> {
		Collections.sort(list);
	};

	private final static ThreadSafeLocalCalendar threadSafeCalendar = new ThreadSafeLocalCalendar();

	private final static Averager<Date> averager = (object1, score1, object2, score2) -> {
		Date dateA = (Date) object1;
		Date dateB = (Date) object2;

		Calendar calendar = threadSafeCalendar.getCalendar();
		calendar.setTime(dateA);

		int yearA = calendar.get(Calendar.YEAR);
		int monthA = calendar.get(Calendar.MONTH);
		int dayA = calendar.get(Calendar.DAY_OF_MONTH);

		calendar.setTime(dateB);

		int yearB = calendar.get(Calendar.YEAR);
		int monthB = calendar.get(Calendar.MONTH);
		int dayB = calendar.get(Calendar.DAY_OF_MONTH);

		if (yearA == yearB) {
			calendar.set(Calendar.YEAR, yearA);

			calendar.set(Calendar.MONTH, 0);
			calendar.set(Calendar.DAY_OF_MONTH, 1);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);

			if (monthA == monthB) {
				calendar.set(Calendar.MONTH, monthA);

				if (dayA == dayB) {
					calendar.set(Calendar.DAY_OF_MONTH, dayA);
				}
			}
			Date date = calendar.getTime();
			return date;
		} else {

			if (score2 > score1)
				return object2;
			else
				return object1;
		}
	};

	private final static Comparer<Date> comparer = (object, other) -> {
		return object.compareTo(other);
	};

	@Inject
	public BaseDateFuser(SimilarityCalculator sc) {
		super(new GenericMedianFuser<>(typeChecker, typeCaster, listSorter, averager, comparer), sc.getMeasure(ColumnDataType.date));
	}
} 
