package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class BaseMajorityFuser implements BaseFuser {

	@Inject
	public BaseMajorityFuser() {

	}

	@Override
	public FusedValue simpleFuse(List<Object> values) {
		Map<Object, Integer> countMap = new HashMap<Object, Integer>();

		for (Object o : values) {

			Integer count = countMap.get(o);
			if (count == null) {
				count = 0;
			}
			count++;
			countMap.put(o, count);

		}

		Comparator<Entry<Object, Integer>> comparator = (e1, e2) -> {
			if (e1.getValue() < e2.getValue())
				return -1;
			else if (e1.getValue() > e2.getValue()) {
				return 1;
			} else {
				return 0;
			}
		};

		// Find the max count
		Optional<Entry<Object, Integer>> o = countMap.entrySet().stream().max(comparator);

		if (o.isPresent())
			return new FusedValue(o.get().getKey(), o.get().getValue().doubleValue());
		else
			return null;
	}

	@Override
	public FusedValue fuse(List<? extends ValueProbabilityPair> values, Double threshold) {

		Map<Object, List<Double>> countMap = new HashMap<Object, List<Double>>();

		for (ValueProbabilityPair pair : values) {
			Double probability = pair.getScore();
			Object o = pair.getValue();
			List<Double> count = countMap.get(o);
			if (count == null) {
				count = new ArrayList<Double>();
				countMap.put(o, count);
			}
			count.add(probability);

		}

		Function<List<Double>, Double> sumListOfDoubles = (listOfDouble) -> {
			return listOfDouble.stream().mapToDouble(Double::doubleValue).sum();
		};

		Map<Object, Double> summedMap = countMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, e -> {
			return sumListOfDoubles.apply(e.getValue());
		}));

		Comparator<Entry<Object, Double>> comparator = (e1, e2) -> {
			if (e1.getValue() < e2.getValue())
				return -1;
			else if (e1.getValue() > e2.getValue()) {
				return 1;
			} else {
				return 0;
			}
		};

		Optional<Entry<Object, Double>> o = summedMap.entrySet().stream().max(comparator);

		if (o.isPresent())
			if (threshold == null || o.get().getValue() > threshold)
				return new FusedValue(o.get().getKey(), o.get().getValue());
			else
				return null;
		else
			return null;
	}

	@Override
	public boolean checkType(Object value) {
		return true;
	}
}
