package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class BaseMedianFuser implements BaseFuser {

	@Inject
	public BaseMedianFuser() {

	}

	@Override
	public FusedValue simpleFuse(List<Object> list) {
		List<Double> values = new ArrayList<Double>();

		for (Object o : list) {

			Double d = getDouble(o);
			if (d != null) {
				values.add(d);

			} else {
				// System.err.println("WARNING: Value not numeric: " + o);
			}

		}

		Collections.sort(values);

		if (values.size() > 0) {
			if (values.size() % 2 != 0) {
				int idx = (values.size() / 2);
				Double value = (Double) values.get(idx);
				return new FusedValue(value, (double) list.size());

			} else {
				int idx1 = (values.size() / 2);
				int idx2 = (values.size() / 2) - 1;

				Double value = (((double) values.get(idx1)) + ((double) values.get(idx2))) / 2.0;
				return new FusedValue(value, (double) list.size());
			}
		} else {
			return null;
		}
	}

	@Override
	public FusedValue fuse(List<? extends ValueProbabilityPair> values, Double threshold) {
		if (values.size() == 0)
			return null;
		double sum = 0.0;
		List<Pair> pairs = new ArrayList<Pair>();

		for (ValueProbabilityPair pair : values) {

			Double probability = pair.getScore();

			Object o = pair.getValue();

			Double d = getDouble(o);
			if (d != null) {
				pairs.add(new Pair(d, probability));
				sum += probability;
			}

		}

		if (threshold == null || sum > threshold) {
			Collections.sort(pairs);

			double median = sum / 2.0;

			double current = 0.0;

			Double value = null;

			Pair lastPair = null;
			for (Pair p : pairs) {
				current += p.getScore();

				if (median > current) {
					continue;
				} else {
					if (lastPair == null) {
						value = p.getValue();
						break;
					} else {
						double diff = p.getScore();
						double s1 = (current) - median;
						double s2 = median - (current - p.getScore());

						value = ((p.getValue() * s1) + (lastPair.getValue() * s2)) / diff;
					}
				}

				lastPair = p;
			}

			if (value == null)
				return null;

			return new FusedValue(value, sum);
		}
		return null;
	}

	private static class Pair implements Comparable<Pair> {
		private final Double value;
		private final Double score;

		public Double getValue() {
			return value;
		}

		public Double getScore() {
			return score;
		}

		public Pair(Double value, Double score) {
			super();
			this.value = value;
			this.score = score;
		}

		@Override
		public int compareTo(Pair o) {
			return this.value.compareTo(o.value);
		}

	}

	protected Double getDouble(Object o) {
		if (o instanceof Double) {
			Double d = (Double) o;
			return d;
		} else if (o instanceof String) {
			// TODO: Wrong Place for fix
			String s = (String) o;
			s = s.replace("&nbsp;", "").trim();
			try {
				Double d = Double.valueOf(s);
				return d;
			} catch (NumberFormatException fe) {
				// if (s.length() > 0)
				// System.err.println("WARNING: Value not numeric: " +
				// o);
			}
		}

		return null;
	}

	@Override
	public boolean checkType(Object value) {
		if (getDouble(value) != null)
			return true;
		else
			return false;

	}
}
