package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base;

import java.util.Collections;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMajorityMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.Averager;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.Comparer;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.ListSorter;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.TypeCaster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.GenericMedianFuser.TypeChecker;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class BaseNumericFuser extends GenericMajorityMedianFuser<Double> {

	private final static TypeChecker<Double> typeChecker = (o) -> {

		if (o instanceof Double) {
			return true;
		} else if (o instanceof String) {
			// TODO: Wrong Place for fix
			String s = (String) o;
			s = s.replace("&nbsp;", "").trim();
			try {
				Double d = Double.valueOf(s);
				return true; 
			} catch (NumberFormatException fe) {
				return false;
			}
		}

		return false;
	};

	private final static TypeCaster<Double> typeCaster = (o) -> {

		if (o instanceof Double) {
			Double d = (Double) o;
			return d;
		} else if (o instanceof String) {
			// TODO: Wrong Place for fix
			String s = (String) o;
			s = s.replace("&nbsp;", "").trim();
			try {
				Double d = Double.valueOf(s);
				return d;
			} catch (NumberFormatException fe) {
				// if (s.length() > 0)
				// System.err.println("WARNING: Value not numeric: " +
				// o);
			}
		}

		return null;
	};

	private final static ListSorter<Double> listSorter = (list) -> {
		Collections.sort(list);
	};

	private final static Comparer<Double> comparer = (object, other) -> {
		return object.compareTo(other);
	};

	private final static Averager<Double> averager = (value1, score1, value2, score2) -> {

		return ((value1 * score1) + (value2 * score2)) / (score1 + score2);

	};

	@Inject 
	public BaseNumericFuser(SimilarityCalculator sc) {
		super(new GenericMedianFuser<>(typeChecker, typeCaster, listSorter, averager, comparer), sc.getMeasure(ColumnDataType.numeric));
	}

}
