package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.utils.ProbabilityPair;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;
import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;

public class GenericMajorityMedianFuser<T> implements BaseFuser {
	private final BaseFuser medianFuser;
 
	private final SimilarityMeasure measure;

	public GenericMajorityMedianFuser(BaseFuser medianFuser, SimilarityMeasure measure) {
		super();
		this.medianFuser = medianFuser;
		this.measure = measure;
	}

	@Override
	public FusedValue simpleFuse(List<Object> values) {
		List<ValueProbabilityPair> vpList = values.stream().map((o) -> new ProbabilityPair(o, 1.0)).collect(Collectors.toList());

		return fuse(vpList, null);

	}

	@Override
	public FusedValue fuse(List<? extends ValueProbabilityPair> values, Double threshold) {
		List<Cluster> countMap = new ArrayList<Cluster>();

		for (ValueProbabilityPair vp : values) {
			boolean added = false;
			for (int i = 0; i < countMap.size(); i++) {
				if (measure.isEqual(countMap.get(i).getValue(), vp.getValue())) {
					List<ValueProbabilityPair> valueProbabilityList = new ArrayList<>(countMap.get(i).getValues());
					valueProbabilityList.add(new ProbabilityPair(vp.getValue(), vp.getScore()));
					
					FusedValue fv = medianFuser.fuse(valueProbabilityList);
					if(fv == null) {
						int fsdf = 123;
					}

					countMap.set(i, new Cluster(fv.getSingle(), valueProbabilityList));

					added = true;
					break;
				}
			}
			if (!added) {
				List<ValueProbabilityPair> valueProbabilityList = new ArrayList<>();
				valueProbabilityList.add(new ProbabilityPair(vp.getValue(), vp.getScore()));

				countMap.add(new Cluster(vp.getValue(), valueProbabilityList));
			}
		}

		if (countMap.size() > 0) {
			Collections.sort(countMap);
			Cluster c = countMap.get(countMap.size() - 1);
			if (threshold == null || c.getScore() > threshold) {
				return new FusedValue(c.getValue(), c.getScore());
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	@Override
	public boolean checkType(Object value) {
		return medianFuser.checkType(value);
	}

	private class Cluster implements ValueProbabilityPair {
		private final Object fusedValue;
		private final List<ValueProbabilityPair> values;
		private final double score;

		public Cluster(Object fusedValue, List<ValueProbabilityPair> values) {
			super();
			this.fusedValue = fusedValue;
			this.values = Collections.unmodifiableList(values);
			this.score = values.parallelStream().mapToDouble(ValueProbabilityPair::getScore).sum();
		}

		public List<ValueProbabilityPair> getValues() {
			return values;
		}

		@Override
		public int compareTo(ValueProbabilityPair o) {
			return Double.compare(score, o.getScore());
		}

		@Override
		public Object getValue() {
			return fusedValue;
		}

		@Override
		public Double getScore() {
			return score;
		}

	}

}
