package de.mannheim.uni.expansion.tablefusion.core.fusing.stores;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.NavigableSet;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.utils.ProbabilityPair;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class FusedValue {

	private final boolean isList;
	private final Object singleValue;
	private final Double singleScore;
	private final NavigableSet<ValueProbabilityPair> list;

	public FusedValue(Set<? extends ValueProbabilityPair> list) {
		this.list = new TreeSet<>(list);
		this.isList = true;
		this.singleValue = null;
		this.singleScore = null;
	}

	public FusedValue(Object singleValue, Double singleScore) {
		this.singleValue = singleValue;
		this.singleScore = singleScore;
		this.list = null;
		this.isList = false;
	}

	public static FusedValue fromObject(Object value) {
		if (value instanceof List<?>) {
			@SuppressWarnings("unchecked")
			List<Object> values = (List<Object>) value;

			Set<ValueProbabilityPair> set = values.stream().map((e) -> {
				return new ProbabilityPair(e, 1.0);
			}).collect(Collectors.toSet());
			return new FusedValue(set);

		} else {
			return new FusedValue(value, 1.0);
		}
	}

	public boolean isList() {
		return isList;
	}

	public Object getSingle() {
		return singleValue;
	}

	public Object getSingleScore() {
		return singleScore;
	}

	private final Double delta = 0.00001;

	public List<Set<Object>> getRankedList() {
		List<Set<Object>> rankedList = new ArrayList<>();

		Double lastSim = null;
		Set<Object> currentSet = null;
		for (ValueProbabilityPair e : this.list.descendingSet()) {

			if (currentSet == null) {
				currentSet = new HashSet<>();
				lastSim = e.getScore();

				currentSet.add(e.getValue());
			} else if (equal(lastSim, e.getScore(), delta)) {

				currentSet.add(e.getValue());
			} else {
				rankedList.add(currentSet);
				currentSet = new HashSet<>();
				lastSim = e.getScore();

				currentSet.add(e.getValue());
			}

		}

		rankedList.add(currentSet);

		return rankedList;
	}

	public NavigableSet<ValueProbabilityPair> getScoredList() {
		return Collections.unmodifiableNavigableSet(this.list);
	}

	private boolean equal(Double d1, Double d2, Double delta) {
		if (Math.abs(d1 - d2) < delta) {
			return true;
		} else {
			return false;
		}
	}

	public List<Object> getSimpleList() {
		List<Object> simpleList = new ArrayList<>();

		for (ValueProbabilityPair e : this.list.descendingSet()) {
			simpleList.add(e.getValue());

		}

		return simpleList;
	}

	public int listSize() {
		if (list == null)
			return 0;
		return list.size();

	}
}
