package de.uni_mannheim.informatik.dws.tablefusion.matching;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.commons.io.IOUtils;

import de.mannheim.uni.expansion.columnmatching.CMProperties;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.TableFactory;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParserCSV;
import de.mannheim.uni.expansion.data.tables.reader.InvalidTableException;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.CellMatchesWriter;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.ColumnMatchesForTable;
import de.mannheim.uni.expansion.tablefusion.core.matching.impl.reference.DBPediaMatchingDataBuilder;
import de.mannheim.uni.expansion.utils.MKDIR;
import de.mannheim.uni.matching.dbpedia.Preprocessing;
import de.mannheim.uni.matching.dbpedia.algorithm.KeyIndex;
import de.mannheim.uni.matching.dbpedia.algorithm.WebtableToDBpediaMatchingProcess;
import de.mannheim.uni.matching.dbpedia.algorithm.WebtableToDBpediaMatchingProcess.tableType;
import de.mannheim.uni.matching.dbpedia.model.MatchingParameters;
import de.mannheim.uni.matching.dbpedia.model.MatchingResult;
import de.mannheim.uni.matching.process.Configuration;
import de.mannheim.uni.tableprocessor.ColumnTypeGuesser;
import de.mannheim.uni.tableprocessor.IO.ConvertFileToTable;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;

public class SingleMatchingService {

	  
	  
	  final DBPediaMatchingDataBuilder mdBuilder;
	  
	  
	  final MatchingParameters matchingParameters;
	  
	  final KeyIndex keyIndex;
	  
	  final String equivalentProperties;
	  
	  final Configuration configuration;
	  
	final CellMatchesWriter cellMatchesWriter = new CellMatchesWriter();

	final ColumnMatchesForTable columnMatchesForTable;

	  @Inject
	public SingleMatchingService(DBPediaMatchingDataBuilder mdBuilder, MatchingParameters matchingParameters, @Named(CMProperties.KEYINDEX) KeyIndex keyIndex,
			@Named(CMProperties.SURFACE_FORMS) String surface, @Named(CMProperties.REDIRECTS) String redirects,
			@Named(CMProperties.EQUIVALENT_PROPERTIES) String eqProps, @Named(CMProperties.MATCHING_CONFIG) Configuration configuration,
			ColumnMatchesForTable columnMatchesForTable) {
		this.mdBuilder = mdBuilder;
		this.matchingParameters = matchingParameters;
		this.matchingParameters.setTableType(tableType.webtable);
		this.keyIndex =keyIndex;
		this.equivalentProperties = eqProps;
		 this.configuration = configuration;
		this.columnMatchesForTable = columnMatchesForTable;
		  
		  
		  try {
			Preprocessing.loadSurfaceForms(surface);
		     Preprocessing.loadRedirects(redirects);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	  
	  
	public void matchTable(String dataSetsDirecotry, String matchingFile) throws FileNotFoundException, IOException, InvalidTableException {
		String inputFolder = dataSetsDirecotry + "input/";
		String outputFolder = dataSetsDirecotry + "output/";
		  
		String webtablePath = inputFolder + matchingFile;
          
		  WebtableToDBpediaMatchingProcess m = new WebtableToDBpediaMatchingProcess();

		Set<ClassId> allowedClasses = new HashSet<ClassId>();
		allowedClasses.add(new ClassId("http://dbpedia.org/ontology/Country"));
		m.setData(mdBuilder.getMatchingData());
		  m.setMatchingParameters(matchingParameters);
		  m.setLuceneIndex(keyIndex.getLuceneIndex());
		// m.loadDBpedia("C:/Users/yaser/dev/data/dbpediacsv");
		;
		  
	      m.getData().loadEquivalentProperties(equivalentProperties);
	         
		TableColumnBuilder.computeStatistiscs =true;

		  MatchingResult.matchingResultsDir.set(outputFolder);
		  
		Configuration c = configuration.clone();
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_WEBTABLE, webtablePath);
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_EVALUATE, false);
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_TABLE_TYPE, WebtableToDBpediaMatchingProcess.tableType.webtable);
		m.run(c);
		MatchingResult mr = m.getResult();
		  
		  
		  mr.write(tableType.webtable);

	}

	public void matchColumns(String dataSetsDirecotry, String matchingFile) throws FileNotFoundException, IOException, InvalidTableException {
		String outputFolder = dataSetsDirecotry + "output/";

		String inputFolder = dataSetsDirecotry + "input/";
		String columnMatchesFolder = outputFolder + "matches/";

		MKDIR.createDir(new File(columnMatchesFolder));
		
		
		AnnotatedTableParserCSV csvParser = new AnnotatedTableParserCSV(new TableFactory(new ColumnTypeGuesser(), new ConvertFileToTable()), null);
		
		byte[] content = IOUtils.toByteArray(new FileReader(outputFolder + matchingFile));

		AnnotatedTablesReaderData table = csvParser.get(content, matchingFile, FileEnding.NO_URL, TableType.WEB_TABLE);
		
		
		cellMatchesWriter.writeSingle(columnMatchesFolder + matchingFile, matchingFile, columnMatchesForTable.run(table));
	}
}
