package de.uni_mannheim.informatik.dws.tablefusion.matching;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.columnmatching.CMProperties;
import de.mannheim.uni.expansion.data.tables.reader.InvalidTableException;
import de.mannheim.uni.matching.dbpedia.Preprocessing;
import de.mannheim.uni.matching.dbpedia.algorithm.KeyIndex;
import de.mannheim.uni.matching.dbpedia.algorithm.WebtableToDBpediaMatchingProcess;
import de.mannheim.uni.matching.dbpedia.algorithm.WebtableToDBpediaMatchingProcess.tableType;
import de.mannheim.uni.matching.dbpedia.model.MatchingParameters;
import de.mannheim.uni.matching.dbpedia.model.MatchingResult;
import de.mannheim.uni.matching.process.Configuration;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;

public class SingleMatchingService2 {


	  
	  
	  final MatchingParameters matchingParameters;
	  
	  final KeyIndex keyIndex;
	  
	  final String equivalentProperties;
	  
	  final Configuration configuration;
	  
	  @Inject
	public SingleMatchingService2(MatchingParameters matchingParameters, @Named(CMProperties.KEYINDEX) KeyIndex keyIndex,
			@Named(CMProperties.SURFACE_FORMS) String surface, @Named(CMProperties.REDIRECTS) String redirects,
			@Named(CMProperties.EQUIVALENT_PROPERTIES) String eqProps, @Named(CMProperties.MATCHING_CONFIG) Configuration configuration) {

		this.matchingParameters = matchingParameters;
		this.matchingParameters.setTableType(tableType.webtable);
		this.keyIndex =keyIndex;
		this.equivalentProperties = eqProps;
		 this.configuration = configuration;
		  
		  
		  try {
			Preprocessing.loadSurfaceForms(surface);
		     Preprocessing.loadRedirects(redirects);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	  
	  
	public void matchTable(String dataSetsDirecotry, String matchingFile) throws FileNotFoundException, IOException, InvalidTableException {
		String inputFolder = dataSetsDirecotry + "input/";
		String outputFolder = dataSetsDirecotry + "output/";

		String webtablePath = inputFolder + matchingFile;
		  
		  

          
		  WebtableToDBpediaMatchingProcess m = new WebtableToDBpediaMatchingProcess();



		  m.setMatchingParameters(matchingParameters);
		  m.setLuceneIndex(keyIndex.getLuceneIndex());
		m.loadDBpedia("C:/Users/yaser/dev/data/dbpediacsv");
		;
		  
	      m.getData().loadEquivalentProperties(equivalentProperties);
	         
		TableColumnBuilder.computeStatistiscs =true;

		  MatchingResult.matchingResultsDir.set(outputFolder);
		  
		Configuration c = configuration.clone();
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_WEBTABLE, webtablePath);
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_EVALUATE, false);
		c.getConfig().put(WebtableToDBpediaMatchingProcess.PAR_TABLE_TYPE, WebtableToDBpediaMatchingProcess.tableType.webtable);
		m.run(c);
		MatchingResult mr = m.getResult();
		  
		  
		  mr.write(tableType.webtable);
	  }
}
