package de.mannheim.uni.expansion.tablefusion.adapters.scores;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ColumnScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.HostScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.InstanceScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.PersistentParallel;

public class ScoresOutputCreator {
	private final ValueEvaluator accuracyCalculator;

	@Inject
	public ScoresOutputCreator(ValueEvaluator accuracyCalculator) {
		this.accuracyCalculator = accuracyCalculator;
	}

	Creator<List<ScoresOutputEntry>> listCreator = () -> {
		return new ArrayList<>();
	};

	public Map<ClassPropertyId, List<ScoresOutputEntry>> get(GroupedValues trainingValues, InstanceScores instanceScores, ColumnScores columnScores,
			TableScores tableScores, HostScores hostScores) {

		ConcurrentHashMap<ClassPropertyId, List<ScoresOutputEntry>> map = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(trainingValues.entrySet(), (e) -> {

			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey().getClassId(), e.getKey().getPropertyId());

			List<ScoresOutputEntry> list = new ArrayList<>();

			for (SingleValue sv : e.getValue().getValues()) {
				for (SimilarityType similarityType : SimilarityType.values()) {
					// double instance =
					// instanceScores.getScore(e.getKey().getInstanceId(),
					// e.getKey().getPropertyId(), sv.getId())
					// .getOr(similarityType, new Accuracy(0.0,
					// 0)).getAccuracy();
					// double column =
					// columnScores.getScore(e.getKey().getInstanceId(),
					// e.getKey().getPropertyId(), sv.getId())
					// .getOr(similarityType, new Accuracy(0.0,
					// 0)).getAccuracy();
					// double table =
					// tableScores.getScore(e.getKey().getInstanceId(),
					// e.getKey().getPropertyId(), sv.getId())
					// .getOr(similarityType, new Accuracy(0.0,
					// 0)).getAccuracy();
					// double host =
					// hostScores.getScore(e.getKey().getInstanceId(),
					// e.getKey().getPropertyId(), sv.getId())
					// .getOr(similarityType, new Accuracy(0.0,
					// 0)).getAccuracy();

					double instance = 0.0;
					double column = 0.0;
					double table = 0.0;
					double host = 0.0;
					Boolean correct = accuracyCalculator.correct(e.getKey(), FusedValue.fromObject(sv.getValue()));

					if (correct != null) {
						ScoresOutputEntry soe = new ScoresOutputEntry(instance, column, table, host, correct, similarityType);
						list.add(soe);
					}
				}

			}

			List<ScoresOutputEntry> chmList = CHMHelper.getFromCHM(map, classPropertyId, listCreator);

			synchronized (chmList) {
				chmList.addAll(list);
			}

		} , "ScoresOutputCreator");

		return map;
	}
}
