package de.mannheim.uni.expansion.tablefusion.analysis;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;

import au.com.bytecode.opencsv.CSVWriter;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.learningsets.LearningSetsRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankRepository;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusedResultsSingle;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.ProbabilitiesCalculator;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionRunner;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class DistributionAnalyzer {

	private final GroupedValuesRepository groupedValuesRepository;
	private final OntologyRepository ontologyRepository;
	private final ValueEvaluator valueEvaluator;
	private final ProbabilitiesCalculator probabilitiesCalculator;
	private final LearningSetsRepository learningSetsRepository;
	private final RankRepository rankRepository;

	private final FusionRunner fusionRunner;

	private final AnnotatedTablesRepository annotatedTablesRepository;

	private final MaxCorrectFinder maxCorrectFinder;
	private final AccuracyCalculator accuracyCalculator;

	@Inject
	public DistributionAnalyzer(GroupedValuesRepository groupedValuesRepository, OntologyRepository ontologyRepository, ValueEvaluator valueEvaluator,
			ProbabilitiesCalculator probabilitiesCalculator, FusionRunner fusionRunner, LearningSetsRepository learningSetsRepository,
			RankRepository rankRepository, AnnotatedTablesRepository annotatedTablesRepository, MaxCorrectFinder maxCorrectFinder,
			AccuracyCalculator accuracyCalculator) {
		this.groupedValuesRepository = groupedValuesRepository;
		this.ontologyRepository = ontologyRepository;
		this.valueEvaluator = valueEvaluator;
		this.probabilitiesCalculator = probabilitiesCalculator;
		this.fusionRunner = fusionRunner;
		this.learningSetsRepository = learningSetsRepository;
		this.rankRepository = rankRepository;
		this.annotatedTablesRepository = annotatedTablesRepository;
		this.maxCorrectFinder = maxCorrectFinder;
		this.accuracyCalculator = accuracyCalculator;
	}

	public String printTripleSourceDistribution() {
		StringBuilder sb = new StringBuilder();

		for (Entry<Integer, Integer> e : groupedValuesRepository.getDistribution().entrySet()) {
			sb.append("" + e.getKey() + "," + e.getValue() + "\n");
		}
		String ret = sb.toString();
		System.out.println(ret);
		return ret;
	}

	// Quad<PerformancePR, Map<ColumnDataType, PerformancePR>, Map<ClassId,
	// PerformancePR>, Map<ClassPropertyId, PerformancePR>> getCorrectAndWrong(
	// Map<TripleId, Object> fusedValues, Map<TripleId, MatchedValues>
	// groupedValues, boolean print) {
	//
	// FusedResultsSingle totalResults = new FusedResultsSingle();
	// ConcurrentHashMap<ColumnDataType, FusedResultsSingle> dataTypeMap = new
	// ConcurrentHashMap<>();
	// ConcurrentHashMap<ClassId, FusedResultsSingle> classMap = new
	// ConcurrentHashMap<>();
	// ConcurrentHashMap<ClassPropertyId, FusedResultsSingle> classPropertyMap =
	// new ConcurrentHashMap<>();
	//
	// PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {
	//
	// ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
	// ColumnDataType cdt = ontologyRepository.getDataType(classPropertyId);
	//
	// Boolean correct = accuracyCalculator.correct(e.getKey(), e.getValue());
	//
	// Integer count = groupedValues.get(e.getKey()).getValues().size();
	//
	// if (correct != null) {
	//
	// FusedResultsSingle dataTypeResults = CHMHelper.getFromCHM(dataTypeMap,
	// cdt, () -> new FusedResultsSingle());
	// FusedResultsSingle classResults = CHMHelper.getFromCHM(classMap,
	// classPropertyId.getClassId(), () -> new FusedResultsSingle());
	// FusedResultsSingle classPropertyResults =
	// CHMHelper.getFromCHM(classPropertyMap, classPropertyId, () -> new
	// FusedResultsSingle());
	//
	// if (correct) {
	// totalResults.incCorrect();
	// dataTypeResults.incCorrect();
	// classResults.incCorrect();
	// classPropertyResults.incCorrect();
	// } else {
	// totalResults.incWrong();
	// dataTypeResults.incWrong();
	// classResults.incWrong();
	// classPropertyResults.incWrong();
	// }
	// }
	//
	// } , "Accuracy Distribution");
	//
	// int maxCorrect = maxCorrectFinder.getMaxCorrectValues(groupedValues);
	// PerformancePR totalPR = new
	// PerformancePR(totalResults.getCorrectTriples(),
	// totalResults.getWrongTriples(), maxCorrect);
	//
	// Map<ColumnDataType, Integer> dataTypeMaxCorrect =
	// maxCorrectFinder.getMaxCorrectValuePerDataType(groupedValues);
	//
	// Map<ColumnDataType, PerformancePR> dataTypePerformance =
	// dataTypeMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey,
	// (e) -> {
	// return new PerformancePR(e.getValue().getCorrectTriples(),
	// e.getValue().getWrongTriples(),
	// dataTypeMaxCorrect.getOrDefault(e.getKey(), 0));
	// }));
	//
	// Map<ClassId, Integer> classIdMaxCorrect =
	// maxCorrectFinder.getMaxCorrectValuePerClass(groupedValues);
	//
	// Map<ClassId, PerformancePR> classIdPerformance =
	// classMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e)
	// -> {
	// return new PerformancePR(e.getValue().getCorrectTriples(),
	// e.getValue().getWrongTriples(),
	// classIdMaxCorrect.getOrDefault(e.getKey(), 0));
	// }));
	//
	// Map<ClassPropertyId, Integer> classPropertyIdyMaxCorrect =
	// maxCorrectFinder.getMaxCorrectValuePerClassPropertyId(groupedValues);
	//
	// Map<ClassPropertyId, PerformancePR> classPropertyIdPerformance =
	// classPropertyMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey,
	// (e) -> {
	// return new PerformancePR(e.getValue().getCorrectTriples(), //
	// e.getValue().getWrongTriples(), //
	// classPropertyIdyMaxCorrect.getOrDefault(e.getKey(), 0));
	// }));
	// return new Quad<>(totalPR, dataTypePerformance, classIdPerformance,
	// classPropertyIdPerformance);
	//
	// }

	public void writeResultsByDistribution(Map<TripleId, FusedValue> fusedValues, GroupedValues groupedValues) {

		ConcurrentHashMap<Integer, FusedResultsSingle> totalMap = new ConcurrentHashMap<>();
		ConcurrentHashMap<ColumnDataType, ConcurrentHashMap<Integer, FusedResultsSingle>> dataTypeMap = new ConcurrentHashMap<>();
		ConcurrentHashMap<ClassPropertyId, ConcurrentHashMap<Integer, FusedResultsSingle>> classPropertyMap = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {

			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
			ColumnDataType cdt = ontologyRepository.getDataType(classPropertyId);

			Boolean correct = valueEvaluator.correct(e.getKey(), e.getValue());

			Integer count = groupedValues.get(e.getKey()).getValues().size();

			if (correct != null) {

				FusedResultsSingle totalResults = CHMHelper.getFromCHM(totalMap, count, () -> new FusedResultsSingle());

				ConcurrentHashMap<Integer, FusedResultsSingle> dataTypeInnerMap = CHMHelper.getFromCHM(dataTypeMap, cdt, () -> new ConcurrentHashMap<>());
				FusedResultsSingle dataTypeResults = CHMHelper.getFromCHM(dataTypeInnerMap, count, () -> new FusedResultsSingle());

				ConcurrentHashMap<Integer, FusedResultsSingle> classPropertyInnerMap = CHMHelper.getFromCHM(classPropertyMap, classPropertyId,
						() -> new ConcurrentHashMap<>());
				FusedResultsSingle classPropertyResults = CHMHelper.getFromCHM(classPropertyInnerMap, count, () -> new FusedResultsSingle());

				if (correct) {
					totalResults.incCorrect();
					dataTypeResults.incCorrect();
					classPropertyResults.incCorrect();
				} else {
					totalResults.incWrong();
					dataTypeResults.incWrong();
					classPropertyResults.incWrong();
				}
			}

		} , "Accuracy Distribution");

		try {
			Map<String, Map<Integer, FusedResultsSingle>> totalParentMap = new HashMap<>();
			totalParentMap.put("Total", totalMap);
			String totalCSV = getCSV(totalParentMap);
			String dataTypeCSV = getCSV(dataTypeMap);
			String classPropertyCSV = getCSV(classPropertyMap);

			// int totalCorrect = totalMap.entrySet().stream().mapToInt((e) -> {
			// return e.getValue().getCorrectTriples();
			// }).sum();
			// int totalWrong = totalMap.entrySet().stream().mapToInt((e) -> {
			// return e.getValue().getWrongTriples();
			// }).sum();
			//
			// Map<ColumnDataType, FusedResultsSingle> dataTypeCounts =
			// dataTypeMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey,
			// (e) -> {
			// int totalCorrectDataType =
			// e.getValue().entrySet().stream().mapToInt((e2) -> {
			// return e2.getValue().getCorrectTriples();
			// }).sum();
			//
			// int totalWrongDataType =
			// e.getValue().entrySet().stream().mapToInt((e2) -> {
			// return e2.getValue().getCorrectTriples();
			// }).sum();
			//
			// return new FusedResultsSingle(totalCorrectDataType,
			// totalWrongDataType, true);
			// }));

			// if (print) {
			System.out.println("Total");
			System.out.println(totalCSV);
			System.out.println();
			System.out.println();
			System.out.println();
			System.out.println("DataType");
			System.out.println(dataTypeCSV);
			System.out.println();
			System.out.println();
			System.out.println();
			System.out.println("ClassPropertyId");
			System.out.println(classPropertyCSV);
			// }

			// return new ImmutablePair<>(totalCorrect, totalWrong);

		} catch (

		IOException e1)

		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

	}

	private <T, M extends Map<Integer, FusedResultsSingle>> String getCSV(Map<T, M> map) throws IOException {
		StringWriter sw = new StringWriter();
		CSVWriter csvWriter = new CSVWriter(sw);

		int lineLength = 4;

		String[] header = new String[lineLength];
		header[0] = "Name";
		header[1] = "Tables";
		header[2] = "Correct";
		header[3] = "Wrong";

		csvWriter.writeNext(header);

		for (Entry<T, M> e : map.entrySet()) {
			String[] line = new String[lineLength];
			line[0] = e.getKey().toString();

			for (Entry<Integer, FusedResultsSingle> e2 : e.getValue().entrySet()) {
				line[1] = e2.getKey().toString();
				line[2] = String.valueOf(e2.getValue().getCorrectTriples());
				line[3] = String.valueOf(e2.getValue().getWrongTriples());

				csvWriter.writeNext(line);
			}
		}

		csvWriter.close();

		return sw.toString();

	}

	// public void performanceUsingCrossValidation() {
	// performanceUsingCrossValidation(FusionStrategyRepository.KBT, (prefix,
	// instanceScores, columnScores, tableScores, hostScores) -> {
	// return columnScores;
	// } , 0.35);
	// }

	// void performanceUsingCrossValidation(FusionStrategy fusionRunType,
	// ScoresGetter scoresGetter, double probabilityThreshold) {
	// boolean fuseLists = false;
	// GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();
	//
	// List<FusionDataType> methodType = new ArrayList<>();
	// methodType.add(FusionDataType.REFERENCE);
	// methodType.add(FusionDataType.STRING);
	// methodType.add(FusionDataType.NUMERIC);
	// methodType.add(FusionDataType.DATE);
	// methodType.add(FusionDataType.LINK);
	//
	// List<FusionPreproccesser> preprocessors = new ArrayList<>();
	// preprocessors.add(FusionPreproccesser.NONE);
	//
	// Map<Integer, CreatedSets> createdSetsMap =
	// learningSetsRepository.getCreatedSets();
	//
	// String totalString = "";
	// Map<ColumnDataType, AtomicReference<String>> dataTypesString = new
	// HashMap<>();
	// Map<ClassId, AtomicReference<String>> classIdString = new HashMap<>();
	// Map<ClassPropertyId, AtomicReference<String>> classPropertyIdString = new
	// HashMap<>();
	//
	// for (Entry<Integer, CreatedSets> createdSets : createdSetsMap.entrySet())
	// {
	//
	// InstanceScores instanceScores =
	// probabilitiesCalculator.getInstanceScores(createdSets.getValue().getLearningSet());
	// ColumnScores columnScores =
	// probabilitiesCalculator.byProperty(createdSets.getValue().getLearningSet());
	// TableScoresImpl tableScores =
	// probabilitiesCalculator.byTable(createdSets.getValue().getLearningSet(),
	// fuseLists);
	// HostScores hostScores = probabilitiesCalculator.byHost(tableScores);
	//
	// ScoresContainer probabilities = scoresGetter.getScores("",
	// instanceScores, columnScores, tableScores, hostScores);
	//
	// Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap =
	// fusionRunner.run(methodType, preprocessors, fusionRunType,
	// createdSets.getValue().getTestingSet(), probabilities, fuseLists,
	// probabilityThreshold);
	//
	// Map<TripleId, FusedValue> fusedValues =
	// CHMHelper.flattenMap(fusedValuesMap);
	//
	// FusionPerformance<PerformancePR> fusionPerformance =
	// accuracyCalculator.analyzeAccuracy(createdSets.getValue().getTestingSet(),
	// fusedValues);
	// // Quad<PerformancePR, Map<ColumnDataType, PerformancePR>,
	// // Map<ClassId, PerformancePR>, Map<ClassPropertyId, PerformancePR>>
	// // counts = getCorrectAndWrong(
	// // fusedValues, createdSets.getValue().getTestingSet(), false);
	// {
	// PerformancePR performance = fusionPerformance.getTotal();
	// totalString += "," + performance.getCorrect() + "," +
	// performance.getWrong() + "," + performance.getMaxCorrect() + ","
	// + performance.getPrecision() + "," + performance.getRecall() + "," +
	// performance.getfMeasure();
	// }
	//
	// {
	// for (Entry<ColumnDataType, PerformancePR> e :
	// fusionPerformance.getPerDataType().entrySet()) {
	// PerformancePR performance = e.getValue();
	// AtomicReference<String> ar = CHMHelper.getFromMap(dataTypesString,
	// e.getKey(), () -> new AtomicReference<String>(""));
	// String s = ar.get();
	// s += "," + performance.getCorrect() + "," + performance.getWrong() + ","
	// + performance.getMaxCorrect() + "," + performance.getPrecision()
	// + "," + performance.getRecall() + "," + performance.getfMeasure();
	// ar.set(s);
	// }
	// }
	//
	// {
	// for (Entry<ClassId, PerformancePR> e :
	// fusionPerformance.getPerClassId().entrySet()) {
	// PerformancePR performance = e.getValue();
	// AtomicReference<String> ar = CHMHelper.getFromMap(classIdString,
	// e.getKey(), () -> new AtomicReference<String>(""));
	// String s = ar.get();
	// s += "," + performance.getCorrect() + "," + performance.getWrong() + ","
	// + performance.getMaxCorrect() + "," + performance.getPrecision()
	// + "," + performance.getRecall() + "," + performance.getfMeasure();
	// ar.set(s);
	// }
	// }
	//
	// {
	// for (Entry<ClassPropertyId, PerformancePR> e :
	// fusionPerformance.getPerProperty().entrySet()) {
	// PerformancePR performance = e.getValue();
	// AtomicReference<String> ar = CHMHelper.getFromMap(classPropertyIdString,
	// e.getKey(), () -> new AtomicReference<String>(""));
	// String s = ar.get();
	// s += "," + performance.getCorrect() + "," + performance.getWrong() + ","
	// + performance.getMaxCorrect() + "," + performance.getPrecision()
	// + "," + performance.getRecall() + "," + performance.getfMeasure();
	// ar.set(s);
	// }
	// }
	//
	// }
	//
	// System.out.print("Total");
	// System.out.print("," + "TC1" + "," + "TW1" + "," + "MC1" + "," + "P1" +
	// "," + "R1" + "," + "FM1");
	// System.out.print("," + "TC2" + "," + "TW2" + "," + "MC2" + "," + "P2" +
	// "," + "R2" + "," + "FM2");
	// System.out.print("," + "TC3" + "," + "TW3" + "," + "MC3" + "," + "P3" +
	// "," + "R3" + "," + "FM3");
	// System.out.print("," + "TC4" + "," + "TW4" + "," + "MC4" + "," + "P4" +
	// "," + "R4" + "," + "FM4");
	// System.out.print("," + "TC5" + "," + "TW5" + "," + "MC5" + "," + "P5" +
	// "," + "R5" + "," + "FM5");
	// System.out.println();
	// System.out.print("Total");
	// System.out.println(totalString);
	//
	// System.out.print("DataType");
	// System.out.print("," + "TC1" + "," + "TW1" + "," + "MC1" + "," + "P1" +
	// "," + "R1" + "," + "FM1");
	// System.out.print("," + "TC2" + "," + "TW2" + "," + "MC2" + "," + "P2" +
	// "," + "R2" + "," + "FM2");
	// System.out.print("," + "TC3" + "," + "TW3" + "," + "MC3" + "," + "P3" +
	// "," + "R3" + "," + "FM3");
	// System.out.print("," + "TC4" + "," + "TW4" + "," + "MC4" + "," + "P4" +
	// "," + "R4" + "," + "FM4");
	// System.out.print("," + "TC5" + "," + "TW5" + "," + "MC5" + "," + "P5" +
	// "," + "R5" + "," + "FM5");
	// System.out.println();
	// for (Entry<ColumnDataType, AtomicReference<String>> e :
	// dataTypesString.entrySet()) {
	// System.out.print(e.getKey().toString());
	// System.out.println(e.getValue().get());
	// }
	//
	// System.out.print("Class");
	// System.out.print("," + "TC1" + "," + "TW1" + "," + "MC1" + "," + "P1" +
	// "," + "R1" + "," + "FM1");
	// System.out.print("," + "TC2" + "," + "TW2" + "," + "MC2" + "," + "P2" +
	// "," + "R2" + "," + "FM2");
	// System.out.print("," + "TC3" + "," + "TW3" + "," + "MC3" + "," + "P3" +
	// "," + "R3" + "," + "FM3");
	// System.out.print("," + "TC4" + "," + "TW4" + "," + "MC4" + "," + "P4" +
	// "," + "R4" + "," + "FM4");
	// System.out.print("," + "TC5" + "," + "TW5" + "," + "MC5" + "," + "P5" +
	// "," + "R5" + "," + "FM5");
	// System.out.println();
	// for (Entry<ClassId, AtomicReference<String>> e :
	// classIdString.entrySet()) {
	// System.out.print(e.getKey().toString());
	// System.out.println(e.getValue().get());
	// }
	//
	// System.out.print("ClassPropertyId");
	// System.out.print("," + "TC1" + "," + "TW1" + "," + "MC1" + "," + "P1" +
	// "," + "R1" + "," + "FM1");
	// System.out.print("," + "TC2" + "," + "TW2" + "," + "MC2" + "," + "P2" +
	// "," + "R2" + "," + "FM2");
	// System.out.print("," + "TC3" + "," + "TW3" + "," + "MC3" + "," + "P3" +
	// "," + "R3" + "," + "FM3");
	// System.out.print("," + "TC4" + "," + "TW4" + "," + "MC4" + "," + "P4" +
	// "," + "R4" + "," + "FM4");
	// System.out.print("," + "TC5" + "," + "TW5" + "," + "MC5" + "," + "P5" +
	// "," + "R5" + "," + "FM5");
	// System.out.println();
	// for (Entry<ClassPropertyId, AtomicReference<String>> e :
	// classPropertyIdString.entrySet()) {
	// System.out.print(e.getKey().toString());
	// System.out.println(e.getValue().get());
	// }
	//
	// }

}
