package de.mannheim.uni.expansion.tablefusion.analysis;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.adapters.learningsets.LearningSetsRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.ProbabilitiesCalculator;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionRunner;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;

public class GroupAnalyzer {

	private final GroupedValuesRepository groupedValuesRepository;

	private final AnnotatedTablesRepository annotatedTablesRepository;

	private final FusionRunner fusionRunner;

	private final String resultsFolder;

	private final ProbabilitiesCalculator probabilitiesCalculator;

	private final OntologyRepository ontologyRepository;

	private final DBpediaRepositoryNewInterface dbBpediaRepository;

	private final ValueEvaluator accuracyCalculator;

	private final ScoreFusionThresholdFinder scoreFusionThresholdFinder;

	private final LearningSetsRepository learningSetsRepository;

	@Inject
	public GroupAnalyzer(GroupedValuesRepository groupedValuesRepository, AnnotatedTablesRepository annotatedTablesRepository, FusionRunner fusionRunner,
			@Named(NamedProperties.RESULTS_FOLDER) String resultsFolder, ProbabilitiesCalculator probabilitiesCalculator, OntologyRepository ontologyRepository,
			DBpediaRepositoryNewInterface dbBpediaRepository, ValueEvaluator accuracyCalculator, ScoreFusionThresholdFinder scoreFusionThresholdFinder,
			LearningSetsRepository learningSetsRepository) {
		super();
		this.groupedValuesRepository = groupedValuesRepository;
		this.annotatedTablesRepository = annotatedTablesRepository;
		this.fusionRunner = fusionRunner;
		this.resultsFolder = resultsFolder;
		this.probabilitiesCalculator = probabilitiesCalculator;
		this.ontologyRepository = ontologyRepository;
		this.dbBpediaRepository = dbBpediaRepository;
		this.accuracyCalculator = accuracyCalculator;
		this.scoreFusionThresholdFinder = scoreFusionThresholdFinder;
		this.learningSetsRepository = learningSetsRepository;
	}

	// public void run() throws IOException {
	// List<FusionDataType> methodTypes = new ArrayList<>();
	// methodTypes.add(FusionDataType.REFERENCE);
	// methodTypes.add(FusionDataType.STRING);
	// methodTypes.add(FusionDataType.NUMERIC);
	// methodTypes.add(FusionDataType.DATE);
	// methodTypes.add(FusionDataType.LINK);
	//
	// List<FusionPreproccesser> preprocessors = new ArrayList<>();
	// preprocessors.add(FusionPreproccesser.NONE);
	//
	// GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();
	//
	// Map<Integer, CreatedSets> createdSets =
	// learningSetsRepository.getCreatedSets();
	//
	// double threshold =
	// scoreFusionThresholdFinder.findOptimumThresholdCrossValidation(createdSets,
	// FusionStrategyRepository.KBT, false);
	//
	// ScoresContainer scores =
	// probabilitiesCalculator.byProperty(groupedValues);
	//
	// Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap =
	// fusionRunner.run(methodTypes, FusionPreproccesser.NONE,
	// FusionStrategyRepository.KBT,
	// groupedValues, scores, false, threshold);
	//
	// Map<TripleId, FusedValue> fusedValues =
	// CHMHelper.flattenMap(fusedValuesMap);
	// CSVWriter csvWriter = new CSVWriter(new FileWriter(resultsFolder +
	// "/groups.csv"));
	//
	// System.out.println("Size of groupedvalues: " + groupedValues.size());
	//
	// int lineLength = 9;
	// String[] headerLine = new String[lineLength];
	// headerLine[0] = "DataType";
	// headerLine[1] = "Class";
	// headerLine[2] = "Property";
	// headerLine[3] = "Instance";
	// headerLine[4] = "NoOfSources";
	// headerLine[5] = "NoOfPLDS";
	// headerLine[6] = "ExistsInDBP";
	// headerLine[7] = "CorrectInGroup";
	// headerLine[8] = "FilteredAS";
	//
	// csvWriter.writeNext(headerLine);
	//
	// for (Entry<TripleId, MatchedValues> e : groupedValues.entrySet()) {
	// String[] line = new String[lineLength];
	//
	// // DataType
	// ColumnDataType cdt = ontologyRepository.getDataType(new
	// ClassPropertyId(e.getKey()));
	// if (cdt != null)
	// line[0] = cdt.toString();
	// else
	// line[0] = ColumnDataType.unknown.toString();
	//
	// // Class
	// line[1] = e.getKey().getClassId().getFinalPart();
	//
	// // Property
	// line[2] = e.getKey().getPropertyId().getFinalPart();
	//
	// // Instance
	// line[3] = e.getKey().getInstanceId().getReadableString();
	//
	// // NoOfSources
	// line[4] = String.valueOf(e.getValue().getValues().size());
	//
	// // NoOfPLDS
	//
	// Set<String> uniquePLDs = new HashSet<>();
	// for (SingleValue sv : e.getValue().getValues()) {
	// String source =
	// annotatedTablesRepository.getMetaData(sv.getId()).getSource().getSource();
	//
	// String PLD = DomainUtils.getPayLevelDomainFromWholeURL(source);
	// uniquePLDs.add(PLD);
	//
	// }
	//
	// line[5] = String.valueOf(uniquePLDs.size());
	//
	// // ExistsInDBP
	// if (dbBpediaRepository.hasValue(e.getKey())) {
	// line[6] = "1";
	// } else {
	// line[6] = "0";
	// }
	//
	// // Object correctValue = dbBpediaRepository.getValue(e.getKey());
	// // CorrectInGroup
	// int correctInGroup = 0;
	// for (SingleValue sv : e.getValue().getValues()) {
	// Boolean c = accuracyCalculator.correct(e.getKey(),
	// FusedValue.fromObject(sv.getValue()));
	//
	// if (c != null && c) {
	// correctInGroup++;
	// }
	//
	// // Probability score =
	// // scores.getScore(e.getKey().getInstanceId(),
	// // e.getKey().getPropertyId(), sv.getId());
	// // int fdf = 123;
	// }
	//
	// line[7] = String.valueOf(correctInGroup);
	//
	// // FilteredAS
	// if (fusedValues.containsKey(e.getKey())) {
	// line[8] = "0";
	// } else {
	// line[8] = "1";
	// }
	//
	// csvWriter.writeNext(line);
	//
	// }
	//
	// csvWriter.close();
	//
	// }

	public void printUniqueMatchedValues() {
		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		System.out.println("Unique matches: " + groupedValues.size());

	}
}
