package de.mannheim.uni.expansion.tablefusion.analysis.stores;

import java.util.Collections;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.Map;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class FusionPerformance<E> {
	private final E total;
	private final Map<ColumnDataType, E> perDataType;
	private final Map<ClassId, E> perClassId;
	private final Map<ClassPropertyId, E> perProperty;

	public FusionPerformance(E total, Map<ColumnDataType, E> perDataType, Map<ClassId, E> perClassId, Map<ClassPropertyId, E> perProperty) {

		this.total = total;
		Map<ColumnDataType, E> perDataTypeTemp = new EnumMap<>(ColumnDataType.class);
		perDataTypeTemp.putAll(perDataType);
		this.perDataType = Collections.unmodifiableMap(perDataTypeTemp);
		this.perClassId = Collections.unmodifiableMap(new HashMap<>(perClassId));
		this.perProperty = Collections.unmodifiableMap(new HashMap<>(perProperty));
	}

	public E getTotal() {
		return total;
	}

	public Map<ColumnDataType, E> getPerDataType() {
		return perDataType;
	}

	public Map<ClassId, E> getPerClassId() {
		return perClassId;
	}

	public Map<ClassPropertyId, E> getPerProperty() {
		return perProperty;
	}
}
