package de.mannheim.uni.expansion.tablefusion.analysis.stores;

import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.tablefusion.profiling.accumulation.Accumulateable;
import de.mannheim.uni.expansion.tablefusion.profiling.accumulation.ConcurrentAccumulator;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class FusionPerformanceSynchronizedCreator<E extends Accumulateable<E>> {
	private ConcurrentAccumulator<E> total;
	private final ConcurrentHashMap<ColumnDataType, ConcurrentAccumulator<E>> perDataType = new ConcurrentHashMap<>();
	private final ConcurrentHashMap<ClassId, ConcurrentAccumulator<E>> perClassId = new ConcurrentHashMap<>();
	private final ConcurrentHashMap<ClassPropertyId, ConcurrentAccumulator<E>> perProperty = new ConcurrentHashMap<>();

	private final E empty;

	public FusionPerformanceSynchronizedCreator(E empty) {
		this.empty = empty;
		total = new ConcurrentAccumulator<E>(empty);
	}

	public FusionPerformance<E> get() {

		Function<Entry<?, ConcurrentAccumulator<E>>, E> valueMapper = (e) -> {
			return e.getValue().get();
		};

		E total = this.total.get();
		Map<ColumnDataType, E> perDataType = this.perDataType.entrySet().stream().collect(Collectors.toMap(Entry::getKey, valueMapper));
		Map<ClassId, E> perClassId = this.perClassId.entrySet().stream().collect(Collectors.toMap(Entry::getKey, valueMapper));
		Map<ClassPropertyId, E> perProperty = this.perProperty.entrySet().stream().collect(Collectors.toMap(Entry::getKey, valueMapper));
		return new FusionPerformance<E>(total, perDataType, perClassId, perProperty);
	}

	public void addTotal(E val) {
		this.total.add(val);
	}

	public void addPerDataType(ColumnDataType key, E val) {
		CHMHelper.getFromCHM(this.perDataType, key, () -> new ConcurrentAccumulator<>(empty)).add(val);
	}

	public void addPerClassId(ClassId key, E val) {
		CHMHelper.getFromCHM(this.perClassId, key, () -> new ConcurrentAccumulator<>(empty)).add(val);
	}

	public void addPerProperty(ClassPropertyId key, E val) {
		CHMHelper.getFromCHM(this.perProperty, key, () -> new ConcurrentAccumulator<>(empty)).add(val);
	}
}
