package de.mannheim.uni.expansion.tablefusion.analysis.stores;

import de.mannheim.uni.expansion.tablefusion.analysis.Accuracy;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.performance.PERFMPerformanceI;

public class PerformancePR implements PERFMPerformanceI {

	private final int correct;
	private final int wrong;
	private final int maxCorrect;

	private final double precision;
	private final double recall;
	private final double fMeasure;

	private final int newTriples;

	public PerformancePR(int correct, int wrong, int maxCorrect, int newTriples) {
		super();

		this.correct = correct;
		this.wrong = wrong;
		this.maxCorrect = maxCorrect;
		double precision = Double.NaN;
		precision = ((double) correct) / (correct + wrong);
		if (Double.isNaN(precision)) {
			this.precision = 0;
		} else {
			this.precision = precision;
		}
		if (maxCorrect == 0)
			this.recall = 1.0;
		else
			this.recall = ((double) correct) / (maxCorrect);
		double fMeasure = 2 * ((precision * recall) / (precision + recall));
		if (Double.isNaN(fMeasure))
			this.fMeasure = 0.0;
		else
			this.fMeasure = fMeasure;

		this.newTriples = newTriples;

	}

	public int getCorrect() {
		return correct;
	}

	public int getWrong() {
		return wrong;
	}

	public int getOverlapping() {
		return wrong + correct;
	}

	public int getMaxCorrect() {
		return maxCorrect;
	}

	public double getPrecision() {
		return precision;
	}

	public double getRecall() {
		return recall;
	}

	public double getfMeasure() {
		return fMeasure;
	}

	public int getNew() {
		return this.newTriples;
	}

	@Override
	public String toString() {
		return "PerformancePR [correct=" + correct + ", wrong=" + wrong + ", maxCorrect=" + maxCorrect + ", precision=" + precision + ", recall=" + recall
				+ ", fMeasure=" + fMeasure + ", newTriples=" + newTriples + "]";
	}

	public int getTotal() {
		return wrong + correct + newTriples;
	}

	public Accuracy getAccuracy() {
		return new Accuracy(this.precision, this.getOverlapping());
	}

}
