package de.mannheim.uni.expansion.tablefusion.analysis.writer;

import java.io.IOException;
import java.util.EnumMap;

public abstract class AbstractResultsWriter {
	private final EnumMap<ResultsCategory, PrefixCSVWriter> map = new EnumMap<>(ResultsCategory.class);

	protected AbstractResultsWriter() {

		String[] headerLine = getHeaderLine();
		for (ResultsCategory pc : ResultsCategory.values()) {
			map.put(pc, new PrefixCSVWriter(pc.getPrefix(), headerLine));
		}

	}

	protected abstract String[] getHeaderLine();

	protected void write(ResultsCategory pc, String[] prefix, String[] line) {
		map.get(pc).writeLine(prefix, line);
	}

	public String getAll(ResultsCategory pc) throws IOException {
		return map.get(pc).getAll();
	}

	public int getCurrentIndex(ResultsCategory pc) {
		return map.get(pc).getCurrentIndex();
	}

	public String getFrom(ResultsCategory pc, int index) throws IOException {
		return map.get(pc).getFrom(index);
	}

	public String getNoHeader(ResultsCategory pc, int index) throws IOException {
		return map.get(pc).getNoHeader(index);
	}
}
