package de.mannheim.uni.expansion.tablefusion.core.fusing;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.RandomFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseAverageFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseDateFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseDateMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseMajorityFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseMedianFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.base.BaseNumericFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.AbstractNormalFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.AbstractScoreFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic.AbstractWeightedScoreFuser;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserOptionsBuilder;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionDataType;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionMethod;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategy;
import de.mannheim.uni.expansion.utils.OtsuThresholder;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class FuserRepository {
	public final BaseAverageFuser averageFuser;

	public final BaseMedianFuser medianFuser;
	public final BaseMajorityFuser majorityFuser;

	public final BaseDateMedianFuser dateMedianFuser;
	public final BaseNumericFuser baseNumericFuser;
	public final BaseDateFuser baseDateFuser;

	public final RandomFuser randomFuser;

	private final OtsuThresholder otsuThresholder;
	private final SimilarityCalculator similarityCalculator;
	private final ClusterFuser clusterFuser;

	@Inject
	public FuserRepository(BaseAverageFuser averageFuser, BaseMedianFuser medianFuser, BaseMajorityFuser majorityFuser, BaseDateMedianFuser dateMedianFuser,
			RandomFuser randomFuser, de.mannheim.uni.expansion.utils.OtsuThresholder otsuThresholder, SimilarityCalculator similarityCalculator,
			ClusterFuser clusterFuser, BaseNumericFuser baseNumericFuser, BaseDateFuser baseDateFuser) {
		super();
		this.averageFuser = averageFuser;
		this.medianFuser = medianFuser;
		this.majorityFuser = majorityFuser;
		this.dateMedianFuser = dateMedianFuser;
		this.randomFuser = randomFuser;
		this.otsuThresholder = otsuThresholder;
		this.similarityCalculator = similarityCalculator;
		this.clusterFuser = clusterFuser;
		this.baseNumericFuser = baseNumericFuser;
		this.baseDateFuser = baseDateFuser;
	}

	public BaseFuser getBaseFuser(ColumnDataType cdt) {

		BaseFuser baseFuser = null;
		switch (FusionDataType.get(cdt).getBaseMethod()) {
		case DATE_MEDIAN:
			baseFuser = baseDateFuser;
			break;
		case MAJORTIY:
			baseFuser = majorityFuser;
			break;
		case NUMERIC_MEDIAN:
			baseFuser = medianFuser;
			break;
		case NUMERIC_MAJORITY_MEDIAN:
			baseFuser = baseNumericFuser;
			break;
		default:
			throw new IllegalStateException("UNDETECTED FUSION BASE METHOD");

		}

		return baseFuser;
	}

	public Fuser get(ColumnDataType cdt, NewFuserOptionsBuilder fuserOptions) {

		FusionStrategy fusionStrategy = fuserOptions.getFusionStrategy(cdt);
		if (fusionStrategy == null)
			return null;

		BaseFuser baseFuser = getBaseFuser(cdt);
		// switch (FusionDataType.get(cdt).getBaseMethod()) {
		// case DATE_MEDIAN:
		// baseFuser = baseDateFuser;
		// break;
		// case MAJORTIY:
		// baseFuser = majorityFuser;
		// break;
		// case NUMERIC_MEDIAN:
		// baseFuser = medianFuser;
		// break;
		// case NUMERIC_MAJORITY_MEDIAN:
		// baseFuser = baseNumericFuser;
		// break;
		// default:
		// throw new IllegalStateException("UNDETECTED FUSION BASE METHOD");
		//
		// }

		FusionMethod fm = fusionStrategy.getFusionMethod();
		if (fm == FusionMethod.RANDOM)
			return randomFuser;
		else if (fm == FusionMethod.NORMAL) {
			return new AbstractNormalFuser(similarityCalculator, otsuThresholder, clusterFuser, baseFuser);
		} else if (fm == FusionMethod.SCORED) {
			return new AbstractScoreFuser(similarityCalculator, otsuThresholder, clusterFuser, baseFuser);
		} else if (fm == FusionMethod.WEIGHTED_SCORED) {
			return new AbstractWeightedScoreFuser(similarityCalculator, otsuThresholder, clusterFuser, baseFuser);
		}

		throw new IllegalStateException("UNEXPTECTED FUSION METHOD");
	}

}
