package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.Cluster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.ClusterValue;
import de.mannheim.uni.expansion.tablefusion.core.fusing.Fuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.utils.OtsuThresholder;

public class AbstractNormalFuser implements Fuser {
	private final SimilarityCalculator sc;
	private final OtsuThresholder otsuThresholder;
	private final ClusterFuser clusterFuser;
	private final BaseFuser baseFuser;

	public AbstractNormalFuser(SimilarityCalculator sc, OtsuThresholder otsuThresholder, ClusterFuser clusterFuser, BaseFuser baseFuser) {
		this.sc = sc;
		this.otsuThresholder = otsuThresholder;
		this.clusterFuser = clusterFuser;
		this.baseFuser = baseFuser;
	}

	@Override
	public FusedValue fuse(MatchedValues values, TableScores probabilities, double threshold) {

		List<Object> objectValues = new ArrayList<Object>();
		for (SingleValue e : values.getValues()) {

			Object o = e.getValue();
			if (o instanceof List) {
				objectValues.addAll((List<?>) o);
			} else {

				objectValues.add(o);
			}

		}

		return baseFuser.simpleFuse(objectValues);

	}

	@Override
	public FusedValue fuseList(MatchedValues values, TableScores probabilities, double threshold) {

		SimilarityMeasure sm = sc.getMeasure(values.getDataType());

		List<ClusterValue> clusterValues = new ArrayList<>();

		for (SingleValue e : values.getValues()) {

			Object o = e.getValue();
			if (o instanceof List) {

				for (Object o2 : ((List<?>) o)) {
					clusterValues.add(new ClusterValue(o2, 1.0));
				}

			} else {

				clusterValues.add(new ClusterValue(o, 1.0));
			}

		}

		SortedSet<Cluster> clusters = clusterFuser.fuse(clusterValues, baseFuser, sm);

		clusters = otsuThresholder.threshold(clusters);

		if (clusters.size() == 0)
			return null;
		else
			return new FusedValue(clusters);
	}

}
