package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.Cluster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.ClusterValue;
import de.mannheim.uni.expansion.tablefusion.core.fusing.Fuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ListFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.utils.OtsuThresholder;
import de.mannheim.uni.expansion.utils.ProbabilityPair;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class AbstractScoreFuser implements Fuser, ListFuser {
	private final SimilarityCalculator sc;
	private final OtsuThresholder otsuThresholder;
	private final ClusterFuser clusterFuser;
	private final BaseFuser baseFuser;

	public AbstractScoreFuser(SimilarityCalculator sc, OtsuThresholder otsuThresholder, ClusterFuser clusterFuser, BaseFuser simpleFuser) {
		this.sc = sc;
		this.otsuThresholder = otsuThresholder;
		this.clusterFuser = clusterFuser;
		this.baseFuser = simpleFuser;
	}

	@Override
	public FusedValue fuse(MatchedValues values, TableScores probabilities, double threshold) {

		List<ValueProbabilityPair> list = new ArrayList<>();
		double probabilitySum = 0.0;
		//
		// List<SingleValue> sortedValues =
		// values.getValues().stream().sorted((e1, e2) -> {
		// Probability p1 = probabilities.get(e1.getId());
		// Probability p2 = probabilities.get(e2.getId());
		// Double d1 =
		// probabilities.get(e1.getId()).get(SimilarityType.DIRECT).getAccuracy();
		// Double d2 =
		// probabilities.get(e2.getId()).get(SimilarityType.DIRECT).getAccuracy();
		// return Double.compare(d1, d2);
		// }).collect(Collectors.toList());
		//
		// double perValue = 1.0 / sortedValues.size();

		// double max = 0.0;
		// double min = 1.0;
		// if (values.getValues().size() > 1) {
		// for (SingleValue e : values.getValues()) {
		// Probability p = probabilities.get(e.getId());
		// if (p != null) {
		// Double probability = p.get(SimilarityType.DIRECT).getAccuracy();
		// if (probability > max) {
		// max = probability;
		// }
		//
		// if (probability < min) {
		// min = probability;
		// }
		// }
		// }
		// }
		//
		// if (min > max) {
		// max = 1.0;
		// min = 0.0;
		// }

		int i = 1;
		for (SingleValue e : values.getValues()) {
			Probability p = probabilities.get(e.getId());
			if (p != null) {
				Double probability = p.get(SimilarityType.DIRECT).getAccuracy();
				if (probability == null || probability.isNaN()) {
					probability = 0.0;

				}

				// else {
				// probability = (probability - min) / (diff);
				// }
				// if (probability < 0.1) {
				// probability = 0.1;
				// }

				// probability = i * perValue;

				probabilitySum += probability;

				if (probability > threshold) {
					Object o = e.getValue();

					if (o instanceof List) {
						List<?> oList = (List<?>) o;

						for (Object o2 : oList) {
							list.add(new ProbabilityPair(o2, probability));
						}

					} else {

						list.add(new ProbabilityPair(o, probability));
					}
				} else {
					int fdfd = 234324;
				}

			} else {
				int wfsdf = 5;
			}
			i++;
		}
		FusedValue fv = baseFuser.fuse(list, null);
		if (fv != null && fv.getSingle() == null) {
			int fdfdf = 123;
		}
		// if (probabilitySum > threshold) {
		return fv;
		// } else {
		// return null;
		// }
	}

	@Override
	public FusedValue fuseList(MatchedValues values, TableScores probabilities, double threshold) {
		SimilarityMeasure sm = sc.getMeasure(values.getDataType());

		List<ClusterValue> clusterValues = new ArrayList<>();

		for (SingleValue e : values.getValues()) {
			Probability p = probabilities.get(e.getId());
			if (p != null) {
				Double probability = probabilities.get(e.getId()).get(SimilarityType.LIST_OVERLAP).getAccuracy();

				if (probability != null) {

					Object o = e.getValue();
					if (o instanceof List) {

						for (Object o2 : ((List<?>) o)) {
							clusterValues.add(new ClusterValue(o2, probability));
						}

					} else {

						clusterValues.add(new ClusterValue(o, probability));
					}
				}
			}

		}

		SortedSet<Cluster> clusters = clusterFuser.fuse(clusterValues, baseFuser, sm);

		clusters = clusters.stream().filter((e) -> {
			return e.getScore() > threshold;
		}).collect(Collectors.toCollection(() -> new TreeSet<Cluster>()));

		clusters = otsuThresholder.threshold(clusters);

		if (clusters.size() == 0)
			return null;
		else
			return new FusedValue(clusters);
	}

}
