package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import de.mannheim.uni.expansion.tablefusion.analysis.Accuracy;
import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.Cluster;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ClusterFuser.ClusterValue;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.fusing.Fuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.ListFuser;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.utils.OtsuThresholder;
import de.mannheim.uni.expansion.utils.ProbabilityPair;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class AbstractWeightedScoreFuser implements Fuser, ListFuser {
	private final SimilarityCalculator sc;
	private final OtsuThresholder otsuThresholder;
	private final ClusterFuser clusterFuser;
	private final BaseFuser baseFuser;

	public AbstractWeightedScoreFuser(SimilarityCalculator sc, OtsuThresholder otsuThresholder, ClusterFuser clusterFuser, BaseFuser simpleFuser) {
		this.sc = sc;
		this.otsuThresholder = otsuThresholder;
		this.clusterFuser = clusterFuser;
		this.baseFuser = simpleFuser;
	}

	@Override
	public FusedValue fuse(MatchedValues values, TableScores probabilities, double threshold) {

		List<ValueProbabilityPair> list = new ArrayList<>();
		int accuracyCountSum = 0;

		for (SingleValue e : values.getValues()) {
			Probability p = probabilities.get(e.getId());
			Accuracy accuracy = p.get(SimilarityType.DIRECT);
			Double probability = accuracy.getAccuracy();
			if (probability == null || probability.isNaN()) {
				probability = 0.0;

			}

			if (probability > threshold) {
				accuracyCountSum += p.get(SimilarityType.DIRECT).getTotal();
			}

		}

		for (SingleValue e : values.getValues()) {
			Probability p = probabilities.get(e.getId());
			if (p != null) {
				Accuracy accuracy = p.get(SimilarityType.DIRECT);
				Double probability = accuracy.getAccuracy();
				if (probability == null || probability.isNaN()) {
					probability = 0.0;

				}

				if (probability > threshold) {
					probability = (probability * accuracy.getTotal()) / accuracyCountSum;
					Object o = e.getValue();

					if (o instanceof List) {
						List<?> oList = (List<?>) o;

						for (Object o2 : oList) {
							list.add(new ProbabilityPair(o2, probability));
						}

					} else {

						list.add(new ProbabilityPair(o, probability));
					}
				} else {
					int fdfd = 234324;
				}

			} else {
				int wfsdf = 5;
			}
		}

		// if (probabilitySum > threshold) {
		return baseFuser.fuse(list, null);
		// } else {
		// return null;
		// }
	}

	@Override
	public FusedValue fuseList(MatchedValues values, TableScores probabilities, double threshold) {
		SimilarityMeasure sm = sc.getMeasure(values.getDataType());

		List<ClusterValue> clusterValues = new ArrayList<>();

		for (SingleValue e : values.getValues()) {
			Probability p = probabilities.get(e.getId());
			if (p != null) {
				Double probability = probabilities.get(e.getId()).get(SimilarityType.LIST_OVERLAP).getAccuracy();

				if (probability != null) {

					Object o = e.getValue();
					if (o instanceof List) {

						for (Object o2 : ((List<?>) o)) {
							clusterValues.add(new ClusterValue(o2, probability));
						}

					} else {

						clusterValues.add(new ClusterValue(o, probability));
					}
				}
			}

		}

		SortedSet<Cluster> clusters = clusterFuser.fuse(clusterValues, baseFuser, sm);

		clusters = otsuThresholder.threshold(clusters);

		if (clusters.size() == 0)
			return null;
		else
			return new FusedValue(clusters);
	}

}
