package de.mannheim.uni.expansion.tablefusion.core.matching;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.mannheim.uni.expansion.data.tables.model.CellId;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MatchedValues implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	final List<SingleValue> values;
	private final ColumnDataType columnDataType;

	public List<SingleValue> getValues() {
		return Collections.unmodifiableList(values);
	}

	public ColumnDataType getDataType() {

		return columnDataType;
	}

	MatchedValues(List<SingleValue> list, ColumnDataType columnDataType, boolean identifier) {
		this.values = list;
		this.columnDataType = columnDataType;
	}

	public MatchedValues(List<SingleValue> list, ColumnDataType columnDataType) {
		this.values = list;
		this.columnDataType = columnDataType;
	}

	public MatchedValues(Map<CellId, Object> map, ColumnDataType columnDataType) {

		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>();

		for (Entry<CellId, Object> e : map.entrySet()) {
			values.add(new SingleValue(e.getKey(), e.getValue()));
		}

		this.values = values;
		this.columnDataType = columnDataType;
	}

	public MatchedValues(MatchedValues mv1, MatchedValues mv2) {
		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>(mv1.getValues());
		values.addAll(mv2.getValues());

		this.values = values;
		this.columnDataType = mv1.getDataType();

	}

	public MatchedValues(MatchedValues mv) {
		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>(mv.getValues());

		this.values = values;
		this.columnDataType = mv.getDataType();

	}

	public MatchedValues(CellId id, Object object, ColumnDataType columnDataType) {
		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>();
		values.add(new SingleValue(id, object));

		this.values = values;
		this.columnDataType = columnDataType;

	}

	public MatchedValues(SingleValue sv, ColumnDataType columnDataType) {
		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>();
		values.add(sv);

		this.values = values;
		this.columnDataType = columnDataType;

	}

	public MatchedValues(MatchedValues mv, SingleValue singleValue) {
		List<SingleValue> values = new ArrayList<MatchedValues.SingleValue>(mv.getValues());
		values.add(singleValue);

		this.values = values;
		this.columnDataType = mv.getDataType();
	}

	@Override
	public String toString() {
		return values.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((values == null) ? 0 : values.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MatchedValues other = (MatchedValues) obj;
		if (values == null) {
			if (other.values != null)
				return false;
		} else if (!values.equals(other.values))
			return false;
		return true;
	}

	public static class SingleValue implements Serializable {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private final CellId id;
		private final Object value;

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((id == null) ? 0 : id.hashCode());
			result = prime * result + ((value == null) ? 0 : value.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			SingleValue other = (SingleValue) obj;
			if (id == null) {
				if (other.id != null)
					return false;
			} else if (!id.equals(other.id))
				return false;
			if (value == null) {
				if (other.value != null)
					return false;
			} else if (!value.equals(other.value))
				return false;
			return true;
		}

		public CellId getId() {
			return id;
		}

		public Object getValue() {
			return value;
		}

		public SingleValue(CellId id, Object value) {
			this.id = id;
			this.value = value;
		}

		@Override
		public String toString() {
			return "[id=" + id + ", value=" + value + "]";
		}

	}
}
