package de.mannheim.uni.expansion.tablefusion.core.matching.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.CellId;
import de.mannheim.uni.expansion.data.tables.model.ColumnIdx;
import de.mannheim.uni.expansion.data.tables.model.RowIdx;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchCreator;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class BasicGroupMatcher implements MatchCreator {

	private final AnnotatedTablesRepository annotatedTableData;

	private final DBpediaRepositoryNewInterface dbPediaRepositoryNew;

	private final OntologyRepository ontologyRepository;

	@Inject
	public BasicGroupMatcher(AnnotatedTablesRepository annotatedTableData, DBpediaRepositoryNewInterface dbPediaRepositoryNew,
			OntologyRepository ontologyRepository) {
		this.annotatedTableData = annotatedTableData;

		this.dbPediaRepositoryNew = dbPediaRepositoryNew;
		this.ontologyRepository = ontologyRepository;
	}

	@Override
	public Map<TripleId, MatchedValues> matchColumns(AnnotatedTableId id, Set<ColumnIdx> columns) {
		AnnotatedTableMetaData metaData = annotatedTableData.getMetaData(id);

		Map<TripleId, MatchedValues> groupedMap = new HashMap<TripleId, MatchedValues>();

		// int numHeaderRow = metaData.getNumHeaderRows();

		for (ColumnIdx columnIdx : columns) {

			ClassId dBpediaClass = metaData.getDbPediaClass();
			PropertyId dBpediaProperty = metaData.getProperties().get(columnIdx.getColumnIdx());
			ColumnDataType cdt = ontologyRepository.getDataType(dBpediaClass, dBpediaProperty);

			Map<Integer, InstanceId> rowInstances = metaData.getInstances();

			Map<Integer, Object> cellValues = annotatedTableData.getColumn(id, columnIdx.getColumnIdx());

			for (Entry<Integer, Object> value : cellValues.entrySet()) {
				int instanceIndex = value.getKey();
				Object val = value.getValue();
				if (cdt == ColumnDataType.numeric && !(val instanceof Double) && (val instanceof String)) {
					// TODO: Wrong Place for fix
					String s = (String) val;
					s = s.replace("&nbsp;", "").trim();
					try {
						Double d = Double.valueOf(s);
						val = d;
					} catch (NumberFormatException fe) {
						if (s.length() > 0)
							System.err.println("WARNING: Value not numeric: " + val);
						val = null;
					}
				}
				if (val != null) {
					TripleId mid = new TripleId(dBpediaClass, dBpediaProperty, rowInstances.get(instanceIndex));
					MatchedValues mv = new MatchedValues(new CellId(id, columnIdx, new RowIdx(instanceIndex)), val, cdt);

					groupedMap.put(mid, mv);
				}

			}

		}

		return groupedMap;
	}

	@Override
	public Map<TripleId, MatchedValues> matchColumns(AnnotatedTableId id, ColumnIdx columnIndex) {
		AnnotatedTableMetaData metaData = annotatedTableData.getMetaData(id);

		Map<TripleId, MatchedValues> groupedMap = new HashMap<TripleId, MatchedValues>();

		ClassId dBpediaClass = metaData.getDbPediaClass();
		PropertyId dBpediaProperty = metaData.getProperties().get(columnIndex.getColumnIdx());

		Map<Integer, InstanceId> rowInstances = metaData.getInstances();

		// int numHeaderRow = metaData.getNumHeaderRows();

		ColumnDataType cdt = ontologyRepository.getDataType(dBpediaClass, dBpediaProperty);

		Map<Integer, Object> cellValues = annotatedTableData.getColumn(id, columnIndex.getColumnIdx());

		for (Entry<Integer, Object> value : cellValues.entrySet()) {

			int instanceIndex = value.getKey();

			InstanceId instanceId = rowInstances.get(instanceIndex);

			if (instanceId != null) {

				Object val = value.getValue();
				if (cdt == ColumnDataType.numeric && !(val instanceof Double) && (val instanceof String)) {
					// TODO: Wrong Place for fix
					String s = (String) val;
					s = s.replace("&nbsp;", "").trim();
					try {
						Double d = Double.valueOf(s);
						val = d;
					} catch (NumberFormatException fe) {
						val = null;
					}
				}
				if (val != null) {

					TripleId mid = new TripleId(dBpediaClass, dBpediaProperty, instanceId);
					MatchedValues mv = new MatchedValues(new CellId(id, columnIndex, new RowIdx(instanceIndex)), value.getValue(), cdt);

					groupedMap.put(mid, mv);
				}
			}

		}

		return groupedMap;
	}
}
