package de.mannheim.uni.expansion.tablefusion.core.matching.impl;

import java.util.Map;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.CellId;
import de.mannheim.uni.expansion.data.tables.model.ColumnIdx;
import de.mannheim.uni.expansion.data.tables.model.RowIdx;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.columnmatches.ColumnMatchesRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchCreator;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.SynchronizedMatchedValuesMapMerger;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class ColumnMatchesGrouper implements MatchCreator {
	private final ColumnMatchesRepository columnMatchesRepository;
	private final AnnotatedTablesRepository tablesRepository;

	private final DBpediaRepositoryNewInterface dbPediaRepositoryNew;

	private final OntologyRepository ontologyRepository;

	@Inject
	public ColumnMatchesGrouper(ColumnMatchesRepository columnMatchesRepository, AnnotatedTablesRepository tablesRepository,
			DBpediaRepositoryNewInterface dbPediaRepositoryNew, OntologyRepository ontologyRepository) {
		this.columnMatchesRepository = columnMatchesRepository;
		this.tablesRepository = tablesRepository;

		this.dbPediaRepositoryNew = dbPediaRepositoryNew;
		this.ontologyRepository = ontologyRepository;
	}

	@Override
	public Map<TripleId, MatchedValues> matchColumns(AnnotatedTableId id, Set<ColumnIdx> columns) {
		AnnotatedTableMetaData metaData = tablesRepository.getMetaData(id);

		ClassId classId = metaData.getDbPediaClass();

		SynchronizedMatchedValuesMapMerger mapMerger = new SynchronizedMatchedValuesMapMerger();

		for (ColumnIdx columnIndex : columns) {

			PropertyId propertyId = metaData.getProperties().get(columnIndex);

			Set<RowIdx> instancesWithMatch = columnMatchesRepository.getInstancesWithMatch(id, columnIndex);

			// ColumnDataType cdt =
			// tablesRepository.getData(id).getColumns().get(metaData.getCol(propertyId)).getDataType();
			ColumnDataType cdt = ontologyRepository.getDataType(classId, propertyId);

			for (RowIdx rowIdx : instancesWithMatch) {

				InstanceId instanceId = this.tablesRepository.getMetaData(id).getInstances().get(rowIdx);

				TripleId mid = new TripleId(classId, propertyId, instanceId);

				CellId cellId = new CellId(id, columnIndex, rowIdx);
				mapMerger.merge(mid, new SingleValue(cellId, columnMatchesRepository.getMatch(cellId).getReadableString()), cdt);

			}

		}

		return mapMerger.getCleanMap();
	}

	@Override
	public Map<TripleId, MatchedValues> matchColumns(AnnotatedTableId id, ColumnIdx columnIndex) {

		AnnotatedTableMetaData metaData = tablesRepository.getMetaData(id);

		PropertyId propertyId = metaData.getProperties().get(columnIndex.getColumnIdx());
		ClassId classId = metaData.getDbPediaClass();

		SynchronizedMatchedValuesMapMerger mapMerger = new SynchronizedMatchedValuesMapMerger();

		Set<RowIdx> instancesWithMatch = columnMatchesRepository.getInstancesWithMatch(id, columnIndex);

		// ColumnDataType cdt =
		// tablesRepository.getData(id).getColumns().get(metaData.getCol(propertyId)).getDataType();

		ColumnDataType cdt = ontologyRepository.getDataType(classId, propertyId);

		AnnotatedTableMetaData md = this.tablesRepository.getMetaData(id);

		for (RowIdx rowIdx : instancesWithMatch) {

			InstanceId instanceId = md.getInstances().get(rowIdx.getRowIdx());
			if (instanceId != null) {

				TripleId mid = new TripleId(classId, propertyId, instanceId);

				CellId cellId = new CellId(id, columnIndex.getColumnIdx(), rowIdx.getRowIdx());
				mapMerger.merge(mid, new SingleValue(cellId, columnMatchesRepository.getMatch(cellId).getReadableString()), cdt);
			}

		}

		return mapMerger.getCleanMap();
	}

}
