package de.mannheim.uni.expansion.tablefusion.core.scoring.packs;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.ColumnIdx;
import de.mannheim.uni.expansion.data.tables.model.RowIdx;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;

public abstract class CombinedScores implements ScoresContainer {

	@Override
	public TableScores getTableScores(ClassId classId, PropertyId propertyId, InstanceId instanceId) {
		return new CombinedTableScores(classId, propertyId, instanceId);
	}

	@Override
	public abstract Probability getScore(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId id, ColumnIdx columnIdx,
			RowIdx rowIdx);

	public abstract boolean containsKey(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId tableId, ColumnIdx columnIdx,
			RowIdx rowIdx);

	// protected abstract Set<AnnotatedTableId> keySet(ClassId classId,
	// PropertyId propertyId, InstanceId instanceId);

	class CombinedTableScores implements TableScores {

		private final ClassId classId;
		private final PropertyId propertyId;
		private final InstanceId instanceId;

		private CombinedTableScores(ClassId classId, PropertyId propertyId, InstanceId instanceId) {
			this.classId = classId;
			this.instanceId = instanceId;
			this.propertyId = propertyId;
		}

		@Override
		public Probability get(AnnotatedTableId tableId, ColumnIdx columnIdx, RowIdx rowIdx) {
			return CombinedScores.this.getScore(classId, propertyId, instanceId, tableId, columnIdx, rowIdx);
		}

		@Override
		public boolean containsKey(AnnotatedTableId tableId, ColumnIdx columnIdx, RowIdx rowIdx) {
			return CombinedScores.this.containsKey(classId, propertyId, instanceId, tableId, columnIdx, rowIdx);
		}

		// @Override
		// public Set<AnnotatedTableId> keySet() {
		// return CombinedScores.this.keySet(classId, propertyId, instanceId);
		// }
		//
		// @Override
		// public Set<Entry<AnnotatedTableId, Probability>> entrySet() {
		// Set<Entry<AnnotatedTableId, Probability>> set = new HashSet<>();
		// Set<AnnotatedTableId> keySet = keySet();
		//
		// for (AnnotatedTableId id : keySet) {
		// set.add(new CombinedEntry(id));
		// }
		// return set;
		// }
		//
		// class CombinedEntry implements Entry<AnnotatedTableId, Probability> {
		//
		// private final AnnotatedTableId annotatedTableId;
		//
		// public CombinedEntry(AnnotatedTableId annotatedTableId) {
		// this.annotatedTableId = annotatedTableId;
		// }
		//
		// @Override
		// public AnnotatedTableId getKey() {
		// return annotatedTableId;
		// }
		//
		// @Override
		// public Probability getValue() {
		// return get(this.annotatedTableId);
		// }
		//
		// @Override
		// public Probability setValue(Probability value) {
		// throw new UnsupportedOperationException();
		// }
		// }

	}

}
