package de.mannheim.uni.expansion.tablefusion.core.scoring.packs;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.ColumnIdx;
import de.mannheim.uni.expansion.data.tables.model.RowIdx;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;

public class MultipliedScores extends CombinedScores {
	private final ScoresContainer main;
	private final ScoresContainer weights;
	private final boolean noScoreWhenMissing;

	public MultipliedScores(ScoresContainer main, ScoresContainer weights) {
		this.main = main;
		this.weights = weights;
		this.noScoreWhenMissing = true;
	}

	public MultipliedScores(ScoresContainer main, ScoresContainer weights, boolean noScoreWhenMissing) {
		this.main = main;
		this.weights = weights;
		this.noScoreWhenMissing = noScoreWhenMissing;
	}

	@Override
	public Probability getScore(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId id, ColumnIdx columnIdx, RowIdx rowIdx) {

		Probability probability = main.getScore(classId, propertyId, instanceId, id, columnIdx, rowIdx);
		Probability weight = weights.getScore(classId, propertyId, instanceId, id, columnIdx, rowIdx);
		if (probability == null) {
			return null;
		} else if (weight != null) {
			return probability.multiply(weight);
		} else if (noScoreWhenMissing) {
			return null;
		} else {
			return probability;
		}

	}

	@Override
	public boolean containsKey(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId tabledId, ColumnIdx columnIdx, RowIdx rowIdx) {
		if (main.getScore(classId, propertyId, instanceId, tabledId, columnIdx, rowIdx) != null) {
			return true;
		}

		return false;
	}

}
