package de.mannheim.uni.expansion.tablefusion.core.scoring.packs;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.ColumnIdx;
import de.mannheim.uni.expansion.data.tables.model.RowIdx;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;

public class RegressionScoresClassPropertyId extends CombinedScores {
	private final Map<ClassPropertyId, Double> interceptsMap;
	private final String[] names;
	private final ScoresContainer[] containers;
	private final Map<ClassPropertyId, double[]> weightsMap;

	public RegressionScoresClassPropertyId(String[] names, ScoresContainer[] containers, Map<ClassPropertyId, double[]> weightsMap,
			Map<ClassPropertyId, Double> interceptsMap) {
		this.containers = containers;
		this.weightsMap = Collections.unmodifiableMap(weightsMap);
		this.names = names;
		this.interceptsMap = Collections.unmodifiableMap(interceptsMap);
	}

	@Override
	public Probability getScore(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId id, ColumnIdx columnIdx, RowIdx rowIdx) {
		if (this.containsKey(classId, propertyId, instanceId, id, columnIdx, rowIdx)) {
			double[] weights = weightsMap.get(new ClassPropertyId(classId, propertyId));
			double intercept = interceptsMap.get(new ClassPropertyId(classId, propertyId));

			Probability[] probabilities = new Probability[containers.length];
			for (int i = 0; i < containers.length; i++) {
				Probability probability = containers[i].getScore(classId, propertyId, instanceId, id, columnIdx, rowIdx);
				;
				if (probability != null) {
					probabilities[i] = probability;
				} else {
					probabilities[i] = null;
				}
			}

			Probability prob = new Probability(probabilities, weights, intercept);
			// if (prob.get(SimilarityType.DIRECT).getAccuracy() < 0.0) {
			// return new Probability(new Accuracy(0.0,
			// prob.get(SimilarityType.DIRECT).getTotal()));
			// }
			return prob;
		} else {
			return null;
		}
	}

	@Override
	public boolean containsKey(ClassId classId, PropertyId propertyId, InstanceId instanceId, AnnotatedTableId tabledId, ColumnIdx columnIdx, RowIdx rowIdx) {
		if (!weightsMap.containsKey(new ClassPropertyId(classId, propertyId))) {
			return false;
		}

		return true;
	}

	// @Override
	// protected Set<AnnotatedTableId> keySet(ClassId classId, PropertyId
	// propertyId, InstanceId instanceId) {
	// Set<AnnotatedTableId> keySet = new HashSet<>();
	// for (ScoresContainer sc : containers) {
	// keySet.addAll(sc.getTableScores(classId, propertyId,
	// instanceId).keySet());
	// }
	// return keySet;
	// }

	public Map<String, Double> getWeights(ClassPropertyId classPropertyId) {
		Map<String, Double> ret = new HashMap<>();
		for (int i = 0; i < names.length; i++) {
			ret.put(names[i], weightsMap.get(classPropertyId)[i]);
		}
		return ret;
	}

	public Double getIntercept(ClassPropertyId classPropertyId) {
		return interceptsMap.get(classPropertyId);
	}

	public Map<ClassPropertyId, Map<String, Double>> getWeightsMap() {
		return weightsMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			Map<String, Double> map = new HashMap<>();
			for (int i = 0; i < e.getValue().length; i++) {
				map.put(names[i], e.getValue()[i]);
			}
			return map;
		}));
	}

	public Map<ClassPropertyId, Double> getInterceptsMap() {
		return this.interceptsMap;
	}

}
