package de.mannheim.uni.expansion.tablefusion.core.scoring.regression;

import java.util.ArrayList;
import java.util.List;

import de.mannheim.uni.expansion.tablefusion.core.fusing.Fuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.FuserRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserOptionsBuilder;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategyRepository;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class SimpleRegressionClusterer {

	private final SimilarityCalculator similarityCalculator;

	private final NewFuserOptionsBuilder fuserOptionsBuilder = new NewFuserOptionsBuilder(FusionStrategyRepository.NORMAL, false);

	private final FuserRepository fuserRepository;

	public SimpleRegressionClusterer(SimilarityCalculator similarityCalculator, FuserRepository fuserRepository) {
		super();
		this.similarityCalculator = similarityCalculator;
		this.fuserRepository = fuserRepository;
	}

	public List<RegressionCluster> getClusters(ColumnDataType cdt, List<SingleValue> singleValues) {
		Fuser fuser = fuserRepository.get(cdt, fuserOptionsBuilder);

		List<RegressionCluster> retClusters = new ArrayList<>();

		for (SingleValue sv : singleValues) {
			if (retClusters.isEmpty()) {
				List<SingleValue> values = new ArrayList<>();
				values.add(sv);
				retClusters.add(new RegressionCluster(values, sv.getValue()));
			} else {
				boolean added = false;
				for (int i = 0; i < retClusters.size(); i++) {
					if (similarityCalculator.isEqual(cdt, sv.getValue(), retClusters.get(i).getFusedValue())) {
						List<SingleValue> values = new ArrayList<>(retClusters.get(i).getValues());
						values.add(sv);
						Object fusedValue = fuser.fuse(new MatchedValues(values, cdt), null, 0.0);
						retClusters.set(i, new RegressionCluster(values, fusedValue));
						break;
					}
				}
				if (!added) {
					List<SingleValue> values = new ArrayList<>();
					values.add(sv);
					retClusters.add(new RegressionCluster(values, sv.getValue()));
				}
			}
		}
		return retClusters;
	}
}
