package de.mannheim.uni.expansion.tablefusion.evaluation;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.utils.concurrent.Parallel;

public class CountTypes {

	private final AnnotatedTablesRepository repository;

	@Inject
	public CountTypes(AnnotatedTablesRepository repository) {
		this.repository = repository;
	}

	public void printOutNumbers() {
		AtomicInteger lists = new AtomicInteger();
		AtomicInteger strings = new AtomicInteger();
		AtomicInteger numerics = new AtomicInteger();
		AtomicInteger dates = new AtomicInteger();
		AtomicInteger others = new AtomicInteger();

		ConcurrentLinkedQueue<TableColumn> columns = new ConcurrentLinkedQueue<>();

		try {
			new Parallel<AnnotatedTableId>().foreach(repository.getAllTableIds(), (id) -> {

				Table data = repository.getData(id);
				columns.addAll(data.getColumns());

			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			new Parallel<TableColumn>().foreach(columns, (tc) -> {
				for (Object o : tc.getValues().values()) {
					if (o instanceof String) {
						strings.incrementAndGet();
					} else if (o instanceof Double) {
						numerics.incrementAndGet();
					} else if (o instanceof Date) {
						dates.incrementAndGet();
					} else if (o instanceof List) {
						lists.incrementAndGet();
					} else {
						others.incrementAndGet();
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("Number of Strings:  " + strings.get());
		System.out.println("Number of Numerics: " + numerics.get());
		System.out.println("Number of Dates:    " + dates.get());
		System.out.println("Number of Lists:    " + lists.get());
		System.out.println("Number of Other:    " + others.get());
	}
}
