package de.mannheim.uni.expansion.tablefusion.evaluation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.fusing.FuserRepository;
import de.mannheim.uni.expansion.tablefusion.core.scoring.ProbabilitiesCalculator;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValueData;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionRunner;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.CSVSerializer;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

@Deprecated
public class EvaluationExporter {

	private final GroupedValuesRepository groupedValuesRepository;

	private final String optimizingFolder;
	private final String evaluationDirPath;
	private final String evaluationOutputPath;

	private final String evaluationInputPath;

	private final FuserRepository fusers;

	private final FusionRunner fusionRunner;

	private final DBpediaRepositoryNewInterface dBpediaRepository;

	private final EvaluationNormalizer evaluationNormalizer;

	private final CSVSerializer csvSerializer;

	private final OntologyRepository ontologyRepository;

	private final ValueEvaluator accuracyCalculator;

	private final ProbabilitiesCalculator probabilitiesCalculator;

	private enum AccuracyClasses {
		BAD, MEDIUM, GOOD;

		public static AccuracyClasses getAccuracyClass(double accuracy) {

			if (accuracy >= 0.66666666) {
				return GOOD;
			} else if (accuracy >= 0.333333) {
				return MEDIUM;
			} else {
				return BAD;
			}

		}
	}

	private enum DataType {
		String, Reference, Numeric, Date;

	}

	public DataType getDataType(ClassPropertyId classPropertyId) {

		ColumnDataType dataType = ontologyRepository.getDataType(classPropertyId);

		if (dataType == ColumnDataType.reference) {
			return DataType.Reference;
		} else if (dataType == ColumnDataType.string) {
			return DataType.String;
		} else if (dataType == ColumnDataType.numeric) {
			return DataType.Numeric;
		} else if (dataType == ColumnDataType.date) {
			return DataType.Date;
		}

		throw new IllegalStateException("Couldn't detect DataType of:" + classPropertyId.toString());

	}

	@Inject
	public EvaluationExporter(GroupedValuesRepository groupedValuesRepository, @Named(NamedProperties.OPTIMIZING_FOLDER) String optimizingFolder,
			FuserRepository fusers, FusionRunner fusionRunner, DBpediaRepositoryNewInterface dBpediaRepository, EvaluationNormalizer evaluationNormalizer,
			CSVSerializer csvSerializer, ValueEvaluator accuracyCalculator, OntologyRepository ontologyRepository,
			ProbabilitiesCalculator probabilitiesCalculator) {
		this.groupedValuesRepository = groupedValuesRepository;
		this.optimizingFolder = optimizingFolder;
		this.fusers = fusers;
		this.fusionRunner = fusionRunner;
		this.dBpediaRepository = dBpediaRepository;
		this.evaluationNormalizer = evaluationNormalizer;
		this.csvSerializer = csvSerializer;
		this.accuracyCalculator = accuracyCalculator;
		this.ontologyRepository = ontologyRepository;
		this.probabilitiesCalculator = probabilitiesCalculator;

		evaluationDirPath = this.optimizingFolder + "/evaluation";
		evaluationOutputPath = evaluationDirPath + "/output";
		evaluationInputPath = evaluationDirPath + "/input";
	}

	// public void exportThresholdData() throws IOException {
	//
	// List<FusionMethodType> methodTypes = new ArrayList<>();
	// methodTypes.add(FusionMethodType.REFERENCE);
	// methodTypes.add(FusionMethodType.STRING);
	// methodTypes.add(FusionMethodType.NUMERIC);
	// methodTypes.add(FusionMethodType.DATE);
	// methodTypes.add(FusionMethodType.LINK);
	//
	// List<FusionPreproccesser> preprocessors = new ArrayList<>();
	// preprocessors.add(FusionPreproccesser.NONE);
	//
	// Map<TripleId, MatchedValues> groupedValues =
	// groupedValuesRepository.getGroupedValues();
	//
	// Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted =
	// groupedValuesRepository.getSourceSorted(0);
	//
	// Map<ClassPropertyId, Double> statistics =
	// groupedValuesRepository.getStatistics(0);
	//
	// File evaluationDir = new File(evaluationDirPath);
	// MKDIR.createDirUnchecked(evaluationDir);
	//
	// File evaluationOutputDir = new File(evaluationOutputPath);
	// MKDIR.createDirUnchecked(evaluationOutputDir);
	//
	// File evaluationInputDir = new File(evaluationInputPath);
	// MKDIR.createDirUnchecked(evaluationInputDir);
	//
	// int resultSizeOverlap = 40;
	// int resultSizeNew = 120;
	// double maxProportion = 1.0;
	//
	// Map<FusionRun, Map<TripleId, Object>> fusedValuesMap =
	// fusionRunner.run(methodTypes, FusionPreproccesser.NONE,
	// FusionRunType.NORMAL, groupedValues);
	//
	// Map<TripleId, Object> fusedValues = CHMHelper.flattenMap(fusedValuesMap);
	//
	// Map<GroupKey, List<ClassPropertyId>> groupedClassProperties =
	// getGroupedClassProperties(averageAccuracy);
	//
	// Map<GroupKey, Map<TripleId, Object>> groupedFusedValues =
	// groupFusedValues(fusedValues, averageAccuracy);
	//
	// List<PairData> pairData = new ArrayList<>();
	// List<ValueData> pairDataNoOverLap = new ArrayList<>();
	//
	// for (Entry<GroupKey, List<ClassPropertyId>> e :
	// groupedClassProperties.entrySet()) {
	// Map<TripleId, Object> fusedValuesTemp =
	// groupedFusedValues.get(e.getKey());
	// Map<ClassPropertyId, Double> statisticsTemp =
	// evaluationNormalizer.getStatistics(fusedValuesTemp);
	//
	// pairData.addAll(getPairData(fusedValuesTemp, statisticsTemp,
	// sourceSorted, resultSizeOverlap, maxProportion));
	//
	// if (e.getKey().getAccuracyClass() == AccuracyClasses.GOOD) {
	// pairDataNoOverLap.addAll(getPairDataNoOverLap(fusedValuesTemp,
	// statisticsTemp, sourceSorted, resultSizeNew, maxProportion));
	// }
	// }
	//
	// write(evaluationOutputPath + "/overlap.csv", pairData);
	//
	// writeValueData(evaluationOutputPath + "/new.csv", pairDataNoOverLap);
	//
	// // List<PairData> pairData =
	// // getPairData(FusionMethodType.STRING_EXACT_MAJORITY, groupedValues,
	// // statistics, sourceSorted, resultSize, maxProportion);
	// //
	// // write(thresholdsOutputPath + "/string.csv", pairData, (fusedValue,
	// // dbPediaValue) -> {
	// // return "";
	// // } , (o) -> {
	// // return o.toString();
	// // });
	// //
	// // List<PairData> datePairData =
	// // getPairData(FusionMethodType.DATE_EXACT_MEDIAN, groupedValues,
	// // statistics, sourceSorted, resultSize, maxProportion);
	// // SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
	// //
	// // write(thresholdsOutputPath + "/date.csv", datePairData, (fusedValue,
	// // dbPediaValue) -> {
	// // return "";
	// // } , (o) -> {
	// // return dt.format((Date) o);
	// // });
	// //
	// // List<PairData> numericPairData =
	// // getPairData(FusionMethodType.NUMERIC_EXACT_MEDIAN, groupedValues,
	// // statistics, sourceSorted, resultSize, maxProportion);
	// // DecimalFormat dfmt = new DecimalFormat(",000.0");
	// //
	// // write(thresholdsOutputPath + "/numeric.csv", numericPairData,
	// // (fusedValue, dbPediaValue) -> {
	// // double sim = similarityCalculator.numericSimilarity.get(dbPediaValue,
	// // fusedValue);
	// // if (sim > 0.95) {
	// // return "1";
	// // }
	// // return "0";
	// //
	// // } , (o) -> {
	// // return dfmt.format((Double) o);
	// // });
	// }

	public Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> randomizeSourceSorted(Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sortedMap) {

		Random rnd = new Random(28443350);

		Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> shuffledMap = new HashMap<>();

		for (Entry<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> e : sortedMap.entrySet()) {
			ArrayList<InstanceId> list = new ArrayList<>(e.getValue().keySet());
			Collections.shuffle(list, rnd);
			LinkedHashMap<InstanceId, Integer> shuffledLinkedHashMap = new LinkedHashMap<>();
			for (InstanceId instanceId : list) {
				shuffledLinkedHashMap.put(instanceId, e.getValue().get(instanceId));
			}
			shuffledMap.put(e.getKey(), shuffledLinkedHashMap);
		}

		return shuffledMap;

	}

	private List<ValueData> getPairDataNoOverLap(Map<TripleId, Object> fusedValues, Map<ClassPropertyId, Double> statistics,
			Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted, int resultSize, double maxProportion) {
		Map<ClassPropertyId, Map<InstanceId, ValueData>> valueMap = evaluationNormalizer.getValueMapNoOverlap(fusedValues);
		Map<ClassPropertyId, Double> normalizedStatistics = evaluationNormalizer.normalizeStatistics(statistics, valueMap, resultSize, maxProportion);

		Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> shuffledMap = randomizeSourceSorted(sourceSorted);
		List<ValueData> pairData = evaluationNormalizer.getPairData(valueMap, normalizedStatistics, shuffledMap, resultSize);

		return pairData;
	}

	// private List<PairData> getPairData(Map<TripleId, Object> fusedValues,
	// Map<ClassPropertyId, Double> statistics,
	// Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted,
	// int resultSize, double maxProportion) {
	// Map<ClassPropertyId, Map<InstanceId, PairData>> valuePairsMap =
	// evaluationNormalizer.getValuePairsMap(fusedValues);
	//
	// Map<ClassPropertyId, Double> normalizedStatistics =
	// evaluationNormalizer.normalizeStatistics(statistics, valuePairsMap,
	// resultSize, maxProportion);
	//
	// Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> shuffledMap =
	// randomizeSourceSorted(sourceSorted);
	//
	// List<PairData> pairData = evaluationNormalizer.getPairData(valuePairsMap,
	// normalizedStatistics, shuffledMap, resultSize);
	// return pairData;
	//
	// }

	private Map<GroupKey, List<ClassPropertyId>> getGroupedClassProperties(Map<ClassPropertyId, Double> averageAccuracy) {

		Creator<List<ClassPropertyId>> arrayListCreator = () -> {
			return new ArrayList<ClassPropertyId>();
		};

		Map<GroupKey, List<ClassPropertyId>> groupedClassProperties = new HashMap<>();
		for (Entry<ClassPropertyId, Double> e : averageAccuracy.entrySet()) {
			DataType dataType = this.getDataType(e.getKey());
			AccuracyClasses accuracyClass = AccuracyClasses.getAccuracyClass(e.getValue());
			GroupKey groupKey = new GroupKey(accuracyClass, dataType);

			List<ClassPropertyId> list = CHMHelper.getFromMap(groupedClassProperties, groupKey, arrayListCreator);
			list.add(e.getKey());

		}

		return groupedClassProperties;
	}

	public Map<GroupKey, Map<TripleId, Object>> groupFusedValues(Map<TripleId, Object> fusedValues, Map<ClassPropertyId, Double> averageAccuracy) {
		Creator<Map<TripleId, Object>> hashMapCreator = () -> {
			return new HashMap<TripleId, Object>();
		};

		ConcurrentHashMap<GroupKey, Map<TripleId, Object>> groupedFusedResults = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {

			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
			DataType dataType = this.getDataType(classPropertyId);
			AccuracyClasses accuracyClass = AccuracyClasses.getAccuracyClass(averageAccuracy.get(classPropertyId));
			GroupKey groupKey = new GroupKey(accuracyClass, dataType);

			Map<TripleId, Object> fusedValuesMap = CHMHelper.getFromCHM(groupedFusedResults, groupKey, hashMapCreator);

			synchronized (fusedValuesMap) {
				fusedValuesMap.put(e.getKey(), e.getValue());
			}
		} , "Group Fused Values");

		return groupedFusedResults;
	}

	private static class GroupKey {
		private final AccuracyClasses accuracyClass;
		private final DataType dataType;

		public GroupKey(AccuracyClasses accuracyClass, DataType dataType) {
			super();
			this.accuracyClass = accuracyClass;
			this.dataType = dataType;
		}

		@Override
		public String toString() {
			return "GroupKey [accuracyClass=" + accuracyClass + ", dataType=" + dataType + "]";
		}

		public AccuracyClasses getAccuracyClass() {
			return accuracyClass;
		}

		public DataType getDataType() {
			return dataType;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((accuracyClass == null) ? 0 : accuracyClass.hashCode());
			result = prime * result + ((dataType == null) ? 0 : dataType.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			GroupKey other = (GroupKey) obj;
			if (accuracyClass != other.accuracyClass)
				return false;
			if (dataType != other.dataType)
				return false;
			return true;
		}

	}

}
