package de.mannheim.uni.expansion.tablefusion.learning;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.CellId;
import de.mannheim.uni.expansion.data.tables.model.TableColumnId;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.SynchronizedMatchedValuesMapMerger;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.PersistentParallel3;

public class LearningSetCreator {

	@Inject
	public LearningSetCreator() {

	}

	private final Creator<List<ValueWithIdentifier>> listCreator = () -> {
		return new ArrayList<>();
	};

	public Map<Integer, CreatedSets> createByTableProperty(int numberOfSets, GroupedValues groupedValues) {

		ConcurrentHashMap<TableColumnId, SortedMap<Integer, ValueWithIdentifier>> valuesByTable = new ConcurrentHashMap<TableColumnId, SortedMap<Integer, ValueWithIdentifier>>();

		try {
			PersistentParallel3.runAndFinish(groupedValues.entrySet(), (e) -> {
				PropertyId propertyId = e.getKey().getPropertyId();

				for (SingleValue sv : e.getValue().getValues()) {
					CellId cellId = sv.getId();
					TableColumnId tableProperty = cellId.getTableColumnId();
					SortedMap<Integer, ValueWithIdentifier> values = CHMHelper.getFromCHM(valuesByTable, tableProperty, () -> new TreeMap<>());

					synchronized (values) {
						values.put(cellId.getRowIdx().getRowIdx(), new ValueWithIdentifier(e.getKey(), sv));
					}

				}
			} , "createByTableProperty");
		} catch (Exception e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}

		double percent = 1.0 / numberOfSets;
		Map<Integer, CreatedSets> createdSets = new HashMap<Integer, LearningSetCreator.CreatedSets>();

		PersistentParallel3<Entry<TableColumnId, SortedMap<Integer, ValueWithIdentifier>>> parallel = new PersistentParallel3<Entry<TableColumnId, SortedMap<Integer, ValueWithIdentifier>>>(
				"LearningSetCreator");

		for (int i = 0; i < numberOfSets; i++) {
			SynchronizedMatchedValuesMapMerger learningSetMerger = new SynchronizedMatchedValuesMapMerger();
			SynchronizedMatchedValuesMapMerger testingSetMerger = new SynchronizedMatchedValuesMapMerger();

			final int currentI = i;
			//
			// Map<TripleId, MatchedValues> learningSet = new HashMap<TripleId,
			// MatchedValues>();
			// Map<TripleId, MatchedValues> testingSet = new HashMap<TripleId,
			// MatchedValues>();

			try {
				parallel.foreach(valuesByTable.entrySet(), (e) -> {
					int smallSetSize = (int) (percent * e.getValue().size());

					int s = 0;
					int sStart = smallSetSize * currentI;
					int sEnd = smallSetSize * (currentI + 1);
					for (ValueWithIdentifier vwi : e.getValue().values()) {
						if (s >= sStart & s < sEnd) {
							testingSetMerger.merge(vwi.getTripleId(), vwi.getSingleValue(), groupedValues.get(vwi.getTripleId()).getDataType());
							// MatchedValues mv =
							// learningSet.get(vwi.getTripleId());
							// if (mv == null) {
							// mv = new MatchedValues(vwi.getSingleValue(),
							// groupedValues.get(vwi.getTripleId()).getDataType());
							// } else {
							// mv = new MatchedValues(mv, vwi.getSingleValue());
							// }
							//
							// learningSet.put(vwi.getTripleId(), mv);
						} else {
							learningSetMerger.merge(vwi.getTripleId(), vwi.getSingleValue(), groupedValues.get(vwi.getTripleId()).getDataType());
							// MatchedValues mv =
							// testingSet.get(vwi.getTripleId());
							// if (mv == null) {
							// mv = new MatchedValues(vwi.getSingleValue(),
							// groupedValues.get(vwi.getTripleId()).getDataType());
							// } else {
							// mv = new MatchedValues(mv, vwi.getSingleValue());
							// }
							//
							// testingSet.put(vwi.getTripleId(), mv);
						}

						s++;
					}
				});
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			createdSets.put(i, new CreatedSets(new GroupedValues(learningSetMerger.getCleanMap()), new GroupedValues(testingSetMerger.getCleanMap())));
		}

		parallel.shutdown();

		return createdSets;

	}

	public static class CreatedSets implements Serializable {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		private final GroupedValues learningSet;

		private final GroupedValues testingSet;

		private CreatedSets(GroupedValues learningSet, GroupedValues testingSet) {
			this.learningSet = learningSet;
			this.testingSet = testingSet;
		}

		public GroupedValues getLearningSet() {
			return learningSet;
		}

		public GroupedValues getTestingSet() {
			return testingSet;
		}

	}

	private static class TableProperty {
		private final AnnotatedTableId tableId;
		private final PropertyId propertyId;

		private TableProperty(AnnotatedTableId tableId, PropertyId propertyId) {
			this.tableId = tableId;
			this.propertyId = propertyId;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((propertyId == null) ? 0 : propertyId.hashCode());
			result = prime * result + ((tableId == null) ? 0 : tableId.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			TableProperty other = (TableProperty) obj;
			if (propertyId == null) {
				if (other.propertyId != null)
					return false;
			} else if (!propertyId.equals(other.propertyId))
				return false;
			if (tableId == null) {
				if (other.tableId != null)
					return false;
			} else if (!tableId.equals(other.tableId))
				return false;
			return true;
		}

	}

	private static class ValueWithIdentifier {
		private final TripleId tripleId;
		private final SingleValue singleValue;

		public ValueWithIdentifier(TripleId tripleId, SingleValue singleValue) {
			this.tripleId = tripleId;
			this.singleValue = singleValue;
		}

		public TripleId getTripleId() {
			return tripleId;
		}

		public SingleValue getSingleValue() {
			return singleValue;
		}

	}
}
