package de.mannheim.uni.expansion.tablefusion.optimization;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class ScoreEquivalenceClusterer {

	@Inject
	public ScoreEquivalenceClusterer() {

	}

	public List<Cluster> addValue(List<Cluster> clusters, SimilarityMeasure measure, BaseFuser baseFuser, ValueProbabilityPair value) {
		clusters = new ArrayList<>(clusters);
		boolean added = false;
		for (int i = 0; i < clusters.size(); i++) {
			if (measure.isEqual(clusters.get(i).getValue().getSingle(), value.getValue())) {
				List<ValueProbabilityPair> valueList = new ArrayList<>(clusters.get(i).getValues());
				valueList.add(value);

				clusters.set(i, new Cluster(baseFuser.fuse(valueList), valueList));

				added = true;
				break;
			}
		}
		if (!added) {
			List<ValueProbabilityPair> valueList = new ArrayList<>();
			valueList.add(value);

			clusters.add(new Cluster(FusedValue.fromObject(value.getValue()), valueList));
		}

		return clusters;
	}

	public Cluster getHighestRatedCluster(List<Cluster> clusters) {
		clusters = new ArrayList<>(clusters);
		Collections.sort(clusters);
		return clusters.get(clusters.size() - 1);
	}

	public static class Cluster implements Comparable<Cluster> {
		private final FusedValue fusedValue;
		private final List<ValueProbabilityPair> values;
		private final double score;

		public Cluster(FusedValue fusedValue, List<ValueProbabilityPair> values) {
			super();
			this.fusedValue = fusedValue;
			this.values = Collections.unmodifiableList(values);
			this.score = values.parallelStream().mapToDouble(ValueProbabilityPair::getScore).sum();
		}

		public List<ValueProbabilityPair> getValues() {
			return values;
		}

		@Override
		public int compareTo(Cluster o) {
			return Double.compare(getScore(), o.getScore());
		}

		public FusedValue getValue() {
			return fusedValue;
		}

		public double getScore() {
			return score;
		}

	}
}
