package de.mannheim.uni.expansion.tablefusion.optimization;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.function.BiFunction;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.FuserRepository;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.ProbabilitiesCalculator;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdClassPropertyId;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdConstant;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdPropertyId;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.tablefusion.services.GroupedValuesFilterer;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionDataType;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionPreproccesser;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategy;
import de.mannheim.uni.expansion.tablefusion.services.fusion.strategies.ScorerRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.expansion.utils.ProbabilityPair;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class ScoreFusionThresholdFinderUsingScorer {

	private final ValueEvaluator valueEvaluator;
	private final ProbabilitiesCalculator probabilitiesCalculator;

	private final GroupedValuesFilterer groupedValuesFilterer;

	private final ScorerRepository scorerRepository;

	private final ScoreEquivalenceClusterer equivalenceClusterer;

	private final SimilarityCalculator similarityCalculator;
	private final FuserRepository fuserRepository;

	@Inject
	public ScoreFusionThresholdFinderUsingScorer(ValueEvaluator valueEvaluator, ProbabilitiesCalculator probabilitiesCalculator,
			GroupedValuesFilterer groupedValuesFilterer, ScorerRepository scorerRepository, ScoreEquivalenceClusterer equivalenceClusterer,
			SimilarityCalculator similarityCalculator, FuserRepository fuserRepository) {
		super();
		this.valueEvaluator = valueEvaluator;
		this.probabilitiesCalculator = probabilitiesCalculator;
		this.groupedValuesFilterer = groupedValuesFilterer;
		this.scorerRepository = scorerRepository;
		this.equivalenceClusterer = equivalenceClusterer;
		this.similarityCalculator = similarityCalculator;
		this.fuserRepository = fuserRepository;
	}

	// public ThresholdConstant findOptimumThresholdForPageRank() {
	// // TODO Auto-generated method stub
	// return null;
	// }

	public ThresholdContainer findOptimumThresholdCrossValidation(GroupedValues groupedValues, ScoresContainer scores, ThresholdType thresholdType,
			List<FusionDataType> dataTypes, List<FusionPreproccesser> preprocessors, FusionStrategy fusionRunType, boolean fuseLists) {
		// return new ThresholdConstant(0.9);

		switch (thresholdType) {
		case ClassPropertyId:
			return getThresholdClassProperty(groupedValues, scores);

		case General:
			return getThreshold(groupedValues, scores);

		case PropertyId:
			return getThresholdPropertyId(groupedValues, scores);

		}
		return null;
		//
	}

	// @Override
	// public ThresholdConstant findOptimumThresholdListExpansion(Map<Integer,
	// FilteredValues> setsFilters, GroupedValues groupedValues, FusionStrategy
	// strategy,
	// boolean print) {
	//
	// List<GroupedValues> listOfGroupedValues = new ArrayList<>();
	//
	// for (Entry<Integer, FilteredValues> setsFilter : setsFilters.entrySet())
	// {
	// GroupedValues filteredGroupedValues =
	// groupedValuesFilterer.filterBySetFilter(setsFilter.getValue(),
	// groupedValues);
	//
	// listOfGroupedValues.add(filteredGroupedValues);
	// }
	//
	// return new ThresholdConstant(getThreshold(listOfGroupedValues,
	// strategy.getScoringMethod()));
	//
	// }

	private <T> Map<T, Double> getThresholdGeneric(GroupedValues groupedValues, ScoresContainer probabilities, BiFunction<TripleId, ColumnDataType, T> getKey,
			Double alternative) {
		boolean group = false;
		int length = 21;
		double step = 0.05;

		ConcurrentHashMap<T, ConcurrentLinkedQueue<Double>> correctMap = new ConcurrentHashMap<>();

		ConcurrentHashMap<T, ConcurrentLinkedQueue<Double>> incorrectMap = new ConcurrentHashMap<>();

		try {
			parallel.foreach(groupedValues.entrySet(), (matchedValues) -> {

				SimilarityMeasure measure = similarityCalculator.getMeasure(matchedValues.getValue().getDataType());
				BaseFuser baseFuser = fuserRepository.getBaseFuser(matchedValues.getValue().getDataType());
				TripleId tid = matchedValues.getKey();

				T key = getKey.apply(tid, matchedValues.getValue().getDataType());

				ConcurrentLinkedQueue<Double> correct = CHMHelper.getFromCHM(correctMap, key, () -> new ConcurrentLinkedQueue<>());
				ConcurrentLinkedQueue<Double> incorrect = CHMHelper.getFromCHM(incorrectMap, key, () -> new ConcurrentLinkedQueue<>());

				if (group) {

					List<ScoreEquivalenceClusterer.Cluster> clusters = new ArrayList<>();

					for (SingleValue singleValue : matchedValues.getValue().getValues()) {
						Probability probability = probabilities.getScore(tid.getClassId(), tid.getPropertyId(), tid.getInstanceId(), singleValue.getId());
						if (probability != null) {
							Double score = probability.get(SimilarityType.DIRECT).getAccuracy();

							clusters = equivalenceClusterer.addValue(clusters, measure, baseFuser, new ProbabilityPair(singleValue.getValue(), score));
						}
					}

					for (ScoreEquivalenceClusterer.Cluster c : clusters) {

						Boolean valueCorrect = valueEvaluator.correct(tid, c.getValue());
						if (valueCorrect != null) {
							if (valueCorrect) {
								correct.add(c.getScore());
							} else {
								incorrect.add(c.getScore());
							}
						}
					}

				} else {
					for (SingleValue singleValue : matchedValues.getValue().getValues()) {
						Probability probability = probabilities.getScore(tid.getClassId(), tid.getPropertyId(), tid.getInstanceId(), singleValue.getId());
						if (probability != null) {
							Double score = probability.get(SimilarityType.DIRECT).getAccuracy();

							FusedValue fv = FusedValue.fromObject(singleValue.getValue());

							Boolean valueCorrect = valueEvaluator.correct(tid, fv);
							if (valueCorrect != null) {
								if (valueCorrect) {
									correct.add(score);
								} else {
									incorrect.add(score);
								}
							}

						}

					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		Set<T> cpidKeys = new HashSet<>();
		cpidKeys.addAll(correctMap.keySet());
		cpidKeys.addAll(incorrectMap.keySet());

		Map<T, Double> thresholdMap = new HashMap<>();

		PersistentParallel3.runAndFinish(cpidKeys, (cpid) -> {

			ConcurrentLinkedQueue<Double> correct = correctMap.get(cpid);
			ConcurrentLinkedQueue<Double> incorrect = incorrectMap.get(cpid);

			thresholdMap.put(cpid, getThreshold(correct, incorrect, length, step, alternative));

		} , "ClassPropertyThresholds");

		return thresholdMap;
	}

	private ThresholdContainer getThresholdPropertyId(GroupedValues groupedValues, ScoresContainer scores) {
		return new ThresholdPropertyId(getThresholdGeneric(groupedValues, scores, (tid, cdt) -> {
			return tid.getPropertyId();
		} , ((ThresholdConstant) getThreshold(groupedValues, scores)).getScore()));

		//
	}

	private final PersistentParallel3<Entry<TripleId, MatchedValues>> parallel = new PersistentParallel3<>("THREHSOLD FINDER");

	private ThresholdContainer getThreshold(GroupedValues groupedValues, ScoresContainer scores) {
		return new ThresholdConstant(getThresholdGeneric(groupedValues, scores, (tid, cdt) -> {
			return Boolean.TRUE;
		} , null).get(Boolean.TRUE));

		// int length = 21;
		// double step = 0.05;
		// ConcurrentLinkedQueue<Double> correct = new
		// ConcurrentLinkedQueue<>();
		// ConcurrentLinkedQueue<Double> incorrect = new
		// ConcurrentLinkedQueue<>();
		//
		// try {
		// parallel.foreach(groupedValues.entrySet(), (matchedValues) -> {
		// TripleId tid = matchedValues.getKey();
		// for (SingleValue singleValue : matchedValues.getValue().getValues())
		// {
		// Probability probability = probabilities.getScore(tid.getClassId(),
		// tid.getPropertyId(), tid.getInstanceId(), singleValue.getId());
		// if (probability != null) {
		// Double score = probability.get(SimilarityType.DIRECT).getAccuracy();
		// FusedValue fv = FusedValue.fromObject(singleValue.getValue());
		//
		// Boolean valueCorrect = valueEvaluator.correct(tid, fv);
		// if (valueCorrect != null) {
		// if (valueCorrect) {
		// correct.add(score);
		// } else {
		// incorrect.add(score);
		// }
		// }
		// }
		//
		// }
		// });
		// } catch (Exception e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		//
		// return new ThresholdConstant(getThreshold(correct, incorrect, length,
		// step, null));
	}

	private ThresholdClassPropertyId getThresholdClassProperty(GroupedValues groupedValues, ScoresContainer scores) {

		return new ThresholdClassPropertyId(getThresholdGeneric(groupedValues, scores, (tid, cdt) -> {
			return new ClassPropertyId(tid);
		} , ((ThresholdConstant) getThreshold(groupedValues, scores)).getScore()));

		// )

		// ThresholdConstant constant = (ThresholdConstant)
		// getThreshold(groupedValues, probabilities);
		// Double thresholdConstant = constant.getScore();
		//
		// int length = 21;
		// double step = 0.05;
		//
		// ConcurrentHashMap<ClassPropertyId, ConcurrentLinkedQueue<Double>>
		// correctMap = new ConcurrentHashMap<>();
		//
		// ConcurrentHashMap<ClassPropertyId, ConcurrentLinkedQueue<Double>>
		// incorrectMap = new ConcurrentHashMap<>();
		//
		// try {
		// parallel.foreach(groupedValues.entrySet(), (matchedValues) -> {
		// TripleId tid = matchedValues.getKey();
		//
		// ClassPropertyId cpid = new ClassPropertyId(tid);
		//
		// ConcurrentLinkedQueue<Double> correct =
		// CHMHelper.getFromCHM(correctMap, cpid, () -> new
		// ConcurrentLinkedQueue<>());
		// ConcurrentLinkedQueue<Double> incorrect =
		// CHMHelper.getFromCHM(incorrectMap, cpid, () -> new
		// ConcurrentLinkedQueue<>());
		//
		// for (SingleValue singleValue : matchedValues.getValue().getValues())
		// {
		// Probability probability = probabilities.getScore(tid.getClassId(),
		// tid.getPropertyId(), tid.getInstanceId(), singleValue.getId());
		// if (probability != null) {
		// Double score = probability.get(SimilarityType.DIRECT).getAccuracy();
		// FusedValue fv = FusedValue.fromObject(singleValue.getValue());
		//
		// Boolean valueCorrect = valueEvaluator.correct(tid, fv);
		// if (valueCorrect != null) {
		// if (valueCorrect) {
		// correct.add(score);
		// } else {
		// incorrect.add(score);
		// }
		// }
		// }
		//
		// }
		// });
		// } catch (Exception e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		//
		// Set<ClassPropertyId> cpidKeys = new HashSet<>();
		// cpidKeys.addAll(correctMap.keySet());
		// cpidKeys.addAll(incorrectMap.keySet());
		//
		// Map<ClassPropertyId, Double> thresholdMap = new HashMap<>();
		//
		// PersistentParallel3.runAndFinish(cpidKeys, (cpid) -> {
		//
		// ConcurrentLinkedQueue<Double> correct = correctMap.get(cpid);
		// ConcurrentLinkedQueue<Double> incorrect = incorrectMap.get(cpid);
		//
		// thresholdMap.put(cpid, getThreshold(correct, incorrect, length, step,
		// thresholdConstant));
		//
		// } , "ClassPropertyThresholds");
		//
		// return new ThresholdClassPropertyId(thresholdMap);

	}

	private double getThreshold(Collection<Double> correct, Collection<Double> incorrect, int length, double step, Double alternative) {

		if (!correct.isEmpty() || !incorrect.isEmpty()) {
			double precision[] = new double[length];
			double recall[] = new double[length];
			double fMeasure[] = new double[length];

			for (int i = 0; i < length; i++) {
				Double probabilityThreshold = step * i;

				int tp = 0;
				int fp = 0;
				int tn = 0;
				int fn = 9;

				for (Double d : correct) {
					if (d >= probabilityThreshold) {
						tp++;
					} else {
						fn++;
					}
				}

				for (Double d : incorrect) {
					if (d < probabilityThreshold) {
						tn++;
					} else {
						fp++;
					}
				}

				precision[i] = ((double) tp) / (tp + fp);
				recall[i] = ((double) tp) / (tp + fn);

				fMeasure[i] = 1.1 * ((0.2 * precision[i] * recall[i]) / (0.1 * precision[i] + recall[i]));
				fMeasure[i] = 2 * ((precision[i] * recall[i]) / (precision[i] + recall[i]));

			}

			double maxFMeasure = 0.0;
			double maxThreshold = 0.0;
			int maxThresholdIndex = 0;
			int maxStep = 0;

			for (int i = 0; i < length; i++) {
				// double precision = sumPrecision / count;
				// double recall = sumRecall / count;

				if (fMeasure[i] > maxFMeasure) {
					maxFMeasure = fMeasure[i];
					maxThreshold = step * i;
					maxThresholdIndex = i;
				}
			}

			// for (int i = maxThresholdIndex + 1; i < length; i++) {
			// if (precision[i] > precision[maxThresholdIndex]) {
			// if (3 * (precision[i] - precision[maxThresholdIndex]) >
			// (Math.abs(fMeasure[i] - fMeasure[maxThresholdIndex]))) {
			// maxThreshold = step * i;
			// maxStep = i;
			// maxThresholdIndex = i;
			// }
			// }
			//
			// }

			// if (alternative != null && alternative > maxThreshold)
			// return (maxThreshold + alternative) / 2.0;
			// else
			return maxThreshold;
		} else {
			if (alternative != null) {
				return alternative;
			} else {
				return 0.0;
			}
		}
	}
}
