package de.mannheim.uni.expansion.tablefusion.profiling.timestamps;

import java.util.List;
import java.util.Map.Entry;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParserJSON;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesGzReader;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.tableprocessor.model.TableColumn;

public class TimeStampQualityProfiler {

	private final AnnotatedTablesRepository annotatedTablesRepository;
	private final AnnotatedTablesGzReader annotatedTablesGzReader;

	@Inject
	public TimeStampQualityProfiler(AnnotatedTablesRepository annotatedTablesRepository,
			@Named(DataNamedProperties.TABLES_MAPPING_FOLDER) String mappingsFolder, @Named(DataNamedProperties.TABLES_MAPPING_ENDING) FileEnding fileEnding,
			@Named(DataNamedProperties.TABLES_TYPE) TableType tableType) {

		this.annotatedTablesRepository = annotatedTablesRepository;
		AnnotatedTableParserJSON parser = new AnnotatedTableParserJSON(null);

		this.annotatedTablesGzReader = new AnnotatedTablesGzReader(mappingsFolder, fileEnding, tableType, parser, null);
	}

	public void analyze() {

		annotatedTablesGzReader.runForEachTable((data) -> {
			if (data == null) {
				int dfdf = 123;
			}
			if (annotatedTablesRepository.getMetaData(data.getId()) == null) {
				System.out.println("META DATA NULL, THIS SHOULD NOT HAPPEN");
				return;
			}
			printOut("TABLE-TITLE", annotatedTablesRepository.getMetaData(data.getId()).getMaxYearInTableTitle(), data.getMetaData().getTableTitle());
			printOut("PAGE-TITLE", annotatedTablesRepository.getMetaData(data.getId()).getMaxYearInPageTitle(), data.getMetaData().getPageTitle());
			printOut("BEFORE-TABLE", annotatedTablesRepository.getMetaData(data.getId()).getMaxYearBeforeText(), data.getMetaData().getTextBeforeTable());
			printOut("AFTER-TABLE", annotatedTablesRepository.getMetaData(data.getId()).getMaxYearAfterText(), data.getMetaData().getTextAfterTable());
			printOut("CONTEXT", annotatedTablesRepository.getMetaData(data.getId()).getMaxYearInContextBefore(),
					data.getMetaData().getTableContextTimeStampBeforeTable());
			if (annotatedTablesRepository.getMetaData(data.getId()).getPropertiesMaxYear() != null) {
				AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(data.getId());
				for (Entry<Integer, Integer> e : md.getPropertiesMaxYear().entrySet()) {
					List<TableColumn> columns = data.getTableData().getColumns();
					printOut("COLUMN-HEADER", e.getValue(), (String) columns.get(e.getKey()).getHeader());
				}
			}

		});

	}

	public void printOut(String type, Integer year, String string) {

		if (year != null) {
			synchronized (System.out) {
				System.out.println(type + ":");
				System.out.println(" -- MAX-YEAR: " + year);
				System.out.println(" -- String: " + string);
				System.out.println("==");
			}
		}
	}
}
