package de.mannheim.uni.expansion.tablefusion.profiling.timestamps;

import java.util.function.BinaryOperator;

import de.mannheim.uni.expansion.tablefusion.profiling.accumulation.Accumulateable;

public class TimeStampsAggregatedResults implements Accumulateable<TimeStampsAggregatedResults> {

	private static final BinaryOperator<TimeStampsAggregatedResults> accumulator = (a, b) -> {
		return new TimeStampsAggregatedResults(a, b);
	};

	private final int totalTables;
	private final int tablesWithHeaderTimestamps;
	private final int tablesWithTableTimestamps;
	private final int tablesHTTimestamps;
	private final int totalMappedColumns;
	private final int mappedColumnsWithTableTimestamp;
	private final int mappedColumnsWithColumnTimestamp;
	private final int mappedColumnsWithHeaderTimestamp;
	private final int mappedColumnsWithTCTimestamp;
	private final int mappedColumnsWithHTTimestamp;
	private final int mappedColumnsWithHCTimestamp;
	private final int mappedColumnsWithAllTimestamps;

	public TimeStampsAggregatedResults() {
		super();
		this.totalTables = 0;
		this.tablesWithHeaderTimestamps = 0;
		this.tablesWithTableTimestamps = 0;
		this.tablesHTTimestamps = 0;
		this.totalMappedColumns = 0;
		this.mappedColumnsWithTableTimestamp = 0;
		this.mappedColumnsWithColumnTimestamp = 0;
		this.mappedColumnsWithHeaderTimestamp = 0;
		this.mappedColumnsWithTCTimestamp = 0;
		this.mappedColumnsWithHTTimestamp = 0;
		this.mappedColumnsWithHCTimestamp = 0;
		this.mappedColumnsWithAllTimestamps = 0;
	}

	private TimeStampsAggregatedResults(int totalTables, int tablesWithHeaderTimestamps, int tablesWithTableTimestamps, int tablesHTTimestamps,
			int totalMappedColumns, int mappedColumnsWithHeaderTimestamp, int mappedColumnsWithTableTimestamp, int mappedColumnsWithColumnTimestamp,
			int mappedColumnsWithTCTimestamp, int mappedColumnsWithHTTimestamp, int mappedColumnsWithHCTimestamp, int mappedColumnsWithAllTimestamps) {
		super();
		this.totalTables = totalTables;
		this.tablesWithHeaderTimestamps = tablesWithHeaderTimestamps;
		this.tablesWithTableTimestamps = tablesWithTableTimestamps;
		this.tablesHTTimestamps = tablesHTTimestamps;
		this.totalMappedColumns = totalMappedColumns;
		this.mappedColumnsWithTableTimestamp = mappedColumnsWithTableTimestamp;
		this.mappedColumnsWithColumnTimestamp = mappedColumnsWithColumnTimestamp;
		this.mappedColumnsWithHeaderTimestamp = mappedColumnsWithHeaderTimestamp;
		this.mappedColumnsWithTCTimestamp = mappedColumnsWithTCTimestamp;
		this.mappedColumnsWithHTTimestamp = mappedColumnsWithHTTimestamp;
		this.mappedColumnsWithHCTimestamp = mappedColumnsWithHCTimestamp;
		this.mappedColumnsWithAllTimestamps = mappedColumnsWithAllTimestamps;
	}

	public TimeStampsAggregatedResults(TimeStampsAggregatedResults a, TimeStampsAggregatedResults b) {
		this.totalTables = a.getTotalTables() + b.getTotalTables();
		this.tablesWithHeaderTimestamps = a.getTablesWithHeaderTimestamps() + b.getTablesWithHeaderTimestamps();
		this.tablesWithTableTimestamps = a.getTablesWithTableTimestamps() + b.getTablesWithTableTimestamps();
		this.tablesHTTimestamps = a.getTablesHTTimestamps() + b.getTablesHTTimestamps();
		this.totalMappedColumns = a.getTotalMappedColumns() + b.getTotalMappedColumns();
		this.mappedColumnsWithTableTimestamp = a.getMappedColumnsWithTableTimestamp() + b.getMappedColumnsWithTableTimestamp();
		this.mappedColumnsWithColumnTimestamp = a.getMappedColumnsWithColumnTimestamp() + b.getMappedColumnsWithColumnTimestamp();
		this.mappedColumnsWithHeaderTimestamp = a.getMappedColumnsWithHeaderTimestamp() + b.getMappedColumnsWithHeaderTimestamp();
		this.mappedColumnsWithTCTimestamp = a.getMappedColumnsWithTCTimestamp() + b.getMappedColumnsWithTCTimestamp();
		this.mappedColumnsWithHTTimestamp = a.getMappedColumnsWithHTTimestamp() + b.getMappedColumnsWithHTTimestamp();
		this.mappedColumnsWithHCTimestamp = a.getMappedColumnsWithHCTimestamp() + b.getMappedColumnsWithHCTimestamp();
		this.mappedColumnsWithAllTimestamps = a.getMappedColumnsWithAllTimestamps() + b.getMappedColumnsWithAllTimestamps();
	}

	public int getTotalTables() {
		return totalTables;
	}

	public int getTablesWithHeaderTimestamps() {
		return tablesWithHeaderTimestamps;
	}

	public int getTablesWithTableTimestamps() {
		return tablesWithTableTimestamps;
	}

	public int getTablesHTTimestamps() {
		return tablesHTTimestamps;
	}

	public int getTotalMappedColumns() {
		return totalMappedColumns;
	}

	public int getMappedColumnsWithTableTimestamp() {
		return mappedColumnsWithTableTimestamp;
	}

	public int getMappedColumnsWithColumnTimestamp() {
		return mappedColumnsWithColumnTimestamp;
	}

	public int getMappedColumnsWithHeaderTimestamp() {
		return mappedColumnsWithHeaderTimestamp;
	}

	public int getMappedColumnsWithTCTimestamp() {
		return mappedColumnsWithTCTimestamp;
	}

	public int getMappedColumnsWithHTTimestamp() {
		return mappedColumnsWithHTTimestamp;
	}

	public int getMappedColumnsWithHCTimestamp() {
		return mappedColumnsWithHCTimestamp;
	}

	public int getMappedColumnsWithAllTimestamps() {
		return mappedColumnsWithAllTimestamps;
	}

	@Override
	public BinaryOperator<TimeStampsAggregatedResults> getAccumulator() {
		return TimeStampsAggregatedResults.accumulator;
	}

	public static TimeStampsAggregatedResults createForColumnWithNoTS(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0);
	}

	public static TimeStampsAggregatedResults createForColumnWithT(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0);

	}

	public static TimeStampsAggregatedResults createForColumnWithC(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0);

	}

	public static TimeStampsAggregatedResults createForColumnWithH(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0);

	}

	public static TimeStampsAggregatedResults createForColumnWithHT(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0);

	}

	public static TimeStampsAggregatedResults createForColumnWithHC(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0);
	}

	public static TimeStampsAggregatedResults createForColumnWithTC(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0);

	}

	public static TimeStampsAggregatedResults createForColumnWithHTC(int val) {
		return new TimeStampsAggregatedResults(0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1);

	}

}
