package de.mannheim.uni.expansion.tablefusion.profiling.timestamps;

import java.util.function.BinaryOperator;

import de.mannheim.uni.expansion.tablefusion.profiling.accumulation.Accumulateable;

public class TimeStampsSingleResults implements Accumulateable<TimeStampsSingleResults> {

	private static final BinaryOperator<TimeStampsSingleResults> accumulator = (a, b) -> {
		return new TimeStampsSingleResults(a, b);
	};

	private int total;
	private int atLeastOne;
	private int beforeText;
	private int afterText;
	private int contextBefore;
	private int tableTitle;
	private int pageTitle;
	private int columnHeader;
	private int rowCell;

	public TimeStampsSingleResults() {
		super();
		this.total = 0;
		this.atLeastOne = 0;
		this.beforeText = 0;
		this.afterText = 0;
		this.contextBefore = 0;
		this.tableTitle = 0;
		this.pageTitle = 0;
		this.columnHeader = 0;
		this.rowCell = 0;
	}

	public TimeStampsSingleResults(TimeStampsSingleResults a, TimeStampsSingleResults b) {
		this.total = a.getTotal() + b.getTotal();
		this.atLeastOne = a.getAtLeastOne() + b.getAtLeastOne();
		this.beforeText = a.getBeforeText() + b.getBeforeText();
		this.afterText = a.getAfterText() + b.getAfterText();
		this.contextBefore = a.getContextBefore() + b.getContextBefore();
		this.tableTitle = a.getTableTitle() + b.getTableTitle();
		this.pageTitle = a.getPageTitle() + b.getPageTitle();
		this.columnHeader = a.getColumnHeader() + b.getColumnHeader();
		this.rowCell = a.getRowCell() + b.getRowCell();
	}

	public int getAtLeastOne() {
		return atLeastOne;
	}

	public int getTotal() {
		return total;
	}

	public int getBeforeText() {
		return beforeText;
	}

	public int getAfterText() {
		return afterText;
	}

	public int getContextBefore() {
		return contextBefore;
	}

	public int getTableTitle() {
		return tableTitle;
	}

	public int getPageTitle() {
		return pageTitle;
	}

	public int getColumnHeader() {
		return columnHeader;
	}

	public int getRowCell() {
		return rowCell;
	}

	public void setRowCell(int rowCell) {
		this.rowCell = rowCell;
	}

	@Override
	public BinaryOperator<TimeStampsSingleResults> getAccumulator() {
		return accumulator;
	}

	public TimeStampsSingleResults addTotalWithoutTS(int val) {
		this.total += val;
		return this;

	}

	public TimeStampsSingleResults addTotalWithTS(int val) {
		this.total += val;
		this.atLeastOne += val;
		return this;
	}

	public TimeStampsSingleResults beforeText(int val) {
		this.beforeText += val;
		return this;
	}

	public TimeStampsSingleResults afterText(int val) {
		this.afterText += val;
		return this;
	}

	public TimeStampsSingleResults inContextVefore(int val) {
		this.contextBefore += val;
		return this;

	}

	public TimeStampsSingleResults inTableTitle(int val) {
		this.tableTitle += val;
		return this;

	}

	public TimeStampsSingleResults inPageTitle(int val) {
		this.pageTitle += val;
		return this;
	}

	public TimeStampsSingleResults inColumnHeader(int val) {
		this.columnHeader += val;
		return this;
	}

	public TimeStampsSingleResults inRowCellValue(int val) {
		this.rowCell += val;
		return this;
	}

}
