package de.mannheim.uni.expansion.tablefusion.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ListType;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.analysis.FilteredValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class GroupedValuesFilterer {

	private final DBpediaRepositoryNewInterface dBpediaRepository;
	private final OntologyRepository ontologyRepository;

	@Inject
	public GroupedValuesFilterer(DBpediaRepositoryNewInterface dBpediaRepository, OntologyRepository ontologyRepository) {
		this.dBpediaRepository = dBpediaRepository;
		this.ontologyRepository = ontologyRepository;
	}

	public GroupedValues getNonListTypes(GroupedValues groupedValues) {
		Map<TripleId, MatchedValues> filtered = groupedValues.entrySet().stream().filter((e) -> {
			return !dBpediaRepository.isListType(e.getKey().getClassPropertyId());
		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		return new GroupedValues(filtered);
	}

	public GroupedValues getListTypes(GroupedValues groupedValues) {
		Map<TripleId, MatchedValues> filtered = groupedValues.entrySet().stream().filter((e) -> {
			return dBpediaRepository.isListType(e.getKey().getClassPropertyId());
		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		return new GroupedValues(filtered);
	}

	public GroupedValues getListTypes(ListType listType, GroupedValues groupedValues) {
		Map<TripleId, MatchedValues> filtered = groupedValues.entrySet().stream().filter((e) -> {
			return dBpediaRepository.isListType(listType, e.getKey().getClassPropertyId());
		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		return new GroupedValues(filtered);
	}

	public GroupedValues getDataType(ColumnDataType cdt, GroupedValues groupedValues) {
		Map<TripleId, MatchedValues> filtered = groupedValues.entrySet().stream().filter((e) -> {
			return ontologyRepository.getDataType(e.getKey().getClassPropertyId()) == cdt;
		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		return new GroupedValues(filtered);
	}

	public GroupedValues getByProperty(Set<PropertyId> set, GroupedValues groupedValues) {
		Map<TripleId, MatchedValues> filtered = groupedValues.entrySet().stream().filter((e) -> {
			return set.contains(e.getKey().getPropertyId());

		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		return new GroupedValues(filtered);
	}

	public GroupedValues filterBySetFilter(FilteredValues filter, GroupedValues groupedValues) {

		Map<TripleId, MatchedValues> ret = new HashMap<>();

		for (Entry<TripleId, MatchedValues> e : groupedValues.entrySet()) {

			Set<Object> filteredValues = filter.getFilteredValues(e.getKey());

			if (filteredValues != null) {

				List<SingleValue> newList = new ArrayList<>();

				for (SingleValue sv : e.getValue().getValues()) {
					if (sv.getValue() instanceof List<?>) {
						@SuppressWarnings("unchecked")
						List<Object> svList = new ArrayList<>((List<Object>) sv.getValue());

						for (Iterator<Object> it = svList.iterator(); it.hasNext();) {
							if (filteredValues.contains(it.next())) {
								it.remove();
							}
						}

						if (!svList.isEmpty()) {
							newList.add(new SingleValue(sv.getId(), svList));
						}
					} else {
						if (!filteredValues.contains(sv.getValue())) {
							newList.add(sv);
						}
					}
				}

				ret.put(e.getKey(), new MatchedValues(newList, e.getValue().getDataType()));
			}
		}

		return new GroupedValues(ret);
	}

}
