package de.mannheim.uni.expansion.tablefusion.services.fusion;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategy;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class NewFuserOptionsBuilder {

	final private Map<ColumnDataType, FusionStrategy> fusionStrategies;
	final private FusionStrategy defaultFusionStrategy;
	final private boolean fuseLists;

	public FusionStrategy getDefaultFusionStrategy() {
		return defaultFusionStrategy;
	}

	public Map<ColumnDataType, FusionStrategy> getFusionStrategies() {
		return Collections.unmodifiableMap(fusionStrategies);
	}

	public FusionStrategy getFusionStrategy(ColumnDataType cdt) {
		return fusionStrategies.get(cdt);
	}

	public NewFuserOptionsBuilder(FusionStrategy fusionStrategy, boolean fuseLists, ColumnDataType... columnTypes) {
		defaultFusionStrategy = null;
		this.fuseLists = fuseLists;
		this.fusionStrategies = new HashMap<ColumnDataType, FusionStrategy>();

		for (ColumnDataType cdt : columnTypes) {
			this.fusionStrategies.put(cdt, fusionStrategy);
		}
	}

	public NewFuserOptionsBuilder(FusionStrategy defaultFusionStrategy, boolean fuseLists) {
		this.defaultFusionStrategy = defaultFusionStrategy;
		this.fuseLists = fuseLists;
		this.fusionStrategies = new HashMap<ColumnDataType, FusionStrategy>();
	}

	private NewFuserOptionsBuilder(FusionStrategy defaultFusionStrategy, Map<ColumnDataType, FusionStrategy> options, FusionStrategy FusionStrategy,
			boolean fuseLists, ColumnDataType[] columnTypes) {
		this.fuseLists = fuseLists;
		this.fusionStrategies = new HashMap<ColumnDataType, FusionStrategy>(options);
		for (ColumnDataType cdt : columnTypes) {
			this.fusionStrategies.put(cdt, FusionStrategy);
		}

		this.defaultFusionStrategy = defaultFusionStrategy;
	}

	@SuppressWarnings("unused")
	private NewFuserOptionsBuilder(FusionStrategy defaultFusionStrategy, Map<ColumnDataType, FusionStrategy> options, boolean fuseLists) {
		this.fuseLists = fuseLists;
		this.fusionStrategies = new HashMap<ColumnDataType, FusionStrategy>(options);

		this.defaultFusionStrategy = defaultFusionStrategy;
	}

	public NewFuserOptionsBuilder() {
		this.fuseLists = true;
		defaultFusionStrategy = null;
		fusionStrategies = new HashMap<ColumnDataType, FusionStrategy>();
	}

	public NewFuserOptionsBuilder withNewFusionStrategy(FusionStrategy FusionStrategy, boolean fuseLists, ColumnDataType... columnTypes) {

		return new NewFuserOptionsBuilder(getDefaultFusionStrategy(), getFusionStrategies(), FusionStrategy, fuseLists, columnTypes);
	}

}
