package de.mannheim.uni.expansion.tablefusion.services.fusion;

import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.core.fusing.Fuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.FuserRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdContainer;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class NewFuserService {

	private final DBpediaRepositoryNewInterface dbPediaRepositoryNew;

	private final OntologyRepository ontologyRepository;

	private final PersistentParallel3<Entry<TripleId, MatchedValues>> parallel;

	private final FuserRepository fusers;

	@Inject
	public NewFuserService(DBpediaRepositoryNewInterface dbPediaRepositoryNew, OntologyRepository ontologyRepository, FuserRepository fusers) {
		this.dbPediaRepositoryNew = dbPediaRepositoryNew;
		this.ontologyRepository = ontologyRepository;
		this.parallel = new PersistentParallel3<Entry<TripleId, MatchedValues>>("BasicFuserService");
		this.fusers = fusers;
	}

	// public Map<TripleId, Object> fuse(Map<TripleId, MatchedValues>
	// groupedValues) {
	// Map<TripleId, Object> map = new HashMap<TripleId, Object>();
	//
	// for (Entry<TripleId, MatchedValues> entry : groupedValues.entrySet()) {
	// TripleId mid = entry.getKey();
	// ColumnDataType cdt = propertyDataTypes.get(mid.getPropertyId());
	// if (cdt.equals(ColumnDataType.numeric)) {
	// map.put(mid, averageFuser.fuse(entry.getValue()));
	//
	// } else {
	// map.put(mid, majorityFuser.fuse(entry.getValue()));
	// }
	// }
	//
	// return map;
	//
	// }

	public Map<TripleId, FusedValue> fuse(NewFuserOptionsBuilder options, GroupedValues groupedValues, boolean fuseLists) {
		return fuse(options, groupedValues, fuseLists, null, null);
	}

	public Map<TripleId, FusedValue> fuse(NewFuserOptionsBuilder options, GroupedValues groupedValues, boolean fuseLists, ScoresContainer probabilities,
			ThresholdContainer threshold) {
		ConcurrentHashMap<TripleId, FusedValue> map = new ConcurrentHashMap<TripleId, FusedValue>();

		FusedReceiver receiver = (entry, fusedValue) -> {
			if (fusedValue != null) {
				map.put(entry.getKey(), fusedValue);
			}
		};

		fuse(options, groupedValues, fuseLists, probabilities, threshold, receiver);

		return map;
	}

	public synchronized void fuse(NewFuserOptionsBuilder options, GroupedValues groupedValues, boolean fuseLists, ScoresContainer probabilities,
			ThresholdContainer thresholdContainer, FusedReceiver receiver) {

		try {
			this.parallel.foreach(groupedValues.entrySet(), new Consumer<Map.Entry<TripleId, MatchedValues>>() {

				@Override
				public void accept(Entry<TripleId, MatchedValues> entry) {
					TripleId mid = entry.getKey();
					ColumnDataType cdt = ontologyRepository.getDataType(mid.getClassId(), mid.getPropertyId());

					Fuser fuser = fusers.get(cdt, options);

					if (fuser == null) {
						return;
					}

					Double threshold = 0.0;

					if (thresholdContainer != null) {
						threshold = thresholdContainer.getScore(mid.getClassId(), mid.getPropertyId(), mid.getInstanceId());
						if (threshold == null) {
							threshold = 0.0;
						}
					}
					FusedValue fusedValue;
					if (fuseLists && dbPediaRepositoryNew.isListType(mid.getClassPropertyId())) {

						if (probabilities != null) {
							TableScores tableProperties = probabilities.getTableScores(mid.getClassId(), mid.getPropertyId(), mid.getInstanceId());
							if (tableProperties != null)
								fusedValue = fuser.fuseList(entry.getValue(), tableProperties, threshold);
							else
								return;
						} else {

							fusedValue = fuser.fuseList(entry.getValue(), null, 0.0);
						}
					} else if (probabilities != null) {
						TableScores tableProperties = probabilities.getTableScores(mid.getClassId(), mid.getPropertyId(), mid.getInstanceId());
						if (tableProperties != null)
							fusedValue = fuser.fuse(entry.getValue(), tableProperties, threshold);
						else
							return;

					} else {
						fusedValue = fuser.fuse(entry.getValue(), null, 0.0);

					}

					receiver.doWork(entry, fusedValue);

				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	@FunctionalInterface
	public interface FusedReceiver {
		void doWork(Entry<TripleId, MatchedValues> entry, FusedValue fusedValue);
	}
}
