package de.mannheim.uni.expansion.tablefusion.services.fusion.runner;

import java.util.EnumMap;
import java.util.Map;

import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public enum FusionDataType {
	REFERENCE("Reference", ColumnDataType.reference, FusionBaseMethod.MAJORTIY), //
	STRING("String", ColumnDataType.string, FusionBaseMethod.MAJORTIY), //
	NUMERIC("Numeric", ColumnDataType.numeric, FusionBaseMethod.NUMERIC_MAJORITY_MEDIAN), //
	DATE("Date", ColumnDataType.date, FusionBaseMethod.DATE_MEDIAN), //
	LINK("Link", ColumnDataType.link, FusionBaseMethod.MAJORTIY);

	private final String name;
	private final FusionBaseMethod baseMethod;
	private final ColumnDataType cdt;

	private FusionDataType(String name, ColumnDataType cdt, FusionBaseMethod baseMethod) {
		this.name = name;
		this.baseMethod = baseMethod;
		this.cdt = cdt;
	}

	public String getName() {
		return name;
	}

	public FusionBaseMethod getBaseMethod() {
		return baseMethod;
	}

	public ColumnDataType getDataType() {
		return cdt;
	}

	static private final Map<ColumnDataType, FusionDataType> typeMapping = new EnumMap<>(ColumnDataType.class);

	static {
		for (FusionDataType fdt : FusionDataType.values()) {
			if (!typeMapping.containsKey(fdt)) {
				typeMapping.put(fdt.getDataType(), fdt);
			} else {
				throw new IllegalStateException("ONE COLUMN DATA TYPE CAN AT MOST BE MAPPED TO ONE FUSIONDATATYPE");
			}
		}
	}

	public static FusionDataType get(ColumnDataType cdt) {
		return typeMapping.get(cdt);
	}

}
