package de.mannheim.uni.expansion.tablefusion.services.fusion.runner;

import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.preprocessors.FusionPreProcessor;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.preprocessors.PreprocessorOnePerHost;
import de.mannheim.uni.expansion.utils.MassConsumerHelper;

public class FusionProprocessersImpl {
	private final PreprocessorOnePerHost preprocessorOnePerHost;

	private final MassConsumerHelper<Entry<TripleId, MatchedValues>> consumerHelper;

	@Inject
	public FusionProprocessersImpl(PreprocessorOnePerHost preprocessorOnePerHost) {
		this.preprocessorOnePerHost = preprocessorOnePerHost;

		this.consumerHelper = MassConsumerHelper.withLinkedQueue("FusionPreprocessor", (e) -> {
		} , new Entry<TripleId, MatchedValues>() {

			@Override
			public TripleId getKey() {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public MatchedValues getValue() {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public MatchedValues setValue(MatchedValues value) {
				// TODO Auto-generated method stub
				return null;
			}
		});

	}

	public GroupedValues preprocessValues(FusionPreproccesser fusionPreProcessor, GroupedValues values, ScoresContainer probabilities) {
		switch (fusionPreProcessor) {
		case NONE:
			break;
		case ONE_PER_HOST:
			values = run(preprocessorOnePerHost, values, probabilities);
			break;

		}
		return values;
	}

	public GroupedValues preprocessValues(FusionPreproccesser fusionPreProcessor, GroupedValues values) {
		switch (fusionPreProcessor) {
		case NONE:
			break;
		case ONE_PER_HOST:
			values = run(preprocessorOnePerHost, values, null);
			break;

		}
		return values;
	}

	private synchronized GroupedValues run(FusionPreProcessor fusionPreProcessor, GroupedValues values, ScoresContainer probabilities) {

		ConcurrentHashMap<TripleId, MatchedValues> map = new ConcurrentHashMap<TripleId, MatchedValues>();

		try {
			consumerHelper.runAll(values.entrySet(), (entry) -> {
				map.put(entry.getKey(), fusionPreProcessor.preprocess(entry.getKey(), entry.getValue(), probabilities));
			});
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return new GroupedValues(map);
	}

}
