package de.mannheim.uni.expansion.tablefusion.services.fusion.runner;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.tablefusion.analysis.AccuracyCalculator;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdContainer;
import de.mannheim.uni.expansion.tablefusion.evaluation.analyzer.FusionRun;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserOptionsBuilder;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserService;

public class FusionRunner {

	private final NewFuserService basicFuserService;

	private final AccuracyCalculator accuracyCalculator;

	private final FusionProprocessersImpl fusionPreprocessors;

	@Inject
	public FusionRunner(NewFuserService basicFuserService, AccuracyCalculator accuracyCalculator, FusionProprocessersImpl fusionPreprocessors) {
		this.basicFuserService = basicFuserService;
		this.accuracyCalculator = accuracyCalculator;
		this.fusionPreprocessors = fusionPreprocessors;

	}

	public Map<TripleId, FusedValue> run(FusionDataType fusionDataType, FusionPreproccesser fusionPreProcessor, FusionStrategy fusionRunType,
			GroupedValues values, ScoresContainer probabilities, boolean fuseLists, ThresholdContainer probabilityThreshold) {

		NewFuserOptionsBuilder fuserOptionsBuilder = new NewFuserOptionsBuilder(fusionRunType, fuseLists, fusionDataType.getDataType());

		values = fusionPreprocessors.preprocessValues(fusionPreProcessor, values, probabilities);

		Map<TripleId, FusedValue> fusedValues = basicFuserService.fuse(fuserOptionsBuilder, values, fuseLists, probabilities, probabilityThreshold);
		// System.out.println(values.size());
		// System.out.println(fusedValues.size());

		return fusedValues;

	}

	public Map<FusionRun, Map<TripleId, FusedValue>> runAll(FusionStrategy fusionRunType, GroupedValues values, ScoresContainer probabilities,
			boolean fuseLists, ThresholdContainer probabilityThreshold) {
		return run(Arrays.asList(FusionDataType.values()), Arrays.asList(FusionPreproccesser.values()), fusionRunType, values, probabilities, fuseLists,
				probabilityThreshold);

	}

	public Map<FusionRun, Map<TripleId, FusedValue>> runAll(FusionStrategy fusionRunType, List<FusionPreproccesser> preprocessers, GroupedValues values,
			ScoresContainer probabilities, boolean fuseLists, ThresholdContainer probabilityThreshold) {
		return run(Arrays.asList(FusionDataType.values()), preprocessers, fusionRunType, values, probabilities, fuseLists, probabilityThreshold);

	}

	public Map<FusionRun, Map<TripleId, FusedValue>> run(List<FusionDataType> dataTypes, List<FusionPreproccesser> preprocessers, FusionStrategy fusionRunType,
			GroupedValues values, ScoresContainer probabilities, boolean fuseLists, ThresholdContainer probabilityThreshold) {
		Map<FusionRun, Map<TripleId, FusedValue>> fusedResultsMap = new HashMap<>();
		for (FusionDataType fusionDataType : dataTypes) {
			for (FusionPreproccesser fusionPreproccesser : preprocessers) {
				FusionRun fusionRun = new FusionRun(fusionDataType, fusionPreproccesser, fusionRunType);
				fusedResultsMap.put(fusionRun, run(fusionDataType, fusionPreproccesser, fusionRunType, values, probabilities, fuseLists, probabilityThreshold));
			}

		}

		return fusedResultsMap;
	}

	public Map<FusionRun, Map<TripleId, FusedValue>> run(List<FusionDataType> dataTypes, FusionPreproccesser fusionPreproccesser, FusionStrategy fusionRunType,
			GroupedValues values, ScoresContainer probabilities, boolean fuseLists, ThresholdContainer probabilityThreshold) {
		ArrayList<FusionPreproccesser> fusionPreprocessors = new ArrayList<>();
		fusionPreprocessors.add(fusionPreproccesser);
		return run(Arrays.asList(FusionDataType.values()), fusionPreprocessors, fusionRunType, values, probabilities, fuseLists, probabilityThreshold);

	}

	public Map<TripleId, FusedValue> run(FusionDataType fusionDataType, FusionPreproccesser fusionPreProcessor, FusionStrategy fusionRunType,
			GroupedValues values, boolean fuseLists) {
		NewFuserOptionsBuilder fuserOptionsBuilder = new NewFuserOptionsBuilder(fusionRunType, fuseLists, fusionDataType.getDataType());

		values = fusionPreprocessors.preprocessValues(fusionPreProcessor, values);

		Map<TripleId, FusedValue> fusedValues = basicFuserService.fuse(fuserOptionsBuilder, values, fuseLists);
		// FusionResults fr = accuracyCalculator.analyzeAccuracy(
		// fusionDataType.getName() + "::" + fusionDataType.getMethod() + "::" +
		// fusionPreProcessor.getName() + "::" + fusionRunType.getName(),
		// fusedValues, false);
		// accuracyStorer.addResult(fusionDataType, fusionPreProcessor,
		// fusionRunType, fr);

		return fusedValues;
	}

	public Map<FusionRun, Map<TripleId, FusedValue>> run(List<FusionDataType> dataTypes, List<FusionPreproccesser> preprocessers, FusionStrategy fusionRunType,
			GroupedValues values, boolean fuseLists) {

		Map<FusionRun, Map<TripleId, FusedValue>> fusedResultsMap = new HashMap<>();
		for (FusionDataType fusionDataType : dataTypes) {

			for (FusionPreproccesser fusionPreproccesser : preprocessers) {
				FusionRun fusionRun = new FusionRun(fusionDataType, fusionPreproccesser, fusionRunType);
				fusedResultsMap.put(fusionRun, run(fusionDataType, fusionPreproccesser, fusionRunType, values, fuseLists));
			}

		}

		return fusedResultsMap;

	}

	public Map<FusionRun, Map<TripleId, FusedValue>> runAll(FusionStrategy fusionRunType, GroupedValues values, boolean fuseLists) {
		return run(Arrays.asList(FusionDataType.values()), Arrays.asList(FusionPreproccesser.values()), fusionRunType, values, fuseLists);
	}

	public Map<FusionRun, Map<TripleId, FusedValue>> run(List<FusionDataType> methodTypes, FusionPreproccesser fusionPreproccesser, FusionStrategy normal,
			GroupedValues values, boolean fuseLists) {

		ArrayList<FusionPreproccesser> fusionPreprocessors = new ArrayList<>();
		fusionPreprocessors.add(fusionPreproccesser);
		return run(methodTypes, fusionPreprocessors, normal, values, fuseLists);

	}

}
