package de.mannheim.uni.expansion.tablefusion.services.fusion.strategies;

import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.function.BiFunction;

import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.Probability;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.expansion.utils.containers.Pair;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class WekaScorer {

	private final ValueEvaluator valueEvaluator;

	public WekaScorer(ValueEvaluator valueEvaluator) {
		this.valueEvaluator = valueEvaluator;
	}

	<T> ConcurrentHashMap<T, ConcurrentLinkedQueue<Pair<Boolean, Double[]>>> getListMap(GroupedValues groupedValues, ScoresContainer[] containers,
			BiFunction<TripleId, ColumnDataType, T> getKey) {
		ConcurrentHashMap<T, ConcurrentLinkedQueue<Pair<Boolean, Double[]>>> listMap = new ConcurrentHashMap<>();

		SortedMap<TripleId, MatchedValues> sortedMap = new TreeMap<>(groupedValues.getValues());

		PersistentParallel3.runAndFinish(sortedMap.entrySet(), (gv) -> {
			for (SingleValue sv : gv.getValue().getValues()) {

				Boolean correct = valueEvaluator.correct(gv.getKey(), FusedValue.fromObject(sv.getValue()));
				if (correct != null) {
					Double[] scores = new Double[containers.length];
					for (int i = 0; i < containers.length; i++) {
						Probability probability = containers[i].getScore(gv.getKey().getClassId(), gv.getKey().getPropertyId(), gv.getKey().getInstanceId(),
								sv.getId());
						if (probability != null)
							scores[i] = probability.get(SimilarityType.DIRECT).getAccuracy();
						else
							scores[i] = 0.0;
					}

					CHMHelper.getFromCHM(listMap, getKey.apply(gv.getKey(), gv.getValue().getDataType()),
							() -> new ConcurrentLinkedQueue<Pair<Boolean, Double[]>>()).add(new Pair<Boolean, Double[]>(correct, scores));

				}

			}
		} , "RegressionScorer");

		return listMap;
	}
}
