package de.mannheim.uni.expansion.tablefusion.services.grouping;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.mannheim.uni.expansion.tablefusion.core.matching.MatchCreator;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MatcherOptionsBuilder {

	final private Map<ColumnDataType, MatchCreator> matchers;
	final private MatchCreator defaultMatcher;
	final private MatchCreator referenceMatcher;

	public MatchCreator getReferenceMatcher() {
		return referenceMatcher;
	}

	public MatchCreator getDefaultMatcher() {
		return defaultMatcher;
	}

	public Map<ColumnDataType, MatchCreator> getMatchers() {
		return Collections.unmodifiableMap(matchers);
	}

	public MatcherOptionsBuilder(MatchCreator defaultMatcher) {
		this.defaultMatcher = defaultMatcher;
		matchers = new HashMap<ColumnDataType, MatchCreator>();
		this.referenceMatcher = null;
	}

	private MatcherOptionsBuilder(MatchCreator defaultMatcher, Map<ColumnDataType, MatchCreator> options, MatchCreator referenceMatcher, MatchCreator matcher,
			ColumnDataType[] matcherTypes) {
		this.matchers = new HashMap<ColumnDataType, MatchCreator>(options);
		for (ColumnDataType cdt : matcherTypes) {
			this.matchers.put(cdt, matcher);
		}

		this.defaultMatcher = defaultMatcher;
		this.referenceMatcher = referenceMatcher;
	}

	public MatcherOptionsBuilder() {
		defaultMatcher = null;
		matchers = new HashMap<ColumnDataType, MatchCreator>();
		referenceMatcher = null;
	}

	public MatcherOptionsBuilder(MatchCreator defaultMatcher, Map<ColumnDataType, MatchCreator> options, MatchCreator referenceMatcher) {
		this.matchers = new HashMap<ColumnDataType, MatchCreator>(options);

		this.defaultMatcher = defaultMatcher;
		this.referenceMatcher = referenceMatcher;
	}

	public MatcherOptionsBuilder withNewMatcher(MatchCreator matcher, ColumnDataType... matcherTypes) {

		return new MatcherOptionsBuilder(getDefaultMatcher(), getMatchers(), getReferenceMatcher(), matcher, matcherTypes);
	}

	public MatcherOptionsBuilder withNewReferenceMatcher(MatchCreator matcher) {

		return new MatcherOptionsBuilder(getDefaultMatcher(), getMatchers(), matcher);
	}

}
