package de.mannheim.uni.expansion.utils;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;

public class BatchConsumerProducer<E> {

	private final PersistentParallel3<E> pp;
	private final Consumer<E> body;

	final AtomicReference<CountDownLatch> latch = new AtomicReference<>();

	public BatchConsumerProducer(String name, Consumer<E> body, int subBatchSize, int subtractProcessors) {

		int numOfProcessors = Runtime.getRuntime().availableProcessors() - subBatchSize;
		pp = new PersistentParallel3<>(name, subBatchSize, numOfProcessors);
		this.body = body;

		latch.set(new CountDownLatch(1));
		// We can start right away, therefore we count down!
		latch.get().countDown();
	}

	public BatchConsumerProducer(String name, Consumer<E> body, int subBatchSize) {
		this(name, body, subBatchSize, 0);
	}


	public synchronized void addBatch(List<E> list) {

		try {
			latch.get().await();

			CountDownLatch newLatch = new CountDownLatch(1);
			latch.set(newLatch);

			AtomicReference<List<E>> listReference = new AtomicReference<List<E>>(list);
			list = null;

			Runnable r2 = () -> {

				PersistentParallel3.setCurrentThreadParallel(false);

				try {
					pp.foreach(listReference.get(), body, false);
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}

				listReference.set(null);

				newLatch.countDown();

			};

			Thread t = new Thread(r2);
			t.start();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void finish() {
		try {

			latch.get().await();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
