package de.mannheim.uni.expansion.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;

import org.apache.commons.lang3.time.DurationFormatUtils;

public class PersistentParallel3<T> {

	public static InheritableThreadLocal<Boolean> alreadyParallel = new InheritableThreadLocal<>();
	private static int defaultNumProcessors = Runtime.getRuntime().availableProcessors();
	private final static AtomicReference<PersistentParallel3<?>> currentTask = new AtomicReference<>(null);
	private static int overrideNumProcessors = 0;

	private final List<Thread> pool;
	private final AtomicReference<ConcurrentLinkedQueue<LinkedList<T>>> queues = new AtomicReference<>();
	private final int numOfThreads;

	private final String name;

	private final int batchSize;

	private boolean running = true;
	private Consumer<T> consumer;

	private AtomicReference<CountDownLatch> latch = new AtomicReference<>();
	private AtomicReference<CountDownLatch> mainLatch = new AtomicReference<>();
	private AtomicReference<AtomicInteger> queueLatch = new AtomicReference<>();
	private AtomicInteger queueSize = new AtomicInteger();
	private int totalTasks;
	private long start;
	private long lastTime;
	private long last;
	private final boolean block;

	private ThreadCounter threadCounter;

	public PersistentParallel3(String name) {
		this(name, -1, getNumProcessors());
	}

	public PersistentParallel3(String name, int batchSize) {
		this(name, batchSize, getNumProcessors());
	}

	public PersistentParallel3(String name, int batchSize, int numProcessors) {
		this(name, batchSize, getNumProcessors(), true);
	}

	public PersistentParallel3(String name, int batchSize, int numProcessors, boolean block) {
		this.block = block;

		this.batchSize = batchSize;

		this.name = name;

		mainLatch.set(new CountDownLatch(1));

		numOfThreads = numProcessors;

		latch.set(new CountDownLatch(numOfThreads));

		this.pool = new ArrayList<>();


		for (int i = 0; i < numOfThreads; i++) {

			Thread t = new Thread(() -> {
				setCurrentThreadParallel(true);
				try {
					CountDownLatch mainLatch = this.mainLatch.get();
					this.latch.get().countDown();
					mainLatch.await();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}




				while (running) {

					int queueSize = this.queueSize.get();
					ConcurrentLinkedQueue<LinkedList<T>> queues = this.queues.get();
					AtomicInteger queueLatch = this.queueLatch.get();

					Consumer<T> consumer = getConsumer();

					LinkedList<T> queue = getNextQueue(queues, queueLatch, queueSize);
					while (queue != null) {

						T item = queue.poll();
						while (item != null) {

							try {
								consumer.accept(item);
							} catch (Exception e1) {
								e1.printStackTrace();
								CountDownLatch latch = this.latch.get();
								latch.countDown();
								return;
							}
							if (threadCounter != null)
								threadCounter.inc();

							item = queue.poll();

						}

						queue = getNextQueue(queues, queueLatch, queueSize);
					}

					CountDownLatch mainLatch = this.mainLatch.get();

					CountDownLatch latch = this.latch.get();
					latch.countDown();

					try {
						mainLatch.await();
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

			} , name + " - " + i);

			t.start();

			pool.add(t);
		}

		try {
			this.latch.get().await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private LinkedList<T> getNextQueue(ConcurrentLinkedQueue<LinkedList<T>> queues, AtomicInteger queueLatch, int queueSize) {
		LinkedList<T> queue = queues.poll();
		if (queue != null) {
			queueLatch.incrementAndGet();
			return queue;
		} else {
			return null;
		}
	}

	private boolean printProgress() {
		
		long tasks = this.totalTasks;
		long done = threadCounter.get();


		long soFar = System.currentTimeMillis() - start;
		long pauseTime = System.currentTimeMillis() - lastTime;
		//long perItem = (long) (((float) soFar) / done);
		long left = (long) (((float) soFar / done) * (tasks - done));
		float itemsPerSecAvg = (float)done / (float)(soFar / 1000.0f);
		float itemsPerSecNow = (float)(done - last) / (pauseTime / 1000.0f);
		
		if((((float) soFar) / done)==Float.POSITIVE_INFINITY)
		{
			//perItem = -1;
			left = -1; 
		}
		String ttl = DurationFormatUtils.formatDuration(soFar, "HH:mm:ss.S");
		//String each = DurationFormatUtils.formatDuration(perItem, "HH:mm:ss.S");
		String remaining = DurationFormatUtils.formatDuration(left, "HH:mm:ss.S");
		
		System.err.println(String.format("%s: %,d of %,d tasks completed after %s (%d/%d active threads). Avg: %.2f items/s, Current: %.2f items/s, %s left.",
				name, done, tasks, ttl, latch.get().getCount(), this.numOfThreads, itemsPerSecAvg, itemsPerSecNow, remaining));

	
		
		
		last = done;
		lastTime = System.currentTimeMillis();

		if (done == tasks) {
			return false;
		} else {
			return true;
		}
//		368 of 1.370 tasks completed after 00:15:19.856 (8/8 active threads). Avg: 0,40 items/s, Current: 0,80 items/s, 00:41:44.607 left.
		
	}

	private Consumer<T> getConsumer() {
		return consumer;
	}

	private static int getNumProcessors() {
		if (overrideNumProcessors > 0)
			return overrideNumProcessors;
		else
			return defaultNumProcessors;
	}


	public void foreach(Collection<T> items, final Consumer<T> body) throws Exception {
		foreach(items, body, batchSize);
	}

	public void foreach(Collection<T> items, final Consumer<T> body, boolean reportProgress) throws Exception {
		foreach(items, body, batchSize, reportProgress);
	}

	public void foreach(Collection<T> items, final Consumer<T> body, int batchSize) throws Exception {
		foreach(items, body, batchSize, false);
	}

	public void foreach(Collection<T> items, final Consumer<T> body, int batchSize, boolean reportProgress) throws Exception {


		if (items.size() == 0)
			return;

		if (reportProgress) {
			this.totalTasks = 0;
			this.start = System.currentTimeMillis();
			this.threadCounter = new ThreadCounter();

			Runnable progressReporter = () -> {
				try {
					boolean stop = false;
					while (!stop) {

						Thread.sleep(10000);

						boolean cont = printProgress();
						if (!cont) {
							stop = true;
						}

					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			};

			Thread progressReporterThread = new Thread(progressReporter);
			progressReporterThread.start();
		}

		if (!block || (getNumProcessors() > 1 && !isCurrentThreadParallel())) {
			this.latch.get().await();

			int batchSizeForQueue;
			if (batchSize == -1) {
				batchSizeForQueue = (items.size() + numOfThreads - 1) / numOfThreads;
				batchSizeForQueue = (batchSizeForQueue + 9) / 10;
			}
			else {
				batchSizeForQueue = batchSize;
			}
			this.consumer = body;
			this.latch.set(new CountDownLatch(numOfThreads));

			ConcurrentLinkedQueue<LinkedList<T>> queues = new ConcurrentLinkedQueue<>();
			LinkedList<T> batch = new LinkedList<>();
			int i = 0;
			int size = 0;
			Iterator<T> it = items.iterator();
			while (it.hasNext()) {
				if (i == batchSizeForQueue) {
					i = 0;
					queues.add(batch);
					size++;
					batch = new LinkedList<>();
				}
				this.totalTasks++;
				batch.add(it.next());
				i++;
			}
			queues.add(batch);
			size++;
			
			items = null;
			this.queues.set(queues);
			this.queueSize.set(size);
			this.queueLatch.set(new AtomicInteger());

			if (queues.size() != queueSize.get()) {
				throw new IllegalStateException("QUEUES SIZE DOES NOT EQUAL QUEUE SIZE FOR STOP CONDITION" + "Actual size of Queue: " + queues.size() + ""
						+ "Size recorded for Stop Condition: " + queueSize.get());
			}
			CountDownLatch mainLatch = this.mainLatch.get();
			this.mainLatch.set(new CountDownLatch(1));
			mainLatch.countDown();

			this.latch.get().await();
		} else {
			sequentialFor(items, body);
		}

	}

	private void sequentialFor(Iterable<T> items, final Consumer<T> body) {

		for (T item : items) {
			body.accept(item);
		}

	}

	public static void setCurrentThreadParallel(boolean parallel) {
		alreadyParallel.set(parallel);
	}

	public static boolean isCurrentThreadParallel() {
		Boolean bool = PersistentParallel3.alreadyParallel.get();
		if (Boolean.TRUE.equals(bool))
			return true;
		else
			return false;

	}


	public synchronized void shutdown() {
		running = false;
		mainLatch.get().countDown();

	}

	public static <E> void runAndFinish(Collection<E> items, final Consumer<E> body, String name, int batchSize) {
		PersistentParallel3<E> pp = new PersistentParallel3<>(name, batchSize);
		try {
			pp.foreach(items, body);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		pp.shutdown();
	}

	public static void forLoop(int from, int to, Consumer<Integer> loopBody, String name, int batchSize) {
		List<Integer> lst = new LinkedList<Integer>();

		for (int i = from; i < to; i++)
			lst.add(i);

		PersistentParallel3.runAndFinish(lst, loopBody, name, batchSize);

	}

	public static <E> void runAndFinish(Collection<E> items, final Consumer<E> body, String name) {
		runAndFinish(items, body, name, -1);

	}
}
