package de.mannheim.uni.expansion.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;

import org.apache.commons.lang3.time.DurationFormatUtils;

public class PersistentParallel4 {


	private static int defaultNumProcessors = Runtime.getRuntime().availableProcessors();

	private static int overrideNumProcessors = 0;

	private final List<Thread> pool;
	private final AtomicReference<ConcurrentLinkedQueue<LinkedList<Object>>> queues = new AtomicReference<>();
	private final int numOfThreads;

	private final AtomicReference<String> name = new AtomicReference<String>();


	private boolean running = true;
	private Consumer<Object> consumer;

	private AtomicReference<CountDownLatch> latch = new AtomicReference<>();
	private AtomicReference<CountDownLatch> mainLatch = new AtomicReference<>();
	private AtomicReference<AtomicInteger> queueLatch = new AtomicReference<>();
	private AtomicInteger queueSize = new AtomicInteger();
	private int totalTasks;
	private long start;
	private long lastTime;
	private long last;

	private ThreadCounter threadCounter;

	// private PersistentParallel4() {
	// this(null, -1, getNumProcessors());
	// }
	//
	// private PersistentParallel4(String name) {
	// this(name, -1, getNumProcessors());
	// }
	//
	// private PersistentParallel4(String name, int batchSize) {
	// this(name, batchSize, getNumProcessors());
	// }
	//
	// private PersistentParallel4(String name, int batchSize, int
	// numProcessors) {
	// this(name, batchSize, getNumProcessors(), true);
	// }

	private PersistentParallel4() {


		mainLatch.set(new CountDownLatch(1));

		numOfThreads = getNumProcessors();

		latch.set(new CountDownLatch(numOfThreads));

		this.pool = new ArrayList<>();


		for (int i = 0; i < numOfThreads; i++) {
			final int threadNumber = i;
			Thread t = new Thread(() -> {
				PersistentParallel3.setCurrentThreadParallel(true);
				try {
					CountDownLatch mainLatch = this.mainLatch.get();
					this.latch.get().countDown();
					mainLatch.await();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}




				while (running) {
					Thread.currentThread().setName(this.name.get() + "_" + threadNumber);

					int queueSize = this.queueSize.get();
					ConcurrentLinkedQueue<LinkedList<Object>> queues = this.queues.get();
					AtomicInteger queueLatch = this.queueLatch.get();

					Consumer<Object> consumer = getConsumer();

					LinkedList<Object> queue = getNextQueue(queues, queueLatch, queueSize);
					while (queue != null) {

						Object item = queue.poll();
						while (item != null) {

							try {
								consumer.accept(item);
							} catch (Exception e1) {
								e1.printStackTrace();
								CountDownLatch latch = this.latch.get();
								latch.countDown();
								return;
							}
							if (threadCounter != null)
								threadCounter.inc();

							item = queue.poll();

						}

						queue = getNextQueue(queues, queueLatch, queueSize);
					}

					CountDownLatch mainLatch = this.mainLatch.get();

					CountDownLatch latch = this.latch.get();
					latch.countDown();

					try {
						mainLatch.await();
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

			});

			t.start();

			pool.add(t);
		}

		try {
			this.latch.get().await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private LinkedList<Object> getNextQueue(ConcurrentLinkedQueue<LinkedList<Object>> queues, AtomicInteger queueLatch, int queueSize) {
		LinkedList<Object> queue = queues.poll();
		if (queue != null) {
			queueLatch.incrementAndGet();
			return queue;
		} else {
			return null;
		}
	}

	private boolean printProgress() {
		
		long tasks = this.totalTasks;
		long done = threadCounter.get();


		long soFar = System.currentTimeMillis() - start;
		long pauseTime = System.currentTimeMillis() - lastTime;
		//long perItem = (long) (((float) soFar) / done);
		long left = (long) (((float) soFar / done) * (tasks - done));
		float itemsPerSecAvg = (float)done / (float)(soFar / 1000.0f);
		float itemsPerSecNow = (float)(done - last) / (pauseTime / 1000.0f);
		
		if((((float) soFar) / done)==Float.POSITIVE_INFINITY)
		{
			//perItem = -1;
			left = -1; 
		}
		String ttl = DurationFormatUtils.formatDuration(soFar, "HH:mm:ss.S");
		//String each = DurationFormatUtils.formatDuration(perItem, "HH:mm:ss.S");
		String remaining = DurationFormatUtils.formatDuration(left, "HH:mm:ss.S");
		
		System.err.println(String.format("%s: %,d of %,d tasks completed after %s (%d/%d active threads). Avg: %.2f items/s, Current: %.2f items/s, %s left.",
				name, done, tasks, ttl, latch.get().getCount(), this.numOfThreads, itemsPerSecAvg, itemsPerSecNow, remaining));

	
		
		
		last = done;
		lastTime = System.currentTimeMillis();

		if (done == tasks) {
			return false;
		} else {
			return true;
		}
//		368 of 1.370 tasks completed after 00:15:19.856 (8/8 active threads). Avg: 0,40 items/s, Current: 0,80 items/s, 00:41:44.607 left.
		
	}

	private Consumer<Object> getConsumer() {
		return consumer;
	}

	private static int getNumProcessors() {
		if (overrideNumProcessors > 0)
			return overrideNumProcessors;
		else
			return defaultNumProcessors;
	}




	@SuppressWarnings("unchecked")
	public <T extends Object> void foreach(Collection<T> items, final Consumer<T> body, String name, boolean block, int batchSize, boolean reportProgress)
			throws Exception {


		if (items.size() == 0)
			return;

		if (reportProgress) {
			this.totalTasks = 0;
			this.start = System.currentTimeMillis();
			this.threadCounter = new ThreadCounter();

			Runnable progressReporter = () -> {
				try {
					boolean stop = false;
					while (!stop) {

						Thread.sleep(10000);

						boolean cont = printProgress();
						if (!cont) {
							stop = true;
						}

					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			};

			Thread progressReporterThread = new Thread(progressReporter);
			progressReporterThread.start();
		}

		if (!block || (getNumProcessors() > 1 && !isCurrentThreadParallel())) {
			this.latch.get().await();

			int batchSizeForQueue;
			if (batchSize == -1) {
				batchSizeForQueue = (items.size() + numOfThreads - 1) / numOfThreads;
				batchSizeForQueue = (batchSizeForQueue + 9) / 10;
			}
			else {
				batchSizeForQueue = batchSize;
			}
			this.consumer = (o) -> {
				body.accept((T) o);
			};
			this.latch.set(new CountDownLatch(numOfThreads));

			ConcurrentLinkedQueue<LinkedList<Object>> queues = new ConcurrentLinkedQueue<>();
			LinkedList<Object> batch = new LinkedList<>();
			int i = 0;
			int size = 0;
			Iterator<T> it = items.iterator();
			while (it.hasNext()) {
				if (i == batchSizeForQueue) {
					i = 0;
					queues.add(batch);
					size++;
					batch = new LinkedList<>();
				}
				this.totalTasks++;
				batch.add(it.next());
				i++;
			}
			queues.add(batch);
			size++;
			
			items = null;
			this.queues.set(queues);
			this.queueSize.set(size);
			this.queueLatch.set(new AtomicInteger());

			if (queues.size() != queueSize.get()) {
				throw new IllegalStateException("QUEUES SIZE DOES NOT EQUAL QUEUE SIZE FOR STOP CONDITION" + "Actual size of Queue: " + queues.size() + ""
						+ "Size recorded for Stop Condition: " + queueSize.get());
			}
			CountDownLatch mainLatch = this.mainLatch.get();
			this.mainLatch.set(new CountDownLatch(1));

			this.name.set(name);
			mainLatch.countDown();

			this.latch.get().await();
		} else {
			sequentialFor(items, body);
		}

	}

	private <T> void sequentialFor(Iterable<T> items, final Consumer<T> body) {

		for (T item : items) {
			body.accept(item);
		}

	}

	public static void setCurrentThreadParallel(boolean parallel) {
		PersistentParallel3.setCurrentThreadParallel(parallel);
	}

	public static boolean isCurrentThreadParallel() {
		return PersistentParallel3.isCurrentThreadParallel();
	}


	// public synchronized void shutdown() {
	// running = false;
	// mainLatch.get().countDown();
	//
	// }

	private static PersistentParallel4 pp = new PersistentParallel4();

	public static <E> void runAndFinish(Collection<E> items, final Consumer<E> body, String name, boolean block, int batchSize, boolean reportProgress) {

		try {
			pp.foreach(items, body, name, block, batchSize, reportProgress);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void forLoop(int from, int to, Consumer<Integer> loopBody, String name, int batchSize) {
		List<Integer> lst = new LinkedList<Integer>();

		for (int i = from; i < to; i++)
			lst.add(i);

		PersistentParallel4.runAndFinish(lst, loopBody, name, true, batchSize, false);

	}

	public static <E> void runAndFinish(Collection<E> items, final Consumer<E> body, String name) {
		runAndFinish(items, body, name, true, -1, false);

	}
}
