package de.mannheim.uni.matching.dbpedia.algorithm;

import de.mannheim.uni.matching.Matcher;
import java.util.ArrayList;
import java.util.List;

import de.mannheim.uni.matching.dbpedia.matchers.instance.PruningInstanceMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.schema.DuplicateBasedSchemaMatcher;
import de.mannheim.uni.matching.dbpedia.model.MatchingResult;
import de.mannheim.uni.matching.dbpedia.model.TableRow;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableColumnMatchingAdapter;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableColumnTableLabelAdapter;
import de.mannheim.uni.matching.evaluation.EvaluationResult;
import de.mannheim.uni.matching.process.Configuration;
import de.mannheim.uni.matching.process.Parameter;
import de.mannheim.uni.similarity.matrix.SimilarityMatrix;
import de.mannheim.uni.similarity.matrix.SparseSimilarityMatrixFactory;
import de.mannheim.uni.tableprocessor.model.TableColumn;

public class IterativeComponent extends WebtableToDBpediaMatchingComponent {

    private EvaluationResult propertyResult;

    public EvaluationResult getPropertyResult() {
        return propertyResult;
    }
    private EvaluationResult instanceResult;

    public EvaluationResult getInstanceResult() {
        return instanceResult;
    }
    private MatchingResult result;

    public MatchingResult getResult() {
        return result;
    }

    private void setResult(MatchingResult result) {
        this.result = result;
    }

    public double evaluate(Configuration config) {
        super.evaluate(config);
        return (getPropertyResult().getF1Score() + getInstanceResult().getF1Score()) / 2.0;
    }
    private SimilarityMatrix<TableColumn> propertySimilarity;

    public SimilarityMatrix<TableColumn> getPropertySimilarity() {
        return propertySimilarity;
    }
    private SimilarityMatrix<TableRow> instanceSimilarity;

    public SimilarityMatrix<TableRow> getInstanceSimilarity() {
        return instanceSimilarity;
    }
    public static final Parameter PAR_PROP_NUM_CANDIDATES = PropertyBasedClassRefinementComponent.PAR_PROP_NUM_CANDIDATES;
    public static final Parameter PAR_PROP_CANDIDATE_THRESHOLD = PropertyBasedClassRefinementComponent.PAR_PROP_CANDIDATE_THRESHOLD;
    public static final Parameter PAR_PROP_VALUE_THRESHOLD = PropertyBasedClassRefinementComponent.PAR_PROP_VALUE_THRESHOLD;
    public static final Parameter PAR_PROP_NUM_VOTES = PropertyBasedClassRefinementComponent.PAR_PROP_NUM_VOTES;
    public static final Parameter PAR_PROP_NUM_RESULTS = PropertyBasedClassRefinementComponent.PAR_PROP_NUM_RESULTS;
    public static final Parameter PAR_PROP_FINAL_THRESHOLD = PropertyBasedClassRefinementComponent.PAR_PROP_FINAL_THRESHOLD;
    public static final Parameter PAR_PROP_KEY_WEIGHT = PropertyBasedClassRefinementComponent.PAR_PROP_KEY_WEIGHT;
    public static final Parameter PAR_INST_FINAL_THRESHOLD = new Parameter("Iterative.Instance.Final_threshold", 0.8);
    public static final Parameter PAR_PROP_LABELS_WEIGHT = new Parameter("Iterative.Property.Labels_weight", 0.0);
    public static final Parameter PAR_PROP_EQUIVALENCES_WEIGHT = new Parameter("Iterative.Property.Equivalences_weight", 0.0);
    public static final Parameter PAR_PROP_VALUES_WEIGHT = new Parameter("Iterative.Property.Values_weight", 1.0);
    private static List<Parameter> params;

    public static List<Parameter> getParams() {
        return params;
    }

    static {
        params = new ArrayList<>();
        params.add(PAR_PROP_NUM_CANDIDATES);
        params.add(PAR_PROP_CANDIDATE_THRESHOLD);
        params.add(PAR_PROP_VALUE_THRESHOLD);
        params.add(PAR_PROP_NUM_VOTES);
        params.add(PAR_PROP_NUM_RESULTS);
        params.add(PAR_PROP_FINAL_THRESHOLD);
        params.add(PAR_INST_FINAL_THRESHOLD);
    }

    public IterativeComponent() {
        setParameters(params);
    }

    protected void initialiseParameters(Configuration config) {
        DuplicateBasedSchemaMatcher prop = getMatchers().getPropertyMatcher();

        prop.setNumCandidatesPerInstance((Integer) config.getValue(PAR_PROP_NUM_CANDIDATES));
        prop.setCandidateThreshold((Double) config.getValue(PAR_PROP_CANDIDATE_THRESHOLD));
        prop.setValueThreshold((Double) config.getValue(PAR_PROP_VALUE_THRESHOLD));
        prop.setNumVotesPerInstance((Integer) config.getValue(PAR_PROP_NUM_VOTES));
        prop.setNumResults((Integer) config.getValue(PAR_PROP_NUM_RESULTS));
        prop.setFinalThreshold((Double) config.getValue(PAR_PROP_FINAL_THRESHOLD));
        prop.setKeyWeight((Integer) config.getValue(PAR_PROP_KEY_WEIGHT));

        PruningInstanceMatcher inst = getMatchers().getInstanceMatcher();
        inst.setSimilarityThreshold((Double) config.getValue(PAR_INST_FINAL_THRESHOLD));
    }

    @Override
    public void run(Configuration config) {
        initialiseParameters(config);

        System.out.println("Iterative Component");

        MatchingResult logResult = new MatchingResult();

        propertySimilarity = getSimilarities().getPropertySimilarity();

        if (propertySimilarity == null || propertySimilarity.getMaxValue() == null || propertySimilarity.getMaxValue() == 0.0) {

            instanceSimilarity = getSimilarities().getCandidateSimilarity();
            mapInstances(instanceSimilarity, getMatchingParameters().isCollectMatchingInfo(), logResult, "");

            logResult.getEvaluation().getInstanceResult();
            setResult(logResult);
            return;
        }

        // Candidate (Instance) Matching
        getMatchers().getInstanceMatcher().setInitialCandidateSimilarity(getSimilarities().getCandidateSimilarity());
        getMatchers().getInstanceMatcher().setPropertySimilarity(propertySimilarity);
        getMatchers().getInstanceMatcher().setValueSimilarity(getSimilarities().getValueSimilarity());
        instanceSimilarity = getMatchers().getInstanceMatcher().match(getData());
        mapInstances(instanceSimilarity, getMatchingParameters().isCollectMatchingInfo(), logResult, "");
        //addIntermediaResult("instances", String.format("[0%d] iteration %d", iterationNum+4,iterationNum), logResult.getEvaluation().getInstanceResult());
        instanceResult = logResult.getEvaluation().getInstanceResult();
        setResult(logResult);

        // Property Matching
        getMatchers().getPropertyMatcher().setCandidateSimilarity(instanceSimilarity);
        getMatchers().getPropertyMatcher().setClassSimilarity(getSimilarities().getClassSimilarity());
        getMatchers().getPropertyMatcher().setFinalClass(getSimilarities().getFinalClass());
        getMatchers().getPropertyMatcher().setLabelSimilarity(getSimilarities().getLabelSimilarity());
        getMatchers().getPropertyMatcher().setValueSimilarity(getSimilarities().getValueSimilarity());
        propertySimilarity = getMatchers().getPropertyMatcher().match(getData());

//
//        double wLbl = (double) config.getValue(PAR_PROP_LABELS_WEIGHT);
//        double wEqiv = (double) config.getValue(PAR_PROP_EQUIVALENCES_WEIGHT);
//        double wProp = (double) config.getValue(PAR_PROP_VALUES_WEIGHT);
//        double sum = wLbl + wEqiv + wProp;
//        wLbl /= sum;
//        wEqiv /= sum;
//        wProp /= sum;
//        SimilarityMatrix<TableColumn> combinedValues = new SparseSimilarityMatrixFactory().createSimilarityMatrix(getSimilarities().getLabelSimilarity().getFirstDimension().size(), getSimilarities().getLabelSimilarity().getSecondDimension().size());
////        for (TableColumn c: getSimilarities().getLabelSimilarity().getFirstDimension()) {
////            for (TableColumn d : getSimilarities().getLabelSimilarity().getSecondDimension()) {
//                
////                if (valueBased.get(c, d) == null) {
////                    combinedValues.set(c, d, getSimilarities().getLabelSimilarity().get(c, d) * wLbl);
////                } else {
//                    if(wLbl>0.0 || wEqiv>0.0) {
//                        valueBased.multiplyScalar(wProp);
//                    }
//                    if (wLbl > 0.0) {
//                        SimilarityMatrix<TableColumn> lbl = getSimilarities().getLabelSimilarity().copy();
//                        lbl.multiplyScalar(wLbl);
//                        combinedValues = Matcher.addSimilarity(valueBased, lbl);
////                    }
//
//                }
////            }
////        }
//        propertySimilarity = combinedValues.copy();

        System.out.println(
                "final: " + propertySimilarity.getOutput2(null, null, new TableColumnMatchingAdapter(), new TableColumnTableLabelAdapter()));
        mapProperties(propertySimilarity, getWebTableName(), getMatchingParameters().isCollectMatchingInfo(), logResult);
        //addIntermediaResult("properties", String.format("[0%d] iteration %d", iterationNum+4,iterationNum), logResult.getEvaluation().getPropertyResult());
        propertyResult = logResult.getEvaluation().getPropertyResult();
    }
}
