package de.mannheim.uni.matching.dbpedia.algorithm;

import de.mannheim.uni.matching.dbpedia.logging.MatchingLogger;
import de.mannheim.uni.matching.dbpedia.matchers.instance.CandidateRefinementMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.instance.CandidateSelectionMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.instance.LinkBasedMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.instance.PruningInstanceMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.instance.ValueBasedInstanceMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.schema.DuplicateBasedSchemaMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.schema.LabelBasedSchemaMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.schema.PropertyBasedClassMatcher;
import de.mannheim.uni.matching.dbpedia.matchers.schema.SecondLineClassMatcher;
import de.mannheim.uni.matching.dbpedia.model.GoldStandard;
import de.mannheim.uni.matching.dbpedia.model.MatchingParameters;
import de.mannheim.uni.matching.dbpedia.model.Similarities;
import de.mannheim.uni.utils.timer.Timer;

/**
 * Contains all partial matchers that are used by the algorithm
 * @author Oliver
 *
 */
public class Matchers {

    private ValueBasedInstanceMatcher valueBasedInstanceMatcher;
    public ValueBasedInstanceMatcher getValueBasedInstanceMatcher() {
        return valueBasedInstanceMatcher;
    }
    public void setValueBasedInstanceMatcher(
            ValueBasedInstanceMatcher valueBasedInstanceMatcher) {
        this.valueBasedInstanceMatcher = valueBasedInstanceMatcher;
    }
    
    private LabelBasedSchemaMatcher labelBasedSchemaMatcher;
    public LabelBasedSchemaMatcher getLabelBasedSchemaMatcher() {
        return labelBasedSchemaMatcher;
    }
    public void setLabelBasedSchemaMatcher(
            LabelBasedSchemaMatcher labelBasedSchemaMatcher) {
        this.labelBasedSchemaMatcher = labelBasedSchemaMatcher;
    }
    
    private CandidateSelectionMatcher candidateSelectionMatcher;
    public CandidateSelectionMatcher getCandidateSelectionMatcher() {
        return candidateSelectionMatcher;
    }
    public void setCandidateSelectionMatcher(
            CandidateSelectionMatcher candidateSelectionMatcher) {
        this.candidateSelectionMatcher = candidateSelectionMatcher;
    }
    
    private CandidateRefinementMatcher candidateRefinementMatcher;
    public CandidateRefinementMatcher getCandidateRefinementMatcher() {
        return candidateRefinementMatcher;
    }
    public void setCandidateRefinementMatcher(
            CandidateRefinementMatcher candidateRefinementMatcher) {
        this.candidateRefinementMatcher = candidateRefinementMatcher;
    }
    
    private DuplicateBasedSchemaMatcher propertyMatcher;
    public DuplicateBasedSchemaMatcher getPropertyMatcher() {
        return propertyMatcher;
    }
    public void setPropertyMatcher(DuplicateBasedSchemaMatcher propertyMatcher) {
        this.propertyMatcher = propertyMatcher;
    }
    
    private PruningInstanceMatcher instanceMatcher;
    public PruningInstanceMatcher getInstanceMatcher() {
        return instanceMatcher;
    }
    public void setInstanceMatcher(PruningInstanceMatcher instanceMatcher) {
        this.instanceMatcher = instanceMatcher;
    }
    
    private SecondLineClassMatcher classMatcher;
    public SecondLineClassMatcher getClassMatcher() {
        return classMatcher;
    }
    public void setClassMatcher(SecondLineClassMatcher classMatcher) {
        this.classMatcher = classMatcher;
    }
    
    private PropertyBasedClassMatcher propertyBasedClassMatcher;
    public PropertyBasedClassMatcher getPropertyBasedClassMatcher() {
        return propertyBasedClassMatcher;
    }
    public void setPropertyBasedClassMatcher(
            PropertyBasedClassMatcher propertyBasedClassMatcher) {
        this.propertyBasedClassMatcher = propertyBasedClassMatcher;
    }
    
    private LinkBasedMatcher linkBasedMatcher;
    
    public Matchers(Similarities sim, MatchingParameters par, Timer t, GoldStandard g, MatchingLogger logger) {
        valueBasedInstanceMatcher = new ValueBasedInstanceMatcher(sim, par, t, g, logger);
        labelBasedSchemaMatcher = new LabelBasedSchemaMatcher(sim, par, t, g, logger);
        candidateSelectionMatcher = new CandidateSelectionMatcher(sim, par, t, g, logger);
        candidateRefinementMatcher = new CandidateRefinementMatcher(sim, par, t, g, logger);
        propertyMatcher = new DuplicateBasedSchemaMatcher(sim, par, t, g, logger);
        instanceMatcher = new PruningInstanceMatcher(sim, par, t, g, logger);
        classMatcher = new SecondLineClassMatcher(sim, par, t, g, logger);
        propertyBasedClassMatcher = new PropertyBasedClassMatcher(sim, par, t, g, logger);
        linkBasedMatcher = new LinkBasedMatcher(sim, par, t, g, logger);
    }

    /**
     * @return the linkBasedMatcher
     */
    public LinkBasedMatcher getLinkBasedMatcher() {
        return linkBasedMatcher;
    }

    /**
     * @param linkBasedMatcher the linkBasedMatcher to set
     */
    public void setLinkBasedMatcher(LinkBasedMatcher linkBasedMatcher) {
        this.linkBasedMatcher = linkBasedMatcher;
    }
}
